Package: urca
Version: 1.3-4
Date: 2024-05-25
Title: Unit Root and Cointegration Tests for Time Series Data
Authors@R: c(person("Bernhard", "Pfaff", email = "bernhard@pfaffikus.de", role = c("aut", "cre")), person("Eric", "Zivot",email = "ezivot@u.washington.edu", role = "ctb"), person("Matthieu", "Stigler", role = "ctb"))
Depends: R (>= 2.0.0), methods
Imports: nlme, graphics, stats
LazyLoad: yes
Description: Unit root and cointegration tests encountered in applied 
 econometric analysis are implemented.
License: GPL (>= 2)
NeedsCompilation: yes
Packaged: 2024-05-25 16:28:23 UTC; bp
Author: Bernhard Pfaff [aut, cre],
  Eric Zivot [ctb],
  Matthieu Stigler [ctb]
Maintainer: Bernhard Pfaff <bernhard@pfaffikus.de>
Repository: CRAN
Date/Publication: 2024-05-27 12:20:03 UTC
Built: R 4.6.0; aarch64-apple-darwin23; 2026-04-26 02:19:51 UTC; unix
Archs: urca.so.dSYM

##
import(methods)
import(nlme)

##
useDynLib(urca, .registration = TRUE)

## Functions
export(ablrtest, alphaols, alrtest, blrtest, bh5lrtest, bh6lrtest, ca.jo, ca.po,
       cajolst, cajools, cajorls, lttest, plotres, punitroot, qunitroot, unitrootTable,
       ur.ers, ur.kpss, ur.pp, ur.df, ur.sp, ur.za)

## Classes       
exportClasses("urca", "ca.jo", "cajo.test", "ur.kpss", "ca.po", "ur.ers", "ur.pp", "ur.sp", "ur.df", "ur.za", "sumurca")

## Methods
exportMethods("show", "plot", "summary")

## Import functions
importFrom("graphics", "abline", "layout", "legend", "lines", "mtext",
           "par", "plot")
importFrom("stats", "AIC", "acf", "anova", "BIC", "coef", "embed", "formula",
           "lm", "na.omit", "pacf", "pchisq", "plot.ts", "resid", "residuals")

			###############
			# R E A D M E #
			###############

The two files in the `/Rcmdr' directory of urca, namely `Rcmdr-urca.R' 
and `Rcmdr-menus.txt', can be utilisied as an add-in to `Rcmdr' of
John Fox. For more information on `Rcmdr' please visit the URL:

	http://socserv.socsci.mcmaster.ca/jfox/Misc/Rcmdr/

as well as the man pages of the package itself. Incidentally, a
tutorial `Getting-Started-with-the-Rcmdr.pdf' is shipped in the
`/doc' subdirectory of `Rcmdr'.

In order to install the `urca' add-in to `Rcmdr' two steps are
necessary:

1. step:
********
Copy the `Rcmdr-urca.R' into the `/etc' sub-directory of the package
`Rcmdr'; that is: `Rcmdr/etc' should now contain this file.

2. step:
********
Copy the contents of the file `Rcmdr-menus.txt' in the `urca/inst'
directory into the file `Rcmdr/etc/Rcmdr-menus.txt' at a suitable
place. For example, you can place the contents right before the 
definition of the `Help-menu' (look for a line, similar to:

	menu  helpMenu  topMenu   ""  ""  ""

Then, the newly created `urca-menu' will be placed left to the
`Help-menu' of `Rcmdr' once you executed: library(Rcmdr) from a 
R Console.

summary(ur.df(Canada[, "prod"],
              type = "trend", lags = 2))
summary(ur.df(diff(Canada[, "prod"]),
              type = "drift", lags = 1))
summary(ur.df(Canada[, "e"],
              type = "trend", lags = 2))
summary(ur.df(diff(Canada[, "e"]),
              type = "drift", lags = 1))
summary(ur.df(Canada[, "U"],
              type = "drift", lags = 1))
summary(ur.df(diff(Canada[, "U"]),
              type = "none", lags = 0))
summary(ur.df(Canada[, "rw"],
              type = "trend", lags = 4))
summary(ur.df(diff(Canada[, "rw"]),
              type = "drift", lags = 3))
summary(ur.df(diff(Canada[, "rw"]),
              type = "drift", lags = 0))

## Impulse response analysis
irf.y1 <- irf(varsimest, impulse = "y1",
              response = "y2", n.ahead = 10,
              ortho = FALSE, cumulative = FALSE,
              boot = FALSE, seed = 12345)
args(vars:::plot.varirf)
plot(irf.y1)
irf.y2 <- irf(varsimest, impulse = "y2",
              response = "y1", n.ahead = 10,
              ortho = TRUE, cumulative = TRUE,
              boot = FALSE, seed = 12345)
plot(irf.y2)

library(urca)
library(uroot)
data(UKconinc)
incl <- ts(UKconinc$incl, start = c(1955,1),
           end = c(1984,4), frequency = 4)
HEGY000 <- HEGY.test(wts = incl, itsd = c(0, 0, c(0)),
                     selectlags = list(mode = c(1,4,5)))
HEGY100 <- HEGY.test(wts = incl, itsd = c(1, 0, c(0)),
                     selectlags = list(mode = c(1,4,5)))
HEGY110 <- HEGY.test(wts = incl, itsd = c(1, 1, c(0)),
                     selectlags = list(mode = c(1,4,5)))
HEGY101 <- HEGY.test(wts = incl,
                     itsd = c(1, 0, c(1, 2, 3)),
                     selectlags = list(mode = c(1,4,5)))
HEGY111 <- HEGY.test(wts = incl,
                     itsd = c(1, 1, c(1, 2, 3)),
                     selectlags = list(mode = c(1,4,5)))

## Forecasting objects of class varest
args(vars:::predict.varest)
predictions <- predict(varsimest, n.ahead = 25,
                       ci = 0.95)
class(predictions)
args(vars:::plot.varprd)
## Plot of predictions for y1
plot(predictions, names = "y1")
## Fanchart for y2
args(fanchart)
fanchart(predictions, names = "y2")

library(urca)
#
# ADF-test
#
urcadf <- function(){
  if (!checkActiveDataSet()) return()
  if (!checkNumeric(n=1)) return()
  initializeDialog(title="ADF Test")
  xBox <- variableListBox(top, Numeric(), title="Variable (pick one)")
  onOK <- function(){
    x <- getSelection(xBox)
    var <- paste(.activeDataSet, "$", x, sep="")
    ttype <- tclvalue(testtypeVariable)
    lags <- tclvalue(lagsVariable)
    if (length(x) == 0){
      errorCondition(recall=urcadf, message="You must select a variable.")
      return()
      }
    if (GrabFocus()) tkgrab.release(top)
    tkdestroy(top)
    doItAndPrint(paste("summary(ur.df(y= ", var, ", type = ", ttype, ", lags = ",  lags, "))", sep=""))
    tkdestroy(top)
    tkfocus(CommanderWindow())
  }
  OKCancelHelp(helpSubject="ur.df")
  radioButtons(name="testtype", buttons=c("none", "drift", "trend"), values=c("'none'", "'drift'", "'trend'" ), labels=c("no deterministic regressor", "drift only", "drift and trend"), title="Type of test")
  rightFrame <- tkframe(top)
  lagsFrame <- tkframe(rightFrame)
  lagsVariable <- tclVar("4")
  lagsField <- tkentry(lagsFrame, width="2", textvariable=lagsVariable)
  tkgrid(getFrame(xBox), sticky="nw")
  tkgrid(tklabel(rightFrame, text=""))
  tkgrid(tklabel(lagsFrame, text="Maximum number of lags = ", fg="blue"), lagsField, sticky="w")
  tkgrid(lagsFrame, sticky="w")
  tkgrid(testtypeFrame, rightFrame, sticky="nw")
  tkgrid(buttonsFrame, columnspan=2, sticky="w")
  tkgrid.configure(lagsField, sticky="e")
  dialogSuffix(rows=4, columns=2)
}
#
# ERS-test
#
urcaers <- function(){
  if (!checkActiveDataSet()) return()
  if (!checkNumeric(n=1)) return()
  initializeDialog(title="ERS Test")
  xBox <- variableListBox(top, Numeric(), title="Variable (pick one)")
  onOK <- function(){
    x <- getSelection(xBox)
    var <- paste(.activeDataSet, "$", x, sep="")
    ttype <- tclvalue(testtypeVariable)
    modtype <- tclvalue(modelVariable)
    lags <- tclvalue(lagsVariable)
    if (length(x) == 0){
      errorCondition(recall=urcaers, message="You must select a variable.")
      return()
      }
    if (GrabFocus()) tkgrab.release(top)
    tkdestroy(top)
    doItAndPrint(paste("summary(ur.ers(y= ", var, ", type = ", ttype, ", model = ", modtype, ", lag.max = ",  lags, "))", sep=""))
    tkdestroy(top)
    tkfocus(CommanderWindow())
  }
  OKCancelHelp(helpSubject="ur.ers")
  radioButtons(name="testtype", buttons=c("DFGLS", "Ptest"), values=c("'DF-GLS'", "'P-test'"), 
               labels=c("DF-GLS statistic", "P-test statistic"), title="Type of test")
  radioButtons(name="model", buttons=c("const", "trend"), values=c("'constant'", "'trend'"), 
               labels=c("Include constant", "Include constant + trend"), title="Model type")
  rightFrame <- tkframe(top)
  lagsFrame <- tkframe(rightFrame)
  lagsVariable <- tclVar("4")
  lagsField <- tkentry(lagsFrame, width="2", textvariable=lagsVariable)
  tkgrid(getFrame(xBox), sticky="nw")
  tkgrid(tklabel(rightFrame, text=""))
  tkgrid(tklabel(lagsFrame, text="Maximum number of lags = ", fg="blue"), lagsField, sticky="w")
  tkgrid(lagsFrame, sticky="w")
  tkgrid(testtypeFrame, rightFrame, sticky="nw")
  tkgrid(modelFrame, rightFrame, sticky="nw")
  tkgrid(buttonsFrame, columnspan=2, sticky="w")
  tkgrid.configure(lagsField, sticky="e")
  dialogSuffix(rows=4, columns=2)
}
#
# KPSS-test
#
urcakpss <- function(){
  if (!checkActiveDataSet()) return()
  if (!checkNumeric(n=1)) return()
  initializeDialog(title="KPSS Test")
  xBox <- variableListBox(top, Numeric(), title="Variable (pick one)")
  radioButtons(name="testtype", buttons=c("constant", "trend"), 
        values=c("'mu'", "'tau'"), initialValue="'mu'", 
        labels=c("Include constant", "Include trend"), title="Deterministic Part")
  radioButtons(name="lags", buttons=c("short", "long", "nil"), 
        values=c("'short'", "'long'", "'nil'"), initialValue="'short'",         labels=c("use short lags", "use long lags", "use no lags"), title="Lag Selection")
  onOK <- function(){
    x <- getSelection(xBox)
    if (length(x) == 0){
      errorCondition(recall=urcakpss, message="You must select a variable.")
      return()
    }
    var <- paste(.activeDataSet, "$", x, sep="")
    ttype <- tclvalue(testtypeVariable)
    ltype <- tclvalue(lagsVariable)
    if (GrabFocus()) tkgrab.release(top)
    tkdestroy(top)
    command <- paste("summary(ur.kpss(", var, ", type = ", ttype, ", lags = ", ltype, "))", sep="")
    doItAndPrint(command)
    tkfocus(CommanderWindow())
  }
  OKCancelHelp(helpSubject="ur.kpss")
  tkgrid(getFrame(xBox), sticky="nw")
  tkgrid(testtypeFrame, lagsFrame,  sticky="w")
  tkgrid(buttonsFrame, columnspan=2, sticky="w")
  dialogSuffix(rows=3, columns=2)
}
#
# Schmidt-Phillips test
#
urcasp <- function(){
  if (!checkActiveDataSet()) return()
  if (!checkNumeric(n=1)) return()
  initializeDialog(title="SP Test")
  xBox <- variableListBox(top, Numeric(), title="Variable (pick one)")
  onOK <- function(){
    x <- getSelection(xBox)
    var <- paste(.activeDataSet, "$", x, sep="")
    ttype <- tclvalue(testtypeVariable)
    pdtype <- tclvalue(poldegVariable)
    sltype <- tclvalue(signifVariable)
    if (length(x) == 0){
      errorCondition(recall=urcasp, message="You must select a variable.")
      return()
      }
    if (GrabFocus()) tkgrab.release(top)
    tkdestroy(top)
    doItAndPrint(paste("summary(ur.sp(y= ", var, ", type = ", ttype, ", pol.deg = ", pdtype, ", signif = ",  sltype, "))", sep=""))
    tkdestroy(top)
    tkfocus(CommanderWindow())
  }
  OKCancelHelp(helpSubject="ur.sp")
  radioButtons(name="testtype", buttons=c("tau", "rho"), values=c("'tau'", "'rho'"), 
               labels=c("tau statistic", "rho statistic"), title="Type of test")
  radioButtons(name="poldeg", buttons=c("pb1", "pb2", "pb3", "pb4"), values=c("1", "2", "3", "4"), 
               labels=c("1st", "2nd", "3rd", "4th"), title="Select pol. degree")
  radioButtons(name="signif", buttons=c("sb1", "sb5", "sb10"), values=c("0.01", "0.05", "0.1"), 
               labels=c("alpha=1%", "alpha=5%", "alpha=10%"), title="Sig. Level")
  tkgrid(getFrame(xBox), testtypeFrame, sticky="nw")
  tkgrid(poldegFrame, signifFrame, sticky="nw")
  tkgrid(buttonsFrame, columnspan=2, sticky="w")
  dialogSuffix(rows=4, columns=2)
}
#
# Phillips-Perron test
#
urcapp <- function(){
  if (!checkActiveDataSet()) return()
  if (!checkNumeric(n=1)) return()
  initializeDialog(title="PP Test")
  xBox <- variableListBox(top, Numeric(), title="Variable (pick one)")
  onOK <- function(){
    x <- getSelection(xBox)
    var <- paste(.activeDataSet, "$", x, sep="")
    ttype <- tclvalue(testtypeVariable)
    modtype <- tclvalue(modelVariable)
    lagtype <- tclvalue(lagsVariable)
    if (length(x) == 0){
      errorCondition(recall=urcapp, message="You must select a variable.")
      return()
      }
    if (GrabFocus()) tkgrab.release(top)
    tkdestroy(top)
    doItAndPrint(paste("summary(ur.pp(x= ", var, ", type = ", ttype, ", model = ", modtype, ", lags = ",  lagtype, "))", sep=""))
    tkdestroy(top)
    tkfocus(CommanderWindow())
  }
  OKCancelHelp(helpSubject="ur.pp")
  radioButtons(name="testtype", buttons=c("Zalpha", "Ztau"), values=c("'Z-alpha'", "'Z-tau'"), 
               labels=c("Z-alpha statistic", "Z-tau statistic"), title="Type of test")
  radioButtons(name="model", buttons=c("const", "trend"), values=c("'constant'", "'trend'"), 
               labels=c("Include constant", "include constant + trend"), title="Model type")
  radioButtons(name="lags", buttons=c("short", "long"), values=c("'short'", "'long'"), 
               labels=c("short lags", "long lags"), title="Lags for error correction")
  tkgrid(getFrame(xBox), testtypeFrame, sticky="nw")
  tkgrid(modelFrame, lagsFrame, sticky="nw")
  tkgrid(buttonsFrame, columnspan=2, sticky="w")
  dialogSuffix(rows=4, columns=2)
}
#
# Zivot-Andrews test
#
urcaza <- function(){
  if (!checkActiveDataSet()) return()
  if (!checkNumeric(n=1)) return()
  initializeDialog(title="Zivot & Andrews Test")
  xBox <- variableListBox(top, Numeric(), title="Variable (pick one)")
  onOK <- function(){
    x <- getSelection(xBox)
    var <- paste(.activeDataSet, "$", x, sep="")
    modtype <- tclvalue(modelVariable)
    lags <- tclvalue(lagsVariable)
    ptype <- if(tclvalue(plotVariable) == 0) "FALSE" else "TRUE"
    if (length(x) == 0){
      errorCondition(recall=urcaza, message="You must select a variable.")
      return()
      }
    if (GrabFocus()) tkgrab.release(top)
    tkdestroy(top)
    command <- paste("ur.za(y= ", var, ",  model = ", modtype, ", lag = ",  lags, ")", sep="")
    logger(paste("ZAstat <- ", command, sep=""))
    assign("ZAstat", justDoIt(command), envir=.GlobalEnv)
    doItAndPrint("summary(ZAstat)")
    if(ptype==TRUE) {
      justDoIt("x11()")
      justDoIt("plot(ZAstat)")
    }
    logger("remove(ZAstat)") 
    remove(ZAstat, envir=.GlobalEnv)       
    tkdestroy(top)
    tkfocus(CommanderWindow())
  }
  OKCancelHelp(helpSubject="ur.za")
  radioButtons(name="model", buttons=c("const", "trend", "both"), values=c("'intercept'", "'trend'", "'both'"), 
               labels=c("Include constant", "Include trend", "Include both"), title="Model type")
  checkBoxes(frame="plotFrame", boxes="plot", initialValues="0", labels="Plot path of Zivot & Andrews Statistic?")
  rightFrame <- tkframe(top)
  lagsFrame <- tkframe(rightFrame)
  lagsVariable <- tclVar("4")
  lagsField <- tkentry(lagsFrame, width="2", textvariable=lagsVariable)
  tkgrid(getFrame(xBox), sticky="nw")
  tkgrid(tklabel(rightFrame, text=""))
  tkgrid(tklabel(lagsFrame, text="Maximum number of lags = ", fg="blue"), lagsField, sticky="w")
  tkgrid(lagsFrame, sticky="w")
  tkgrid(modelFrame, rightFrame, sticky="nw")
  tkgrid(plotFrame, sticky="nw")
  tkgrid(buttonsFrame, columnspan=2, sticky="w")
  tkgrid.configure(lagsField, sticky="e")
  dialogSuffix(rows=3, columns=2)
}
#
# Phillips-Ouliaris test
#
urcacapo <- function(){
  dataSets <- listDataSets()
  if (length(dataSets) == 0){
    tkmessageBox(message="There are no data sets from which to choose.", icon="error", type="ok")
    tkfocus(CommanderWindow())
    return()
  }
  initializeDialog(title="Phillips & Ouliaris Test")
  xBox <- variableListBox(top, dataSets, title="Data Sets (pick one)")
  onOK <- function(){
    x <- getSelection(xBox)
    if (length(x) == 0){
      errorCondition(recall=urcacapo, message="You must select a data set.")
      return()
    }
    meantype <- tclvalue(demeanVariable)
    lags <- tclvalue(lagsVariable)
    ttype <- tclvalue(typeVariable)
    tolvar <- tclvalue(tolVariable)
    if (GrabFocus()) tkgrab.release(top)
    tkdestroy(top)
    doItAndPrint(paste("summary(ca.po(z = ", x, ",  demean = ", meantype, ", lag = ",  lags, ", type = ", ttype, ", tol = ", tolvar, "))", sep=""))
    tkdestroy(top)
    tkfocus(CommanderWindow())
  }
  OKCancelHelp(helpSubject="ca.po")
  radioButtons(name="demean", buttons=c("none", "constant", "trend"), values=c("'none'", "'constant'", "'trend'"), 
               labels=c("None", "Include constant", "Include trend"), title="Demean?")
  radioButtons(name="lags", buttons=c("short", "long"), values=c("'short'", "'long'"), 
               labels=c("short lags", "long lags"), title="Lags for error correction")
  radioButtons(name="type", buttons=c("Pu", "Pz"), values=c("'Pu'", "'Pz'"), 
               labels=c("Pu statistic", "Pz statistic"), title="Type of test")
  rightFrame <- tkframe(top)
  tolFrame <- tkframe(rightFrame)
  tolVariable <- tclVar("NULL")
  tolField <- tkentry(tolFrame, width="8", textvariable=tolVariable)
  tkgrid(getFrame(xBox), sticky="nw")
  tkgrid(tklabel(rightFrame, text=""))
  tkgrid(tklabel(tolFrame, text="Tolerance level =  ", fg="blue"), tolField, sticky="w")
  tkgrid(tolFrame, sticky="w")
  tkgrid(demeanFrame, rightFrame, sticky="nw")
  tkgrid(lagsFrame, typeFrame, sticky="nw")
  tkgrid(buttonsFrame, columnspan=2, sticky="w")
  tkgrid.configure(tolField, sticky="e")
  dialogSuffix(rows=5, columns=2)
}
#
# Johansen-Procedure
#
urcacajo <- function(){
  dataSets <- listDataSets()
  if (length(dataSets) == 0){
    tkmessageBox(message="There are no data sets from which to choose.", icon="error", type="ok")
    tkfocus(CommanderWindow())
    return()
  }
  initializeDialog(title="Johansen's Procedure")
  xBox <- variableListBox(top, dataSets, title="Data Sets (pick one)")
  assign("UpdateModelNumber", UpdateModelNumber() + 1, envir=.GlobalEnv)
  modelName <- tclVar(paste("VECMmodel.", UpdateModelNumber, sep=""))
  modelFrame <- tkframe(top)
  model <- tkentry(modelFrame, width="20", textvariable=modelName)
  onOK <- function(){
    x <- getSelection(xBox)
    if (length(x) == 0){
      errorCondition(recall=urcacajo, message="You must select a data set.")
      return()
    }
    ttype <- tclvalue(typeVariable)
    spect <- tclvalue(specVariable)
    ctype <- tclvalue(ecdetVariable)
    lags <- tclvalue(lagVariable)
    seas <- tclvalue(seasonVariable)
    dummy <- tclvalue(dumVariable)
    if (GrabFocus()) tkgrab.release(top)
    tkdestroy(top)
    modelValue <- tclvalue(modelName)
    if (!is.valid.name(modelValue)){
      assign("UpdateModelNumber", UpdateModelNumber() - 1, envir=.GlobalEnv)
      errorCondition(recall=urcacajo, message=paste('"', modelValue, '" is not a valid name.', sep=""))
      return()
    }
    if (is.element(modelValue, listVECMmodels())) {
      if ("no" == tclvalue(checkReplace(modelValue, type="Model"))){
        assign("UpdateModelNumber", UpdateModelNumber() - 1, envir=.GlobalEnv)
        if (GrabFocus()) tkgrab.release(top)
        tkdestroy(top)
        urcacajo()
        return()
      }
    }
    command <- paste("ca.jo(x = ", x, ",  type = ", ttype, ", ecdet = ",  ctype, ", K = ",
                     lags, ", spec = ", spect, ", season = ", seas, ", dumvar = ", dummy , ")", sep="")
    logger(paste(modelValue, " <- ", command, sep=""))
    assign(modelValue, justDoIt(command), envir=.GlobalEnv)
    doItAndPrint(paste("summary(", modelValue, ")", sep=""))
    tkfocus(CommanderWindow())
  }
  OKCancelHelp(helpSubject="ca.jo")
  radioButtons(name="type", buttons=c("eigen", "trace"), values=c("'eigen'", "'trace'"), 
               labels=c("Eigenvalue statistic", "Trace statistic"), title="Type of statistic")
  radioButtons(name="spec", buttons=c("long", "trans"), values=c("'longrun'", "'transitory'"), 
               labels=c("longrun specification", "transitory specification"), title="VECM specification")
  radioButtons(name="season", buttons=c("none", "monthly", "quarterly"), values=c("NULL", "12", "4"), 
               labels=c("None", "Monthly seasonality", "Quarterly seasonality"), title="Seasonality")
  radioButtons(name="ecdet", buttons=c("none", "const", "trend"), values=c("'none'", "'const'", "'trend'"), 
               labels=c("none", "constant", "trend"), title="Deterministic Variable in Cointegration") 
  rightFrame <- tkframe(top)
  lagFrame <- tkframe(rightFrame)
  lagVariable <- tclVar("2")
  lagField <- tkentry(lagFrame, width="2", textvariable=lagVariable)
  dumFrame <- tkframe(rightFrame)
  dumVariable <- tclVar("NULL")
  dumField <- tkentry(dumFrame, width="8", textvariable=dumVariable)
  tkgrid(tklabel(modelFrame, text="Enter name for model:"), model, sticky="w")
  tkgrid(modelFrame, sticky="w")
  tkgrid(getFrame(xBox), sticky="nw")
  tkgrid(tklabel(rightFrame, text=""), sticky="w")
  tkgrid(tklabel(lagFrame, text="Lag order =  ", fg="blue"), lagField, sticky="w")
  tkgrid(lagFrame, sticky="w")
  tkgrid(tklabel(dumFrame, text="Matrix of dummy variables =  ", fg="blue"), dumField, sticky="w")
  tkgrid(dumFrame, sticky="w")
  tkgrid(typeFrame, rightFrame, sticky="nw")
  tkgrid(specFrame, seasonFrame, sticky="nw")
  tkgrid(ecdetFrame, rightFrame, sticky="w")
  tkgrid(buttonsFrame, sticky="w")
  tkgrid.configure(lagField, sticky="e")
  tkgrid.configure(dumField, sticky="e")
  dialogSuffix(rows=8, columns=2)
}
#
# Linear Trend test
#
urcalttest <- function(){
  VECMmodels <- listVECMmodels()
  if (length(VECMmodels) == 0){
    tkmessageBox(message="There are no VECM models from which to choose.", icon="error", type="ok")
    tkfocus(CommanderWindow())
    return()
  }
  initializeDialog(title="Linear Trend Test")
  xBox <- variableListBox(top, VECMmodels, title="VECM models (pick one)")
  onOK <- function(){
    x <- getSelection(xBox)
    if (length(x) == 0){
      errorCondition(recall=urcalttest, message="You must select a VECM model.")
      return()
    }
    rint <- tclvalue(rankVariable)
    if (GrabFocus()) tkgrab.release(top)
    tkdestroy(top)
    doItAndPrint(paste("lttest(z = ", x, ", r = ", rint, ")", sep=""))
    tkfocus(CommanderWindow())
  }
  OKCancelHelp(helpSubject="lttest")
  rankFrame <- tkframe(top)
  rankVariable <- tclVar("1")
  rankField <- tkentry(rankFrame, width="2", textvariable=rankVariable)
  tkgrid(getFrame(xBox), sticky="nw")
  tkgrid(rankFrame, sticky="w")
  tkgrid(tklabel(rankFrame, text="Number of cointegrating relationships =  ", fg="blue"), rankField, sticky="w")
  tkgrid(buttonsFrame, sticky="w")
  tkgrid.configure(rankField, sticky="e")
  dialogSuffix(rows=3, columns=2)
}
#
# Restrictions on Loading matrix
#
urcaalrtest <- function(){
  VECMmodels <- listVECMmodels()
  matrices <- listMatrix()
  if (length(VECMmodels) == 0){
    tkmessageBox(message="There are no VECM models from which to choose.", icon="error", type="ok")
    tkfocus(CommanderWindow())
    return()
  }
  if (length(matrices) == 0){
    tkmessageBox(message="There are no restriction matrices defined from which to choose.", icon="error", type="ok")
    tkfocus(CommanderWindow())
    return()
  }
  initializeDialog(title="Test Restrictions on Loading Vectors")
  xBox <- variableListBox(top, VECMmodels, title="VECM models (pick one)")
  yBox <- variableListBox(top, matrices, title="Restriction matrices (pick one)")
  onOK <- function(){
    x <- getSelection(xBox)
    if (length(x) == 0){
      errorCondition(recall=urcaalrtest, message="You must select a VECM model.")
      return()
    }
    y <- getSelection(yBox)
    if (length(y) == 0){
      errorCondition(recall=urcaalrtest, message="You must select a restriction matrix.")
      return()
    }
    rint <- tclvalue(rankVariable)
    if (GrabFocus()) tkgrab.release(top)
    tkdestroy(top)
    doItAndPrint(paste("summary(alrtest(z = ", x, ", A = ", y , ", r = ", rint, "))", sep=""))
    tkfocus(CommanderWindow())
  }
  OKCancelHelp(helpSubject="alrtest")
  rankFrame <- tkframe(top)
  rankVariable <- tclVar("1")
  rankField <- tkentry(rankFrame, width="2", textvariable=rankVariable)
  tkgrid(getFrame(xBox), getFrame(yBox), sticky="nw")
  tkgrid(rankFrame, sticky="w")
  tkgrid(tklabel(rankFrame, text="Number of cointegrating relationships =  ", fg="blue"), rankField, sticky="w")
  tkgrid(buttonsFrame, sticky="w")
  tkgrid.configure(rankField, sticky="e")
  dialogSuffix(rows=3, columns=1)
}
#
# Restrictions on cointegration matrix
#
urcablrtest <- function(){
  VECMmodels <- listVECMmodels()
  matrices <- listMatrix()
  if (length(VECMmodels) == 0){
    tkmessageBox(message="There are no VECM models from which to choose.", icon="error", type="ok")
    tkfocus(CommanderWindow())
    return()
  }
  if (length(matrices) == 0){
    tkmessageBox(message="There are no restriction matrices defined from which to choose.", icon="error", type="ok")
    tkfocus(CommanderWindow())
    return()
  }
  initializeDialog(title="Test Restrictions on Cointegration Vectors")
  xBox <- variableListBox(top, VECMmodels, title="VECM models (pick one)")
  yBox <- variableListBox(top, matrices, title="Restriction matrices (pick one)")
  onOK <- function(){
    x <- getSelection(xBox)
    if (length(x) == 0){
      errorCondition(recall=urcaablrtest, message="You must select a VECM model.")
      return()
    }
    y <- getSelection(yBox)
    if (length(y) == 0){
      errorCondition(recall=urcaalrtest, message="You must select a restriction matrix.")
      return()
    }
    rint <- tclvalue(rankVariable)
    if (GrabFocus()) tkgrab.release(top)
    tkdestroy(top)
    doItAndPrint(paste("summary(blrtest(z = ", x, ", H = ", y , ", r = ", rint, "))", sep=""))
    tkfocus(CommanderWindow())
  }
  OKCancelHelp(helpSubject="blrtest")
  rankFrame <- tkframe(top)
  rankVariable <- tclVar("1")
  rankField <- tkentry(rankFrame, width="2", textvariable=rankVariable)
  tkgrid(getFrame(xBox), getFrame(yBox), sticky="nw")
  tkgrid(rankFrame, sticky="w")
  tkgrid(tklabel(rankFrame, text="Number of cointegrating relationships =  ", fg="blue"), rankField, sticky="w")
  tkgrid(buttonsFrame, sticky="w")
  tkgrid.configure(rankField, sticky="e")
  dialogSuffix(rows=3, columns=1)
}
#
# Restrictions for partly known cointegrating vectors
#
urcabh5lrtest <- function(){
  VECMmodels <- listVECMmodels()
  matrices <- listMatrix()
  numerics <- listNumeric()
  elements <- c(matrices, numerics)
  if (length(VECMmodels) == 0){
    tkmessageBox(message="There are no VECM models from which to choose.", icon="error", type="ok")
    tkfocus(CommanderWindow())
    return()
  }
  if (length(matrices) == 0){
    tkmessageBox(message="There are no restriction matrices defined from which to choose.", icon="error", type="ok")
    tkfocus(CommanderWindow())
    return()
  }
  initializeDialog(title="Test validity of partly known cointegrating Vectors")
  xBox <- variableListBox(top, VECMmodels, title="VECM models (pick one)")
  yBox <- variableListBox(top, elements, title="Restriction matrices (pick one)")
  onOK <- function(){
    x <- getSelection(xBox)
    if (length(x) == 0){
      errorCondition(recall=urcabh5lrtest, message="You must select a VECM model.")
      return()
    }
    y <- getSelection(yBox)
    if (length(y) == 0){
      errorCondition(recall=urcabh5lrtest, message="You must select a restriction matrix.")
      return()
    }
    rint <- tclvalue(rankVariable)
    if (GrabFocus()) tkgrab.release(top)
    tkdestroy(top)
    doItAndPrint(paste("summary(bh5lrtest(z = ", x, ", H = ", y , ", r = ", rint, "))", sep=""))
    tkfocus(CommanderWindow())
  }
  OKCancelHelp(helpSubject="bh5lrtest")
  rankFrame <- tkframe(top)
  rankVariable <- tclVar("2")
  rankField <- tkentry(rankFrame, width="2", textvariable=rankVariable)
  tkgrid(getFrame(xBox), getFrame(yBox), sticky="nw")
  tkgrid(rankFrame, sticky="w")
  tkgrid(tklabel(rankFrame, text="Number of cointegrating relationships =  ", fg="blue"), rankField, sticky="w")
  tkgrid(buttonsFrame, sticky="w")
  tkgrid.configure(rankField, sticky="e")
  dialogSuffix(rows=3, columns=1)
}
#
# Restrictions for partly known cointegrating vectors
#
urcabh6lrtest <- function(){
  VECMmodels <- listVECMmodels()
  matrices <- listMatrix()
  numerics <- listNumeric()
  elements <- c(matrices, numerics)
  if (length(VECMmodels) == 0){
    tkmessageBox(message="There are no VECM models from which to choose.", icon="error", type="ok")
    tkfocus(CommanderWindow())
    return()
  }
  if (length(matrices) == 0){
    tkmessageBox(message="There are no restriction matrices defined from which to choose.", icon="error", type="ok")
    tkfocus(CommanderWindow())
    return()
  }
  initializeDialog(title="Test restrictions of partly known cointegrating Vectors")
  xBox <- variableListBox(top, VECMmodels, title="VECM models (pick one)")
  yBox <- variableListBox(top, elements, title="Restriction matrices (pick one)")
  onOK <- function(){
    x <- getSelection(xBox)
    if (length(x) == 0){
      errorCondition(recall=urcabh6lrtest, message="You must select a VECM model.")
      return()
    }
    y <- getSelection(yBox)
    if (length(y) == 0){
      errorCondition(recall=urcaalrtest, message="You must select a restriction matrix.")
      return()
    }
    rint <- tclvalue(rankVariable)
    r1int <- tclvalue(r1Variable)
    maxiter <- tclvalue(maxiterVariable)
    conv <- tclvalue(conVariable)
    if (GrabFocus()) tkgrab.release(top)
    tkdestroy(top)
    doItAndPrint(paste("summary(bh6lrtest(z = ", x, ", H = ", y , ", r = ", rint, ", r1 =", r1int, ", conv.val =", conv, ", max.iter =", maxiter, "))", sep=""))
    tkfocus(CommanderWindow())
  }
  OKCancelHelp(helpSubject="bh6lrtest")
  rankFrame <- tkframe(top)
  r1Frame <- tkframe(top)
  rankVariable <- tclVar("2")
  r1Variable <- tclVar("1")
  rightFrame <- tkframe(top)
  maxiterFrame <- tkframe(rightFrame)
  maxiterVariable <- tclVar("50")
  maxiterField <- tkentry(maxiterFrame, width="4", textvariable=maxiterVariable)
  conFrame <- tkframe(rightFrame)
  conVariable <- tclVar("0.0001")
  conField <- tkentry(conFrame, width="8", textvariable=conVariable)
  tkgrid(tklabel(rightFrame, text=""))
  rankField <- tkentry(rankFrame, width="2", textvariable=rankVariable)
  r1Field <- tkentry(r1Frame, width="2", textvariable=r1Variable)
  tkgrid(getFrame(xBox), getFrame(yBox), sticky="nw")
  tkgrid(rankFrame, rightFrame, sticky="w")
  tkgrid(tklabel(rankFrame, text="Number of cointegrating relationships =  ", fg="blue"), rankField, sticky="w")
  tkgrid(r1Frame, rightFrame, sticky="w")
  tkgrid(tklabel(r1Frame, text="Number of restricted ci relationships =  ", fg="blue"), r1Field, sticky="w")
  tkgrid(maxiterFrame, sticky="w")
  tkgrid(tklabel(maxiterFrame, text="Maximum number of iterations = ", fg="blue"), maxiterField, sticky="w")
  tkgrid(conFrame, sticky="w")
  tkgrid(tklabel(conFrame, text="Convergence criteria = ", fg="blue"), conField, sticky="w")
  tkgrid(buttonsFrame, sticky="w")
  tkgrid.configure(rankField, sticky="e")
  tkgrid.configure(r1Field, sticky="e")
  tkgrid.configure(maxiterField, sticky="e")
  tkgrid.configure(conField, sticky="e")
  dialogSuffix(rows=4, columns=2)
}
#
# Restrictions for loading and cointegration matrix
#
urcaablrtest <- function(){
  VECMmodels <- listVECMmodels()
  matrices <- listMatrix()
  if (length(VECMmodels) == 0){
    tkmessageBox(message="There are no VECM models from which to choose.", icon="error", type="ok")
    tkfocus(CommanderWindow())
    return()
  }
  if (length(matrices) == 0){
    tkmessageBox(message="There are no restriction matrices defined from which to choose.", icon="error", type="ok")
    tkfocus(CommanderWindow())
    return()
  }
  initializeDialog(title="Test Restrictions on Loading & Cointegration vectors")
  xBox <- variableListBox(top, VECMmodels, title="VECM models (pick one)")
  yBox <- variableListBox(top, matrices, title="Restriction matrices for cointegration vectors (pick one)")
  lBox <- variableListBox(top, matrices, title="Restriction matrices for loading vectors (pick one)")
  onOK <- function(){
    x <- getSelection(xBox)
    if (length(x) == 0){
      errorCondition(recall=urcaablrtest, message="You must select a VECM model.")
      return()
    }
    y <- getSelection(yBox)
    if (length(y) == 0){
      errorCondition(recall=urcaablrtest, message="You must select a cointegration restriction matrix.")
      return()
    }
    l <- getSelection(lBox)
    if (length(l) == 0){
      errorCondition(recall=urcaablrtest, message="You must select a loading restriction matrix.")
      return()
    }
    rint <- tclvalue(rankVariable)
    if (GrabFocus()) tkgrab.release(top)
    tkdestroy(top)
    doItAndPrint(paste("summary(ablrtest(z = ", x, ", H = ", y, ", A = ", l,  ", r = ", rint, "))", sep=""))
    tkfocus(CommanderWindow())
  }
  OKCancelHelp(helpSubject="ablrtest")
  rankFrame <- tkframe(top)
  rankVariable <- tclVar("1")
  rankField <- tkentry(rankFrame, width="2", textvariable=rankVariable)
  tkgrid(getFrame(xBox), sticky="nw")
  tkgrid(getFrame(yBox), getFrame(lBox), sticky="nw")
  tkgrid(rankFrame, sticky="w")
  tkgrid(tklabel(rankFrame, text="Number of cointegrating relationships =  ", fg="blue"), rankField, sticky="w")
  tkgrid(buttonsFrame, sticky="w")
  tkgrid.configure(rankField, sticky="e")
  dialogSuffix(rows=3, columns=2)
}
#
# VECM with a structural shift
#
urcacajolst <- function(){
  dataSets <- listDataSets()
  if (length(dataSets) == 0){
    tkmessageBox(message="There are no data sets from which to choose.", icon="error", type="ok")
    tkfocus(CommanderWindow())
    return()
  }
  initializeDialog(title="VECM with level shift")
  xBox <- variableListBox(top, dataSets, title="Data Sets (pick one)")
  assign("UpdateModelNumber", UpdateModelNumber + 1, envir=.GlobalEnv)
  modelName <- tclVar(paste("VECMmodel.", UpdateModelNumber, sep=""))
  modelFrame <- tkframe(top)
  model <- tkentry(modelFrame, width="20", textvariable=modelName)
  onOK <- function(){
    x <- getSelection(xBox)
    if (length(x) == 0){
      errorCondition(recall=urcacajo, message="You must select a data set.")
      return()
    }
    ttype <- if(tclvalue(trendVariable) == 0) "FALSE" else "TRUE"
    lags <- tclvalue(lagVariable)
    seas <- tclvalue(seasonVariable)
    if (GrabFocus()) tkgrab.release(top)
    tkdestroy(top)
    modelValue <- tclvalue(modelName)
    if (!is.valid.name(modelValue)){
      assign("UpdateModelNumber", UpdateModelNumber - 1, envir=.GlobalEnv)
      errorCondition(recall=urcacajolst, message=paste('"', modelValue, '" is not a valid name.', sep=""))
      return()
    }
    if (is.element(modelValue, listVECMmodels())) {
      if ("no" == tclvalue(checkReplace(modelValue, type="Model"))){
        assign("UpdateModelNumber", UpdateModelNumber - 1, envir=.GlobalEnv)
        if (GrabFocus()) tkgrab.release(top)
        tkdestroy(top)
        urcacajolst()
        return()
      }
    }
    command <- paste("cajolst(x = ", x, ",  trend = ", ttype, ", K = ", lags, ", season = ", seas, ")", sep="")
    logger(paste(modelValue, " <- ", command, sep=""))
    assign(modelValue, justDoIt(command), envir=.GlobalEnv)
    doItAndPrint(paste("summary(", modelValue, ")", sep=""))
    tkfocus(CommanderWindow())
  }
  OKCancelHelp(helpSubject="cajolst")
  radioButtons(name="season", buttons=c("none", "monthly", "quarterly"), values=c("NULL", "12", "4"), 
               labels=c("None", "Monthly seasonality", "Quarterly seasonality"), title="Seasonality")
  checkBoxes(frame="trendFrame", boxes="trend", initialValues="1", labels="Include linear trend in the auxiliary regressions?")
  lagFrame <- tkframe(top)
  lagVariable <- tclVar("2")
  lagField <- tkentry(lagFrame, width="2", textvariable=lagVariable)
  tkgrid(tklabel(modelFrame, text="Enter name for model:"), model, sticky="w")
  tkgrid(modelFrame, sticky="w")
  tkgrid(getFrame(xBox), sticky="nw")
  tkgrid(tklabel(lagFrame, text="Lag order =  ", fg="blue"), lagField, sticky="w")
  tkgrid(lagFrame, sticky="w")
  tkgrid(trendFrame, sticky="w")
  tkgrid(buttonsFrame, sticky="w")
  tkgrid.configure(lagField, sticky="e")
  dialogSuffix(rows=5, columns=1)
}
#
# Utility Functions
#
listVECMmodels <- function(envir=.GlobalEnv, ...) {
  objects <- ls(envir=envir, ...)
  if (length(objects) == 0) NULL
  else objects[sapply(objects, function(.x) "ca.jo" == (class(eval(parse(text=.x), envir=envir))[1]))]
}

listMatrix <- function(envir=.GlobalEnv, ...) {
  objects <- ls(envir=envir, ...)
  if (length(objects) == 0) NULL
  else objects[sapply(objects, function(.x) "matrix" == (class(eval(parse(text=.x), envir=envir))[1]))]
}

listNumeric <- function(envir=.GlobalEnv, ...) {
  objects <- ls(envir=envir, ...)
  if (length(objects) == 0) NULL
  else objects[sapply(objects, function(.x) "numeric" == (class(eval(parse(text=.x), envir=envir))[1]))]
}

LR[3, 3] <- 0
LR
svec.oi <- SVEC(vecm, LR = LR, SR = SR, r = 1,
                lrtest = TRUE, boot = FALSE)
svec.oi <- update(svec, LR = LR, lrtest = TRUE,
                  boot = FALSE)
svec.oi$LRover

set.seed(123456)
e <- rnorm(500)
## trend
trd <- 1:500
S <- c(rep(0, 249), rep(1, 251))
## random walk with drift
y1 <- 0.1*trd + cumsum(e)
## random walk with drift and shift
y2 <- 0.1*trd + 10*S + cumsum(e)

set.seed(123456)
y <- arima.sim(n = 100, list(ar = 0.9), innov=rnorm(100))
op <- par(no.readonly=TRUE)
layout(matrix(c(1, 1, 2, 3), 2, 2, byrow=TRUE))
plot.ts(y, ylab='')
acf(y, main='Autocorrelations', ylab='',
    ylim=c(-1, 1), ci.col = "black")
pacf(y, main='Partial Autocorrelations', ylab='',
     ylim=c(-1, 1), ci.col = "black")
par(op)

library(dse1)
library(vars)
## A-model
Apoly   <- array(c(1.0, -0.5, 0.3, 0.8,
                   0.2, 0.1, -0.7, -0.2,
                   0.7, 1, 0.5, -0.3) ,
                 c(3, 2, 2))
## Setting covariance to identity-matrix
B <- diag(2)
## Generating the VAR(2) model 
svarA  <- ARMA(A = Apoly, B = B)
## Simulating 500 observations
svarsim <- simulate(svarA, sampleT = 500,
                    rng = list(seed = c(123456)))
## Obtaining the generated series
svardat <- matrix(svarsim$output, nrow = 500, ncol = 2)
colnames(svardat) <- c("y1", "y2")
## Estimating the VAR
varest <- VAR(svardat, p = 2, type = "none")
## Setting up matrices for A-model
Amat <- diag(2)
Amat[2, 1] <- NA
Amat[1, 2] <- NA
## Estimating the SVAR A-type by direct maximisation
## of the log-likelihood
args(SVAR)
svar.A <- SVAR(varest, estmethod = "direct",
               Amat = Amat, hessian = TRUE)

data(denmark)
sjd <- denmark[, c("LRM", "LRY", "IBO", "IDE")]
sjd.vecm <- summary(ca.jo(sjd, ecdet = "const",
                          type = "eigen",
                          K = 2,
                          spec = "longrun",
                          season = 4))
lue.vecm <- summary(cajolst(sjd, season=4))

library(dse1)
library(vars)
## B-model
Apoly   <- array(c(1.0, -0.5, 0.3, 0,
                   0.2, 0.1, 0, -0.2,
                   0.7, 1, 0.5, -0.3) ,
                 c(3, 2, 2))
## Setting covariance to identity-matrix
B <- diag(2)
B[2, 1] <- -0.8
## Generating the VAR(2) model 
svarB  <- ARMA(A = Apoly, B = B)
## Simulating 500 observations
svarsim <- simulate(svarB, sampleT = 500,
                    rng = list(seed = c(123456)))
svardat <- matrix(svarsim$output, nrow = 500, ncol = 2)
colnames(svardat) <- c("y1", "y2")
varest <- VAR(svardat, p = 2, type = "none")
## Estimating the SVAR B-type by scoring algorithm
## Setting up the restriction matrix and vector
## for B-model
Bmat <- diag(2)
Bmat[2, 1] <- NA
svar.B <- SVAR(varest, estmethod = "scoring",
               Bmat = Bmat, max.iter = 200)

library(fracdiff)
set.seed(123456)
# ARFIMA(0.4,0.4,0.0)
y1 <- fracdiff.sim(n=1000, ar=0.4, ma=0.0, d=0.4)
# ARIMA(0.4,0.0,0.0)
y2 <- arima.sim(model=list(ar=0.4), n=1000)
# Graphics
op <- par(no.readonly=TRUE)
layout(matrix(1:6, 3, 2, byrow=FALSE))
plot.ts(y1$series,
        main='Time series plot of long memory',
        ylab='')
acf(y1$series, lag.max=100,
    main='Autocorrelations of long memory')
spectrum(y1$series,
         main='Spectral density of long memory')
plot.ts(y2,
        main='Time series plot of short memory', ylab='')
acf(y2, lag.max=100,
    main='Autocorrelations of short memory')
spectrum(y2, main='Spectral density of short memory')
par(op)

library(urca)
data(nporg)
gnp <- log(na.omit(nporg[, "gnp.r"]))
gnp.d <- diff(gnp)
gnp.ct.df <- ur.ers(gnp, type = "DF-GLS",
                    model = "trend", lag.max = 4)
gnp.ct.pt <- ur.ers(gnp, type = "P-test",
                    model = "trend")
gnp.d.ct.df <- ur.ers(gnp.d, type = "DF-GLS",
                      model = "trend", lag.max = 4)
gnp.d.ct.pt <- ur.ers(gnp.d, type = "P-test",
                      model = "trend")

vecm <- ca.jo(Canada[, c("rw", "prod", "e", "U")],
              type = "trace", ecdet = "trend",
              K = 3, spec = "transitory")
vecm.r1 <- cajorls(vecm, r = 1)
alpha <- coef(vecm.r1$rlm)[1, ]
beta <- vecm.r1$beta
resids <- resid(vecm.r1$rlm)
N <- nrow(resids)
sigma <- crossprod(resids) / N
## t-stats for alpha 
alpha.se <- sqrt(solve(crossprod(
                 cbind(vecm@ZK %*% beta, vecm@Z1)))
                 [1, 1]* diag(sigma))
alpha.t <- alpha / alpha.se
## t-stats for beta
beta.se <- sqrt(diag(kronecker(solve(
                crossprod(vecm@RK[, -1])),
                solve(t(alpha) %*% solve(sigma)
                %*% alpha))))
beta.t <- c(NA, beta[-1] / beta.se)

c Copyright (c) James G. MacKinnon, 1996 (corrected 2001-1-8)
c
c urcrouts.f: This is a set of subroutines to estimate critical values
c and P values for unit root and cointegration tests. It is written in
c Fortran 77. Simply call urcval, specifying the first seven arguments.
c The result comes back in the last argument.
c
c A standalone program called urcdist.f is also available.
c
c These routines and the associated data files may be used freely for
c non-commercial purposes, provided that proper attribution is made.
c Please cite the paper
c
c  James G. MacKinnon, "Numerical distribution functions for unit root
c  and cointegration tests," Journal of Applied Econometrics, 11,
c  1996, 601-618.
c
c The routines and datas may not be incorporated into any book
c or computer program without the express, written consent of the author.
c
c The routines must have access to the files probs.tab and .urc-#.tab
c for # = 1, 2, 3 ..., 12. As currently written, these files must be
c in the current directory or in the directory /usr/local/urcdist. Make
c sure that these files are in the proper format for your computer (i.e.
c that lines are terminated by CR/LF for DOS, Windows, and OS/2 systems
c and by LF alone for Unix systems).
c

c Author authorisation to use under GPL license:

c From - Sat Nov 15 11:56:30 2008
c X-Mozilla-Status: 0001
c X-Mozilla-Status2: 00000000
c Delivered-To: matthieu.stigler@gmail.com
c Received: by 10.100.110.6 with SMTP id i6cs31074anc;
c        Fri, 14 Nov 2008 18:53:21 -0800 (PST)
c Received: by 10.65.23.5 with SMTP id a5mr1662842qbj.30.1226717600830;
c        Fri, 14 Nov 2008 18:53:20 -0800 (PST)
c Return-Path: <jgm@jgm.econ.queensu.ca>
c Received: from jgm.econ.queensu.ca (JGM.econ.QueensU.CA [130.15.74.85])
c        by mx.google.com with ESMTP id p9si2368721qbp.15.2008.11.14.18.53.20;
c        Fri, 14 Nov 2008 18:53:20 -0800 (PST)
c Received-SPF: pass (google.com: best guess record for domain of jgm@jgm.econ.queensu.ca designates 130.15.74.85 as permitted sender) client-ip=130.15.74.85;
c Authentication-Results: mx.google.com; spf=pass (google.com: best guess record for domain of jgm@jgm.econ.queensu.ca designates 130.15.74.85 as permitted sender) smtp.mail=jgm@jgm.econ.queensu.ca
c Received: from jgm by jgm.econ.queensu.ca with local (Exim 4.63)
c	(envelope-from <jgm@jgm.econ.queensu.ca>)
c	id 1L1BHk-00018U-3o
c	for matthieu.stigler@gmail.com; Fri, 14 Nov 2008 21:53:20 -0500
c Date: Fri, 14 Nov 2008 21:53:20 -0500 (EST)
c From: James MacKinnon <jgm@econ.queensu.ca>
c To: Matthieu Stigler <matthieu.stigler@gmail.com>
c Subject: Re: Program for surface response function
c In-Reply-To: <491D8DF3.20907@gmail.com>
c Message-ID: <Pine.LNX.4.62.0811142148490.4293@jgm.econ.queensu.ca>
c References: <491D8DF3.20907@gmail.com>
c MIME-Version: 1.0
c Content-Type: MULTIPART/MIXED; BOUNDARY="260199754-1936399293-1226717600=:4293"
c
c  This message is in MIME format.  The first part should be readable text,
c  while the remaining parts are likely unreadable without MIME-aware tools.
c
c --260199754-1936399293-1226717600=:4293
c Content-Type: TEXT/PLAIN; charset=ISO-8859-1; format=flowed
c Content-Transfer-Encoding: QUOTED-PRINTABLE
c
c On Fri, 14 Nov 2008, Matthieu Stigler wrote:
c
c > mat@cunix:~/Repertoires/urcdist$ f77  -O urcrouts.f -o urcrout
c > /usr/lib/gcc/i486-linux-gnu/3.4.6/../../../../lib/libfrtbegin.a(frtbegin.o):
c > In function `main':
c > (.text+0x35): undefined reference to `MAIN__'
c > collect2: ld a retourn=E9 1 code d'=E9tat d'ex=E9cution
c
c The problem is simply that urcrouts.f contains no main program. Either add
c a main program that calls one or more routines from urcrouts.f (you will
c need this anyway), or compile with the -c switch (instead of -o urcrout).
c This will create urcrouts.o, but it won't be any use without a main
c program.
c
c > Furthermore, I contributed some functions for the R program and would be
c > interested to try to integrate your functions for the package devoted to
c > unit root tests (urca). Do you agree that your code will be used in a R
c > package under the GPL license?
c
c That would be fine with me. But I would want to make sure the R code works
c properly and provides appropriate citations before you make it available.
c
c Cheers,
c
c James G. MacKinnon
c Head, Department of Economics
c Queen's University
c Kingston, Ontario, Canada
c K7L 3N6
c
c Email: jgm@econ.queensu.ca
c Phone: 613 533-2293
c Fax:   613 533-6668
c --260199754-1936399293-1226717600=:4293--

C ******************************************************************************


      subroutine fcrit(probs, cnorm, beta, wght, cval, size,
     &  precrt, nobs, model, nreg, np, nx)

      implicit double precision (a-h,o-z)
c
c Copyright (c) James G. MacKinnon, 1995
c Routine to find a critical value for any specified test size.
c Uses GLS to estimate approximating regression.
c
      double precision probs(221), cnorm(221), beta(4,221), crits(221),
     &  wght(221)
      double precision yvect(20),xmat(20,4),xomx(4,4),resid(20),gamma(4)
      double precision omega(20,20), fits(20)
      diffm = 1000.d0
      imin = 0
      do i=1,221
      diff = abs(size - probs(i))
      if (diff.lt.diffm) then
         diffm = diff
         imin = i
         if (diffm.lt.1.d-6) go to 100
      end if
      end do
  100 continue
c
      nph = np/2
      nptop = 221 - nph
      if (imin.gt.nph.and.imin.lt.nptop) then
c
c imin is not too close to the end. Use np points around stat.
c
        do i=1,np
          ic = imin - nph - 1 + i
          call eval(beta(1,ic),crits(ic),model,nreg,nobs)
          yvect(i) = crits(ic)
          xmat(i,1) = 1.d0
          xmat(i,2) = cnorm(ic)
          xmat(i,3) = xmat(i,2)*cnorm(ic)
          xmat(i,4) = xmat(i,3)*cnorm(ic)
        end do
c
c form omega matrix
c
        do i=1,np
          do j=i,np
            ic = imin - nph - 1 + i
            jc = imin - nph - 1 + j
            top = probs(ic)*(1.d0 - probs(jc))
            bot = probs(jc)*(1.d0 - probs(ic))
            omega(i,j) = wght(ic)*wght(jc)*sqrt(top/bot)
          end do
        end do
        do i=1,np
          do j=i,np
            omega(j,i) = omega(i,j)
          end do
        end do
c
        nx = 4
        call gls(xmat,yvect,omega,gamma,xomx,fits,resid,ssr,ssrt,
     &    np,nx,20,4,0)
c
c check to see if gamma(4) is needed
c
        sd4 = sqrt((ssrt/(np-nx))*xomx(4,4))
        ttest = abs(gamma(4))/sd4
        if (ttest.gt.precrt) then
          call innorz(size,anorm)
          cval = gamma(1) + gamma(2)*anorm + gamma(3)*anorm**2
     &      + gamma(4)*anorm**3
          return
        else
          nx = 3
          call gls(xmat,yvect,omega,gamma,xomx,fits,resid,ssr,ssrt,
     &      np,nx,20,4,1)
          call innorz(size,anorm)
          cval = gamma(1) + gamma(2)*anorm + gamma(3)*anorm**2
          return
        end if
c
c imin is close to one of the ends. Use points from imin +/- nph to end.
c
      else
        if (imin.lt.np) then
          np1 = imin + nph
          if (np1.lt.5) np1 = 5
          do i=1,np1
            call eval(beta(1,i),crits(i),model,nreg,nobs)
            yvect(i) = crits(i)
            xmat(i,1) = 1.d0
            xmat(i,2) = cnorm(i)
            xmat(i,3) = xmat(i,2)*cnorm(i)
            xmat(i,4) = xmat(i,3)*cnorm(i)
          end do
        else
          np1 = 222 - imin + nph
          if (np1.lt.5) np1 = 5
          do i=1,np1
            call eval(beta(1,222-i),crits(222-i),model,nreg,nobs)
            ic = 222 - i
            yvect(i) = crits(ic)
            xmat(i,1) = 1.d0
            xmat(i,2) = cnorm(ic)
            xmat(i,3) = xmat(i,2)*cnorm(ic)
            xmat(i,4) = xmat(i,3)*cnorm(ic)
          end do
        end if
c
c form omega matrix
c
        do i=1,np1
          do j=i,np1
            if (imin.lt.np) then
              top = probs(i)*(1.d0 - probs(j))
              bot = probs(j)*(1.d0 - probs(i))
              omega(i,j) = wght(i)*wght(j)*sqrt(top/bot)
            else
c
c This is to avoid numerical singularities at the upper end
c
              omega(i,j) = 0.d0
              if (i.eq.j) omega(i,i) = 1.d0
            end if
          end do
        end do
        do i=1,np1
          do j=i,np1
            omega(j,i) = omega(i,j)
          end do
        end do
c
        nx = 4
        call gls(xmat,yvect,omega,gamma,xomx,fits,resid,ssr,ssrt,
     &    np1,nx,20,4,0)
c
c check to see if gamma(4) is needed
c
        sd4 = sqrt((ssrt/(np1-nx))*xomx(4,4))
        ttest = abs(gamma(4)/sd4)
        if (ttest.gt.precrt) then
          call innorz(size,anorm)
          cval = gamma(1) + gamma(2)*anorm + gamma(3)*anorm**2
     &      + gamma(4)*anorm**3
          return
        else
          nx = 3
          call gls(xmat,yvect,omega,gamma,xomx,fits,resid,ssr,ssrt,
     &      np1,nx,20,4,1)
          call innorz(size,anorm)
          cval = gamma(1) + gamma(2)*anorm + gamma(3)*anorm**2
          return
        end if
c
      end if
      end


C ******************************************************************************


      subroutine fpval(beta, cnorm, wght, probs, pval, stat,
     &  precrt, nobs, model, nreg, np, nx)
      implicit double precision (a-h,o-z)
c
c Copyright (c) James G. MacKinnon, 1995
c Routine to find P value for any specified test statistic.
c
      double precision beta(4,221), crits(221), cnorm(221), wght(221),
     &  probs(221)
      double precision yvect(20),xmat(20,4),resid(20),gamma(4)
      double precision omega(20,20), fits(20), xomx(4,4)
c
c first, compute all the estimated critical values
c
      do i=1,221
        call eval(beta(1,i),crits(i),model,nreg,nobs)
      end do
c
c find critical value closest to test statistic
c
      diffm = 1000.d0
      imin = 0
      do i=1,221
      diff = abs(stat - crits(i))
      if (diff.lt.diffm) then
         diffm = diff
         imin = i
      end if
      end do
c
      nph = np/2
      nptop = 221 - nph
      if (imin.gt.nph.and.imin.lt.nptop) then
c
c imin is not too close to the end. Use np points around stat.
c
        do i=1,np
          ic = imin - nph - 1 + i
          yvect(i) = cnorm(ic)
          xmat(i,1) = 1.d0
          xmat(i,2) = crits(ic)
          xmat(i,3) = xmat(i,2)*crits(ic)
          xmat(i,4) = xmat(i,3)*crits(ic)
        end do
c
c form omega matrix
c
        do i=1,np
          do j=i,np
            ic = imin - nph - 1 + i
            jc = imin - nph - 1 + j
            top = probs(ic)*(1.d0 - probs(jc))
            bot = probs(jc)*(1.d0 - probs(ic))
            omega(i,j) = wght(ic)*wght(jc)*sqrt(top/bot)
          end do
        end do
        do i=1,np
          do j=i,np
            omega(j,i) = omega(i,j)
          end do
        end do
c
        nx = 4
        call gls(xmat,yvect,omega,gamma,xomx,fits,resid,ssr,ssrt,
     &    np,nx,20,4,0)
c
c check to see if gamma(4) is needed
c
        sd4 = sqrt((ssrt/(np-nx))*xomx(4,4))
        ttest = abs(gamma(4))/sd4
        if (ttest.gt.precrt) then
          crfit = gamma(1) + gamma(2)*stat + gamma(3)*stat**2
     &      + gamma(4)*stat**3
          call ddnor(crfit,pval)
          return
        else
          nx = 3
          call gls(xmat,yvect,omega,gamma,xomx,fits,resid,ssr,ssrt,
     &      np,nx,20,4,1)
          crfit = gamma(1) + gamma(2)*stat + gamma(3)*stat**2
          call ddnor(crfit,pval)
          return
        end if
      else
c
c imin is close to one of the ends. Use points from imin +/- nph to end.
c
        if (imin.lt.np) then
          np1 = imin + nph
          if (np1.lt.5) np1 = 5
          do i=1,np1
            yvect(i) = cnorm(i)
            xmat(i,1) = 1.d0
            xmat(i,2) = crits(i)
            xmat(i,3) = xmat(i,2)*crits(i)
            xmat(i,4) = xmat(i,3)*crits(i)
          end do
        else
          np1 = 222 - imin + nph
          if (np1.lt.5) np1 = 5
          do i=1,np1
            ic = 222 - i
            yvect(i) = cnorm(ic)
            xmat(i,1) = 1.d0
            xmat(i,2) = crits(ic)
            xmat(i,3) = xmat(i,2)*crits(ic)
            xmat(i,4) = xmat(i,3)*crits(ic)
          end do
        end if
c
c form omega matrix
c
        do i=1,np1
          do j=i,np1
            if (imin.lt.np) then
              top = probs(i)*(1.d0 - probs(j))
              bot = probs(j)*(1.d0 - probs(i))
              omega(i,j) = wght(i)*wght(j)*sqrt(top/bot)
            else
c
c This is to avoid numerical singularities at the upper end
c
              omega(i,j) = 0.d0
              if (i.eq.j) omega(i,i) = 1.d0
            end if
          end do
        end do
        do i=1,np1
          do j=i,np1
            omega(j,i) = omega(i,j)
          end do
        end do
c
        nx = 4
        call gls(xmat,yvect,omega,gamma,xomx,fits,resid,ssr,ssrt,
     &    np1,nx,20,4,0)
c
c check to see if gamma(4) is needed
c
        sd4 = sqrt((ssrt/(np1-nx))*xomx(4,4))
        ttest = abs(gamma(4))/sd4
        if (ttest.gt.precrt) then
          crfit = gamma(1) + gamma(2)*stat + gamma(3)*stat**2
     &      + gamma(4)*stat**3
          call ddnor(crfit,pval)
        else
          nx = 3
          call gls(xmat,yvect,omega,gamma,xomx,fits,resid,ssr,ssrt,
     &      np1,nx,20,4,1)
          crfit = gamma(1) + gamma(2)*stat + gamma(3)*stat**2
          call ddnor(crfit,pval)
        end if
c
c check that nothing crazy has happened at the ends
c
        if (imin.eq.1.and.pval.gt.probs(1)) pval = probs(1)
        if (imin.eq.221.and.pval.lt.probs(221)) pval = probs(221)
        return
      end if
      end


C ******************************************************************************


      subroutine eval(beta,cval,model,nreg,nobs)
      implicit double precision (a-h,o-z)
c
c Copyright (c) James G. MacKinnon, 1995
c Routine to evaluate response surface for specified betas and sample size.
c
      double precision beta(4)
      if (nobs.eq.0) then
         cval = beta(1)
         return
      end if
      if (model.eq.2) then
         onobs = 1.d0/nobs
         cval = beta(1) + beta(2)*onobs + beta(3)*onobs**2
         return
      end if
      if (model.eq.3) then
         onobs = 1.d0/nobs
         cval = beta(1) + beta(2)*onobs + beta(3)*onobs**2
     &     + beta(4)*onobs**3
         return
      end if
      if (model.eq.4) then
         onobs = 1.d0/(nobs - nreg)
         cval = beta(1) + beta(2)*onobs + beta(3)*onobs**2
         return
      end if
      if (model.eq.5) then
         onobs = 1.d0/(nobs - nreg)
         cval = beta(1) + beta(2)*onobs + beta(3)*onobs**2
     &     + beta(4)*onobs**3
         return
      end if
C      write(6,*) '*** Warning! Error in input file. ***'
      return
      end


C ******************************************************************************


      subroutine gls(xmat,yvect,omega,beta,xomx,fits,resid,ssr,ssrt,
     &  nobs,nvar,nomax,nvmax,ivrt)
c
c Copyright (c) James G. MacKinnon, 1995
c Subroutine to do GLS estimation the obvious way
c Use only when sample size is small (nobs <= 50)
c 1995-1-3
c
      implicit double precision (a-h,o-z)
      double precision xmat(nomax,nvmax), yvect(nomax),
     &  omega(nomax,nomax)
      double precision beta(nvmax), xomx(nvmax,nvmax), fits(nomax),
     &  resid(nomax)
      double precision xomy(50)
c
c xomx is covariance matrix of parameter estimates if omega is truly known
c First, invert omega matrix if ivrt=0. Original one gets replaced.
c
      if (ivrt.eq.0) call cholx(omega,nomax,nobs,kxx)
c
c form xomx matrix and xomy vector
c
      do j=1,nvar
        xomy(j) = 0.d0
        do l=j,nvar
          xomx(j,l) = 0.d0
        end do
      end do
c
      do 21 i=1,nobs
      do 21 k=1,nobs
      do 24 j=1,nvar
      xomy(j) = xomy(j) + xmat(i,j)*omega(k,i)*yvect(k)
      do 24 l=j,nvar
      xomx(j,l) = xomx(j,l) + xmat(i,j)*omega(k,i)*xmat(k,l)
   24 continue
   21 continue
c
      do j=1,nvar
        do l=j,nvar
          xomx(l,j) = xomx(j,l)
        end do
      end do
c
c invert xomx matrix
c
      call cholx(xomx,nvmax,nvar,kxx)
c
c  now form estimates of beta.
c
      do 5 i=1,nvar
      beta(i) = 0.d0
      do 5 j=1,nvar
      beta(i) = beta(i) + xomx(i,j)*xomy(j)
    5 continue
c
c find ssr, fitted values, and residuals
c
      ssr = 0.d0
      do i=1,nobs
        fits(i) = 0.d0
        do j=1,nvar
          fits(i) = fits(i) + xmat(i,j)*beta(j)
        end do
        resid(i) = yvect(i) - fits(i)
        ssr = ssr + resid(i)**2
      end do
c
c find ssr from transformed regression
c
      ssrt = 0.d0
      do i=1,nobs
        do k=1,nobs
          ssrt = ssrt + resid(i)*omega(k,i)*resid(k)
        end do
      end do
c
      return
      end


C ******************************************************************************


      subroutine cholx(amat,m,n,kxx)
      implicit double precision (a-h,o-z)
c
c Copyright (c) James G. MacKinnon, 1993
c This routine uses the cholesky decomposition to invert a real
c symmetric matrix.
c
      double precision amat(m,m)
      kxx = 0
      do 8 i=1,n
      kl = i - 1
      do 7 j=i,n
      if (i.gt.1) then
        do 3 k=1,kl
    3   amat(i,j) = amat(i,j) - amat(k,i)*amat(k,j)
      else
        if (amat(i,i).le.0.d0) then
        kxx = i
        go to 20
      end if
      end if
      if (i.eq.j) then
        amat(i,i) = dsqrt(amat(i,i))
      else
        if (j.eq.i+1) ooa = 1.d0/amat(i,i)
        amat(i,j) = amat(i,j)*ooa
      end if
    7 continue
    8 continue
      do 13 i=1,n
      do 12 j=i,n
      ooa = 1.d0/amat(j,j)
      if (i.ge.j) then
        t = 1.d0
        go to 12
      end if
      kl = j - 1
      t = 0.d0
      do 11 k=i,kl
   11 t = t - amat(i,k)*amat(k,j)
   12 amat(i,j) = t*ooa
   13 continue
      do 16 i=1,n
      do 15 j=i,n
      t = 0.d0
      do 14 k=j,n
   14 t = t + amat(i,k)*amat(j,k)
      amat(i,j) = t
   19 amat(j,i) = t
   15 continue
   16 continue
   20 return
      end


C ******************************************************************************


      subroutine ddnor(ystar,gauss)
      implicit double precision(a-h,o-z)
c
c Copyright (c) James G. MacKinnon, 1993
c Routine to evaluate cumulative normal distribution
c Written originally in late 1970's
c Modified 1993 to avoid changing the argument
c
c This subroutine uses Cody's method to evaluate the cumulative
c normal distribution. It is probably accurate to 19 or 20
c significant digits. It was written in 1977, based on the Cody
c article referred to in the documentation for IMSL subroutine mdnor.
c
      double precision p(6), q(5), a(9), b(8), c(5), d(4)
      data p(1)/-6.58749161529837803157d-04/,
     1     p(2)/-1.60837851487422766278d-02/,
     2     p(3)/-1.25781726111229246204d-01/,
     3     p(4)/-3.60344899949804439429d-01/,
     4     p(5)/-3.05326634961232344035d-01/,
     5     p(6)/-1.63153871373020978498d-02/
      data q(1)/2.33520497626869185443d-03/,
     1     q(2)/6.05183413124413191178d-02/,
     2     q(3)/5.27905102951428412248d-01/,
     3     q(4)/1.87295284992346047209d00/,
     4     q(5)/2.56852019228982242072d00/
      data a(1)/1.23033935479799725272d03/,
     1     a(2)/2.05107837782607146532d03/,
     2     a(3)/1.71204761263407058314d03/,
     3     a(4)/8.81952221241769090411d02/,
     4     a(5)/2.98635138197400131132d02/,
     5     a(6)/6.61191906371416294775d01/,
     6     a(7)/8.88314979438837594118d00/,
     7     a(8)/5.64188496988670089180d-01/,
     8     a(9)/2.15311535474403846343d-08/
      data b(1)/1.23033935480374942043d03/,
     1     b(2)/3.43936767414372163696d03/,
     2     b(3)/4.36261909014324715820d03/,
     3     b(4)/3.29079923573345962678d03/,
     4     b(5)/1.62138957456669018874d03/,
     5     b(6)/5.37181101862009857509d02/,
     6     b(7)/1.17693950891312499305d02/,
     7     b(8)/1.57449261107098347253d01/
      data c(1)/3.209377589138469472562d03/,
     1     c(2)/3.774852376853020208137d02/,
     2     c(3)/1.138641541510501556495d02/,
     3     c(4)/3.161123743870565596947d00/,
     4     c(5)/1.857777061846031526730d-01/
      data d(1)/2.844236833439170622273d03/,
     1     d(2)/1.282616526077372275645d03/,
     2     d(3)/2.440246379344441733056d02/,
     3     d(4)/2.360129095234412093499d01/
      data orpi/.5641895835477562869483d0/,
     1   root2/.70710678118654752440083d0/
c
      isw = 1
      y = ystar
      if (ystar.lt.-16.d0) y = -16.d0
      if (ystar.gt.16.d0) y = 16.d0
      x = -y*root2
      if(x.gt.0.d0) go to 1
      if(x.lt.0.d0) go to 2
      gauss = .5d0
      return
    2 continue
      x = - x
      isw = -1
    1 continue
      if(x.lt..477d0) go to 10
      if(x.le.4.d0) go to 20
c
c  evaluate erfc for x.gt.4.0
c
      x2 = x*x
      xm2 = 1.d0/x2
      xm4 = xm2*xm2
      xm6 = xm4*xm2
      xm8 = xm4*xm4
      xm10 = xm6*xm4
      top = p(1) + p(2)*xm2 + p(3)*xm4 + p(4)*xm6 + p(5)*xm8 + p(6)*xm10
      bot = q(1) + q(2)*xm2 + q(3)*xm4 + q(4)*xm6 + q(5)*xm8 + xm10
      crap = orpi + top/(bot*x2)
      erfc = dexp(-x2)*crap/x
c
      if(isw.eq.-1) erfc = 2.d0 - erfc
      gauss = erfc*.5d0
      return
   20 continue
c
c  evaluate erfc for .477.lt.x.le.4.0
c
      x2 = x*x
      x3 = x2*x
      x4 = x2*x2
      x5 = x3*x2
      x6 = x3*x3
      x7 = x3*x4
      x8 = x4*x4
      top = a(1) + a(2)*x + a(3)*x2 + a(4)*x3 + a(5)*x4 + a(6)*x5 +
     &  a(7)*x6 + a(8)*x7 + a(9)*x8
      bot = b(1) + b(2)*x + b(3)*x2 + b(4)*x3 + b(5)*x4 + b(6)*x5 +
     &  b(7)*x6 + b(8)*x7 + x8
      erfc = dexp(-x2)*top/bot
c
      if(isw.eq.-1) erfc = 2.d0 - erfc
      gauss = erfc*.5d0
      return
   10 continue
c
c  evaluate erf for x.lt..477
c
      x2 = x*x
      x4 = x2*x2
      x6 = x4*x2
      x8 = x4*x4
      top = c(1) + c(2)*x2 + c(3)*x4 + c(4)*x6 + c(5)*x8
      bot = d(1) + d(2)*x2 + d(3)*x4 + d(4)*x6 + x8
      erf = x*top/bot
c
      erf = erf*isw
      erfc = 1.d0 - erf
      gauss = erfc*.5d0
      return
      end


C ******************************************************************************


      subroutine innorz(prob,anorm)
      implicit double precision (a-h,o-z)
c
c Copyright (c) James G. MacKinnon, 1995
c Inverse normal routine that adjusts crude result twice.
c It seems to be accurate to about 14 digits.
c Crude result is taken from Abramowitz & Stegun (1968)
c It should have abs. error < 4.5 * 10^-4
c
      data c0/2.515517d0/, d1/1.432788d0/, c1/0.802853d0/
      data c2/0.010328d0/, d3/0.001308d0/, d2/0.189269d0/
      data const/.398942280401432678d0/
c      if (prob.lt.0.d0.or.prob.gt.1.d0) then
c         write(6,*) 'Attempt to find inverse normal of ', prob
c         stop
c      end if
      pr = prob
      if (prob.gt.0.5d0) pr = 1.d0 - prob
      arg = 1/pr**2
      t = sqrt(log(arg))
      anorm = t - (c0 + c1*t + c2*t**2)/
     &  (1 + d1*t + d2*t**2 + d3*t**3)
c
c now correct crude result by direct method
c
      call ddnor(anorm,prob2)
      pr2 = 1.d0 - prob2
      arg = 1/pr2**2
      t = sqrt(log(arg))
      anorm2 = t - (c0 + c1*t + c2*t**2)/
     &  (1 + d1*t + d2*t**2 + d3*t**3)
      anorm = anorm + anorm - anorm2
      if (prob.lt.0.5d0) anorm = -anorm
c
c now correct better result, using Taylor series approximation
c
      call ddnor(anorm,prob2)
      error = prob2 - prob
      dens = const*dexp(-.5d0*anorm**2)
      anorm = anorm - error/dens
      return
      end

##
## Johansen Procedure
##
ca.jo <- function(x, type = c("eigen", "trace"), ecdet = c("none", "const", "trend"), K = 2, spec = c("longrun", "transitory"), season = NULL, dumvar = NULL)
{
  x <- as.matrix(x)
  colnames(x) <- make.names(colnames(x))
  type <- match.arg(type)
  ecdet <- match.arg(ecdet)
  spec <- match.arg(spec)
  K <- as.integer(K)
  if (K < 2) {
    stop("\nK must be at least K=2.\n")
  }
  P <- ncol(x)
  arrsel <- P
  N <- nrow(x)
  if (!is.null(season)) {
    s <- season - 1
  } else {
    s <- 0
  }
  if (N * P < P + s * P + K * P^2 + P * (P + 1)/2)
    stop("\nInsufficient degrees of freedom.\n")
  if (P > 11)
    warning("\nToo many variables, critical values cannot be computed.\n")
  ##
  ## Check of NA's in x
  ## (create index of NA entries)
  ##
  if (NA %in% x) {
    idx.NA <- 1:N
    ind <- as.logical(sapply(idx.NA, function(z) sum(is.na(x[z, ]) * 1)))
    ind2 <- ind * (1:N)
  }
  ##
  ## Setting seasonal dummies
  ## (if applicable)
  ##
  if (!(is.null(season))) {
    dum <- (diag(season) - 1/season)[, -season]
    dums <- dum
    while (nrow(dums) < N) {
      dums <- rbind(dums, dum)
    }
    dums <- dums[1:N, ]
    if (NA %in% x) {
      dums <- dums[-ind2, ]
    }
    colnames(dums) <- paste("sd", 1:ncol(dums), sep="")
  }
  ##
  ## Setting dummy variables
  ## (if applicable)
  ##
  if (!(is.null(dumvar))) {
    dumvar <- as.matrix(dumvar)
    colnames(dumvar) <- make.names(colnames(dumvar))
    if(is.null(colnames(dumvar))){
      dumcols <- ncol(dumvar)
      colnames(dumvar) <- paste("exo", 1:dumcols, sep = "")
      warning("\nNo column names in 'dumvar', using prefix 'exo' instead.\n")
    }
    if (!(nrow(dumvar) == nrow(x))) {
      stop("\nUnequal row length between dummy variables and x matrix.\n")
    }
    if (NA %in% x) {
      dumvar <- dumvar[-ind2, ]
    }
  }
  ##
  ## Setting trend 
  ## (if applicable)
  ##
  if(ecdet == "trend"){
    trend <- 1:nrow(x)
    if (NA %in% x) {
      trend <- trend[-ind2]
    }
  }
  x <- na.omit(x)
  N <- nrow(x)
  Z <- embed(diff(x), K)
  Z0 <- Z[, 1:P]
  ##
  ## Critival values of tests
  ##
  cv.none <- array(c(6.5, 12.91, 18.9, 24.78, 30.84, 36.25, 42.06, 48.43, 54.01, 59., 65.07, 8.18, 14.90, 21.07, 27.14, 33.32, 39.43, 44.91, 51.07, 57.00, 62.42, 68.27, 11.65, 19.19, 25.75, 32.14, 38.78, 44.59, 51.30, 57.07, 63.37, 68.61, 74.36, 6.50, 15.66, 28.71, 45.23, 66.49, 85.18, 118.99, 151.38, 186.54, 226.34, 269.53, 8.18, 17.95, 31.52, 48.28, 70.6, 90.39, 124.25, 157.11, 192.84, 232.49, 277.39, 11.65, 23.52, 37.22, 55.43, 78.87, 104.20, 136.06, 168.92, 204.79, 246.27, 292.65), c(11, 3, 2))
  cv.const <- array(c(7.52, 13.75, 19.77, 25.56, 31.66, 37.45, 43.25, 48.91, 54.35, 60.25, 66.02, 9.24, 15.67, 22.00, 28.14, 34.40, 40.30, 46.45, 52.00, 57.42, 63.57, 69.74, 12.97, 20.20, 26.81, 33.24, 39.79, 46.82, 51.91, 57.95, 63.71, 69.94, 76.63, 7.52, 17.85, 32.00, 49.65, 71.86, 97.18, 126.58, 159.48, 196.37, 236.54, 282.45, 9.24, 19.96, 34.91, 53.12, 76.07, 102.14, 131.70, 165.58, 202.92, 244.15, 291.40, 12.97, 24.60, 41.07, 60.16, 84.45, 111.01, 143.09, 177.20, 215.74, 257.68, 307.64), c(11, 3, 2))
  cv.trend <- array(c(10.49, 16.85, 23.11, 29.12, 34.75, 40.91, 46.32, 52.16, 57.87, 63.18, 69.26, 12.25, 18.96, 25.54, 31.46, 37.52, 43.97, 49.42, 55.50, 61.29, 66.23, 72.72, 16.26, 23.65, 30.34, 36.65, 42.36, 49.51, 54.71, 62.46, 67.88, 73.73, 79.23, 10.49, 22.76, 39.06, 59.14, 83.20, 110.42, 141.01, 176.67, 215.17, 256.72, 303.13, 12.25, 25.32, 42.44, 62.99, 87.31, 114.90, 146.76, 182.82, 222.21, 263.42, 310.81, 16.26, 30.45, 48.45, 70.05, 96.58, 124.75, 158.49, 196.08, 234.41, 279.07, 327.45), c(11, 3, 2))
  if (ecdet == "none") {
    cvals <- cv.none
  } else if (ecdet == "const") {
    cvals <- cv.const
  } else if (ecdet == "trend") {
    cvals <- cv.trend
  }
  ##
  ## Setting of relevant matrices
  ##
  if (ecdet == "const") {
    if (spec == "longrun") {
      ZK <- cbind(x[-c((N - K + 1):N), ], 1)
      Lnotation <- K
    } else if (spec == "transitory") {
      ZK <- cbind(x[-N, ], 1)[K:(N - 1), ]
      Lnotation <- 1
    }
    colnames(ZK) <- c(paste(colnames(x), ".l", Lnotation, sep=""), "constant")
    Z1 <- Z[, -c(1:P)]
    temp1 <- NULL
    for(i in 1:(K-1)){
      temp <- paste(colnames(x), ".dl", i, sep="")
      temp1 <- c(temp1, temp)
    }
    colnames(Z1) <- temp1
    P <- P + 1
    idx <- 0:(P - 2)
    model <- "without linear trend and constant in cointegration"
  } else if(ecdet == "none"){
    if (spec == "longrun") {
      ZK <- x[-c((N - K + 1):N), ]
      Lnotation <- K
    }
    else if (spec == "transitory") {
      ZK <- x[-N, ][K:(N - 1), ]
      Lnotation <- 1
    }
    colnames(ZK) <- paste(colnames(x), ".l", Lnotation, sep="")
    Z1 <- Z[, -c(1:P)]
    Z1 <- cbind(1, Z1)
    temp1 <- NULL
    for(i in 1:(K-1)){
      temp <- paste(colnames(x), ".dl", i, sep="")
      temp1 <- c(temp1, temp)
    }
    temp1 <- c("constant", temp1)
    colnames(Z1) <- temp1
    idx <- 0:(P - 1)
    model <- "with linear trend"
  } else if(ecdet == "trend"){
    if(spec == "longrun") {
      ZK <- cbind(x[-c((N - K + 1):N), ], trend[-c((N - K + 1):N)])
      Lnotation <- K
    } else if (spec == "transitory") {
      ZK <- cbind(x[-N, ], trend[-N])[K:(N - 1), ]
      Lnotation <- 1
    }
    colnames(ZK) <- c(paste(colnames(x), ".l", Lnotation, sep=""), paste("trend.l", Lnotation, sep = ""))
    Z1 <- Z[, -c(1:P)]
    Z1 <- cbind(1, Z1)
    temp1 <- NULL
    for(i in 1:(K-1)){
      temp <- paste(colnames(x), ".dl", i, sep="")
      temp1 <- c(temp1, temp)
    }
    temp1 <- c("constant", temp1)
    colnames(Z1) <- temp1
    P <- P + 1
    idx <- 0:(P - 2)
    model <- "with linear trend in cointegration"
  }
  N <- nrow(Z0)
  if (!(is.null(season))) {
    if(ecdet == "const") {
      Z1 <- cbind(dums[-(1:K), ], Z1)
    } else {
      Z1 <- cbind(Z1[, 1], dums[-(1:K), ], Z1[, -1])
      colnames(Z1) <- c("constant", colnames(Z1)[-1])
    }
  }
  if (!(is.null(dumvar))) {
    tmp <- colnames(Z1)
    if(ecdet == "const"){
      Z1 <- cbind(dumvar[-(1:K), ], Z1)
      colnames(Z1) <- c(colnames(dumvar), tmp)
    } else {
      Z1 <- cbind(Z1[, 1], dumvar[-(1:K), ], Z1[, -1])
      colnames(Z1) <- c("constant", colnames(dumvar), tmp[-1])
    }
  }
  M00 <- crossprod(Z0)/N
  M11 <- crossprod(Z1)/N
  MKK <- crossprod(ZK)/N
  M01 <- crossprod(Z0, Z1)/N
  M0K <- crossprod(Z0, ZK)/N
  MK0 <- crossprod(ZK, Z0)/N
  M10 <- crossprod(Z1, Z0)/N
  M1K <- crossprod(Z1, ZK)/N
  MK1 <- crossprod(ZK, Z1)/N
  M11inv <- solve(M11)
  R0 <- Z0 - t(M01 %*% M11inv %*% t(Z1))
  RK <- ZK - t(MK1 %*% M11inv %*% t(Z1))
  S00 <- M00 - M01 %*% M11inv %*% M10
  S0K <- M0K - M01 %*% M11inv %*% M1K
  SK0 <- MK0 - MK1 %*% M11inv %*% M10
  SKK <- MKK - MK1 %*% M11inv %*% M1K
  Ctemp <- chol(SKK, pivot = TRUE)
  pivot <- attr(Ctemp, "pivot")
  oo <- order(pivot)
  C <- t(Ctemp[, oo])
  Cinv <- solve(C)
  S00inv <- solve(S00)
  valeigen <- eigen(Cinv %*% SK0 %*% S00inv %*% S0K %*% t(Cinv))
  lambda <- valeigen$values
  e <- valeigen$vector
  V <- t(Cinv) %*% e
  Vorg <- V
  V <- sapply(1:P, function(x) V[, x]/V[1, x])
  W <- S0K %*% V %*% solve(t(V) %*% SKK %*% V)
  PI <- S0K %*% solve(SKK)
  DELTA <- S00 - S0K %*% V %*% solve(t(V) %*% SKK %*% V) %*% t(V) %*% SK0
  GAMMA <- M01 %*% M11inv - PI %*% MK1 %*% M11inv
  if (type == "trace") {
    type <- "trace statistic"
    teststat <- as.matrix(rev(sapply(idx, function(x) -N * sum(log(1 - lambda[(x + 1):P])))))
    colnames(teststat) <- "trace"
    if (arrsel > 11) {
      cval <- NULL
    } else {
      cval <- round(cvals[1:arrsel, , 2], 2)
      colnames(cval) <- c("10pct", "5pct", "1pct")
      rownames(cval) <- c(paste("r <= ", (arrsel - 1):1, " |", sep = ""), "r = 0  |")
    }
  } else if (type == "eigen") {
    type <- "maximal eigenvalue statistic (lambda max)"
    teststat <- as.matrix(rev(sapply(idx, function(x) -N * log(1 - lambda[x + 1]))))
    colnames(teststat) <- "lambda max."
    if (arrsel > 11) {
      cval <- NULL
    } else {
      cval <- round(cvals[1:arrsel, , 1], 2)
      colnames(cval) <- c("10pct", "5pct", "1pct")
      rownames(cval) <- c(paste("r <= ", (arrsel - 1):1, " |", sep = ""), "r = 0  |")
    }
  }
  colnames(V) <- colnames(ZK)
  rownames(V) <- colnames(ZK) 
  rownames(W) <- paste(colnames(x), ".d", sep = "")
  colnames(W) <- colnames(ZK)
  colnames(Vorg) <- colnames(V)
  rownames(Vorg) <- rownames(V)
  rownames(PI) <- rownames(W)
  colnames(PI) <- colnames(W)
  colnames(Z0) <- paste(colnames(x), ".d", sep="")
  colnames(R0) <- paste("R0", colnames(Z0), sep=".")
  colnames(RK) <- paste("RK", colnames(ZK), sep=".")
  rownames(GAMMA) <- rownames(W)

  new("ca.jo", x = x, Z0 = Z0, Z1 = Z1, ZK = ZK, type = type, model = model, ecdet = ecdet, lag = K, P = arrsel, season = season, dumvar = dumvar, cval = cval, teststat = as.vector(teststat), lambda = lambda, Vorg = Vorg, V = V, W = W, PI = PI, DELTA = DELTA, GAMMA = GAMMA, R0 = R0, RK = RK, bp = NA, test.name = "Johansen-Procedure", spec = spec, call = match.call())  
}

##
## alrtest
##
alrtest <- function(z, A, r){
  if(!inherits(z, "ca.jo")){
    stop("\nPlease, provide object of class 'ca.jo' as 'z'.\n")
  }
  r <- as.integer(r)
  A <- as.matrix(A)
  if(!(nrow(A)==z@P)){
    stop("\nRow number of 'A' is unequal to VAR order.\n")
  }
  if(r >= z@P || r<1){
    stop("\nCount of cointegrating relationships is out of allowable range.\n")
  }
  type <- "Estimation and testing under linear restrictions on beta"
  B <- qr.Q(qr(A), complete=TRUE)[,-c(1:ncol(A))]
  N <- nrow(z@Z0)
  M00 <- crossprod(z@Z0)/N
  M11 <- crossprod(z@Z1)/N
  MKK <- crossprod(z@ZK)/N
  M01 <- crossprod(z@Z0, z@Z1)/N
  M0K <- crossprod(z@Z0, z@ZK)/N
  MK0 <- crossprod(z@ZK, z@Z0)/N
  M10 <- crossprod(z@Z1, z@Z0)/N
  M1K <- crossprod(z@Z1, z@ZK)/N
  MK1 <- crossprod(z@ZK, z@Z1)/N
  M11inv <- solve(M11)
  S00 <- M00 - M01%*%M11inv%*%M10
  S0K <- M0K - M01%*%M11inv%*%M1K
  SK0 <- MK0 - MK1%*%M11inv%*%M10
  SKK <- MKK - MK1%*%M11inv%*%M1K
  Sab <- t(A)%*%S00%*%B
  Skb <- t(S0K)%*%B
  Sbb <- t(B)%*%S00%*%B
  Sbbinv <- solve(Sbb)
  RA <- z@R0%*%A - z@R0%*%B%*%Sbbinv%*%t(Sab)
  RK <- z@RK - z@R0%*%B%*%Sbbinv%*%t(Skb)
  Saa.b <- crossprod(RA, RA)/N
  Sak.b <- crossprod(RA, RK)/N
  Ska.b <- crossprod(RK, RA)/N
  Skk.b <- crossprod(RK, RK)/N
  Ctemp <- chol(Skk.b, pivot=TRUE)
  pivot <- attr(Ctemp, "pivot")
  oo <- order(pivot)
  C <- t(Ctemp[,oo])
  Cinv <- solve(C)
  Saa.binv <- solve(Saa.b)
  valeigen <- eigen(Cinv%*%Ska.b%*%Saa.binv%*%Sak.b%*%t(Cinv), symmetric = TRUE)
  lambda.res <- valeigen$values
  e <- valeigen$vector
  V <- t(Cinv)%*%e
  V <- as.matrix(V[,1:r])
  Vorg <- V
  idx <- 1:r
  V <- sapply(idx, function(x) V[ , x] / V[1,x])
  PHI <- solve(t(A)%*%A)%*%Sak.b%*%Vorg
  ALPHA <- as.matrix(A%*%PHI)
  ALPHAorg <- ALPHA
  ALPHA <- sapply(idx, function(x) ALPHA[ , x] * Vorg[1,x])
  PI <- ALPHA %*% t(V)
  GAMMA <- M01%*%M11inv - PI%*%MK1%*%M11inv
  DELTA.bb <- Sbb
  DELTA.ab <- Sab - t(A)%*%ALPHA%*%t(V)%*%Skb
  DELTA.aa.b <- Saa.b - t(A)%*%ALPHA%*%t(ALPHA)%*%A
  lambda <- z@lambda
  teststat <- N*sum(log((1-lambda.res[1:r])/(1-lambda[1:r])))
  df <- r*(z@P - ncol(A))
  pval <- c(1-pchisq(q = teststat, df = df), df)
  new("cajo.test", Z0=z@Z0, Z1=z@Z1, ZK=z@ZK, ecdet=z@ecdet, H=NULL, A=A, B=B, type=type, teststat=teststat, pval=pval, lambda=lambda.res, Vorg=Vorg, V=V, W=ALPHA, PI=PI, DELTA=NULL, DELTA.bb=DELTA.bb, DELTA.ab=DELTA.ab, DELTA.aa.b=DELTA.aa.b, GAMMA=GAMMA, test.name="Johansen-Procedure")
}

##
## cajolst
##
cajolst <- function (x, trend = TRUE, K = 2, season = NULL) 
{
    x <- as.matrix(x)
    K <- as.integer(K)
    if(K < 2){
      stop("\nK must be at least K=2.\n")
    }
    P <- ncol(x)
    arrsel <- P
    N <- nrow(x)
    if (!is.null(season)) {
        s <- season - 1
    }
    else {
        s <- 0
    }
    if (N * P < P + s * P + K * P^2 + P * (P + 1)/2) 
        stop("\nInsufficient degrees of freedom.\n")
    if (P > 5) 
        warning("\nToo many variables, critical values cannot be computed.\n")
    if (!(is.null(season))) {
        dum <- (diag(season) - 1/season)[, -season]
        dums <- dum
        while (nrow(dums) < N) {
            dums <- rbind(dums, dum)
        }
        dums <- dums[1:N, ]
        if (NA %in% x) {
            idx.NA <- 1:N
            ind <- as.logical(sapply(idx.NA, function(z) sum(is.na(x[z, 
                ]) * 1)))
            ind2 <- ind * (1:N)
            dums <- dums[-ind2, ]
        }
    }
    x2 <- na.omit(x)
    Ntot <- nrow(x2)
    y <- embed(x2, (K + 1))
    rhs <- y[, -c(1:P)]
    if (!trend) {
        rhs <- y[, -c(1:P)]
    }
    else {
        trd <- seq(K + 1, nrow(y) + K)
        rhs <- cbind(trd, y[, -c(1:P)])
    }
    N <- nrow(y)
    if (!(is.null(season))) {
        rhs <- cbind(dums[-(1:K), ], rhs)
    }
    lhs <- y[, 1:P]
    idx <- 1:(N - 1)
    tau <- function(t) {
        dt <- c(rep(0, t), rep(1, N - t))
        det(crossprod(resid(lm(lhs ~ dt + rhs))))
    }
    tau.hat <- sapply(idx, tau)
    tau.opt <- which.min(tau.hat) + K
    tau.bp <- tau.opt + 1
    dt <- c(rep(0, tau.opt), rep(1, N - tau.opt))
    if(!trend & is.null(season)){
      rhs.aux <- dt
    } else {
      rhs.aux <- cbind(dt, rhs[, -c((ncol(rhs)-K*ncol(x)+1):ncol(rhs))])
    }
    reg.opt <- lm(lhs ~ rhs.aux)
    dt <- c(rep(0, tau.opt), rep(1, Ntot - tau.opt))
    uv <- c(rep(1, Ntot))
    if (!trend) {
        if (!is.null(season)) {
            yfit <- x - uv%*%t(coef(reg.opt)[1, ]) - dt %*% t(coef(reg.opt)[2, ]) - dums %*% coef(reg.opt)[3:(2 + season - 1), ]
        }else{
            yfit <- x - uv%*%t(coef(reg.opt)[1, ]) - dt %*% t(coef(reg.opt)[2, ])
        }
    }else if (trend){
        trd <- 1:Ntot
        if (!is.null(season)) {
            yfit <- x - uv%*%t(coef(reg.opt)[1, ]) - dt %*% t(coef(reg.opt)[2, ]) - dums %*% coef(reg.opt)[3:(2 + season - 1), ] - trd %*% t(coef(reg.opt)[season + 2, ])
        }else{
            yfit <- x - uv%*%t(coef(reg.opt)[1, ]) - dt %*% t(coef(reg.opt)[2, ]) - trd %*% t(coef(reg.opt)[3, ])
        }
    }
    x <- na.omit(yfit)
    N <- nrow(x)
    spec <- "transitory"
    Z <- embed(diff(x), K)
    Z0 <- Z[, 1:P]
    Z1 <- Z[, -c(1:P)]
    ZK <- x[-N, ][K:(N - 1), ]
    idx <- 0:(P - 1)
    if (trend) {
      cvals <- matrix(c(5.423, 13.784, 25.931, 42.083, 61.918, 6.785, 15.826, 28.455, 45.204, 65.662, 10.042, 19.854, 33.757, 51.601, 73.116), nrow=5, ncol=3)
      model <- "with linear trend in shift correction"
    }else if(!trend){
      cvals <- matrix(c(2.996, 10.446, 21.801, 36.903, 55.952, 4.118, 12.276, 24.282, 40.067, 59.749, 6.888, 16.420, 29.467, 46.305, 67.170), nrow=5, ncol=3)
      model <- "without linear trend in shift correction"
    }
    N <- nrow(Z0)
    M00 <- crossprod(Z0)/N
    M11 <- crossprod(Z1)/N
    MKK <- crossprod(ZK)/N
    M01 <- crossprod(Z0, Z1)/N
    M0K <- crossprod(Z0, ZK)/N
    MK0 <- crossprod(ZK, Z0)/N
    M10 <- crossprod(Z1, Z0)/N
    M1K <- crossprod(Z1, ZK)/N
    MK1 <- crossprod(ZK, Z1)/N
    M11inv <- solve(M11)
    R0 <- Z0 - t(M01 %*% M11inv %*% t(Z1))
    RK <- ZK - t(MK1 %*% M11inv %*% t(Z1))
    S00 <- M00 - M01 %*% M11inv %*% M10
    S0K <- M0K - M01 %*% M11inv %*% M1K
    SK0 <- MK0 - MK1 %*% M11inv %*% M10
    SKK <- MKK - MK1 %*% M11inv %*% M1K
    Ctemp <- chol(SKK, pivot = TRUE)
    pivot <- attr(Ctemp, "pivot")
    oo <- order(pivot)
    C <- t(Ctemp[, oo])
    Cinv <- solve(C)
    S00inv <- solve(S00)
    valeigen <- eigen(Cinv %*% SK0 %*% S00inv %*% S0K %*% t(Cinv))
    lambda <- valeigen$values
    e <- valeigen$vector
    V <- t(Cinv) %*% e
    rownames(V) <- colnames(x)
    Vorg <- V
    V <- sapply(1:P, function(x) V[, x]/V[1, x])
    W <- S0K %*% V %*% solve(t(V) %*% SKK %*% V)
    PI <- S0K %*% solve(SKK)
    DELTA <- S00 - S0K %*% V %*% solve(t(V) %*% SKK %*% V) %*% 
        t(V) %*% SK0
    GAMMA <- M01 %*% M11inv - PI %*% MK1 %*% M11inv
    type <- "trace statistic"
    teststat <- as.matrix(rev(sapply(idx, function(x) N * sum(log(1 + lambda[(x + 1):P])))))
    colnames(teststat) <- "trace"
    if (arrsel > 5) {
      cval <- NULL
    } else {
      cval <- round(cvals[1:arrsel, ], 2)
      colnames(cval) <- c("10pct", "5pct", "1pct")
      rownames(cval) <- c(paste("r <= ", (arrsel - 1):1, " |", sep = ""), "r = 0  |")
    }
    temp1 <- NULL
    for (i in 1:(K - 1)) {
      temp <- paste(colnames(x), ".dl", i, sep = "")
      temp1 <- c(temp1, temp)
    }
    colnames(Z1) <- temp1
    colnames(ZK) <- paste(colnames(x), "l1", sep=".")
    colnames(Z0) <- paste(colnames(x), "d", sep=".")
    colnames(V) <- colnames(ZK)
    rownames(V) <- colnames(ZK)
    colnames(W) <- colnames(V)
    rownames(W) <- colnames(x)
    colnames(Vorg) <- colnames(V)
    rownames(Vorg) <- rownames(V)
    rownames(PI) <- colnames(x)
    colnames(PI) <- colnames(W)
    colnames(R0) <- paste("R0", colnames(Z0), sep = ".")
    colnames(RK) <- paste("RK", colnames(ZK), sep = ".")
    
    new("ca.jo", x = x, Z0 = Z0, Z1 = Z1, ZK = ZK, type = type, model = model, ecdet = "none", lag = K, P = arrsel, 
        season = season, dumvar = NULL, cval = cval, teststat = as.vector(teststat), 
        lambda = lambda, Vorg = Vorg, V = V, W = W, PI = PI, 
        DELTA = DELTA, GAMMA = GAMMA, R0 = R0, RK = RK, bp = tau.bp, 
        test.name = "Johansen-Procedure")
}

##
## bh6lrtest
##
bh6lrtest <- function (z, H, r, r1, conv.val=0.0001, max.iter=50) 
{
    if (!inherits(z, "ca.jo")) {
        stop("\nPlease, provide object of class 'ca.jo' as 'z'.\n")
    }
    if (r >= z@P || r < 1) {
        stop("\nCount of cointegrating relationships is out of allowable range.\n")
    }
    if (z@ecdet == "none") {
        P <- z@P
    } else {
        P <- z@P + 1
    }
    r <- as.integer(r)
    H <- as.matrix(H)
    if (!(nrow(H) == P)) {
        stop("\nRow number of 'H' is unequal to VAR order.\n")
    }
    s <- ncol(H)
    r2 <- r - r1
    lambda <- z@lambda
    type <- "Estimation and testing under partly known beta"
    N <- nrow(z@Z0)
    M00 <- crossprod(z@Z0)/N
    M11 <- crossprod(z@Z1)/N
    MKK <- crossprod(z@ZK)/N
    M01 <- crossprod(z@Z0, z@Z1)/N
    M0K <- crossprod(z@Z0, z@ZK)/N
    MK0 <- crossprod(z@ZK, z@Z0)/N
    M10 <- crossprod(z@Z1, z@Z0)/N
    M1K <- crossprod(z@Z1, z@ZK)/N
    MK1 <- crossprod(z@ZK, z@Z1)/N
    M11inv <- solve(M11)
    S00 <- M00 - M01 %*% M11inv %*% M10
    S0K <- M0K - M01 %*% M11inv %*% M1K
    SK0 <- MK0 - MK1 %*% M11inv %*% M10
    SKK <- MKK - MK1 %*% M11inv %*% M1K
    Dtemp <- chol(t(H)%*%SKK%*%H, pivot = TRUE)
    pivot <- attr(Dtemp, "pivot")
    oo <- order(pivot)
    D <- t(Dtemp[, oo])
    Dinv <- solve(D)
    valeigen <- eigen(Dinv %*% t(H) %*% SK0 %*% solve(S00) %*% S0K %*% H %*% t(Dinv))
    beta1 <- H%*%valeigen$vectors[,1:r1]
    i <- 0
    last <- 1
    diff <- 1
    while(diff > conv.val){
      S00.b1 <- S00 - S0K%*%beta1%*%solve(t(beta1)%*%SKK%*%beta1)%*%t(beta1)%*%SK0
      S0K.b1 <- S0K - S0K%*%beta1%*%solve(t(beta1)%*%SKK%*%beta1)%*%t(beta1)%*%SKK
      SK0.b1 <- SK0 - SKK%*%beta1%*%solve(t(beta1)%*%SKK%*%beta1)%*%t(beta1)%*%SK0
      SKK.b1 <- SKK - SKK%*%beta1%*%solve(t(beta1)%*%SKK%*%beta1)%*%t(beta1)%*%SKK
      valeigen <- eigen(SKK.b1)
      C <- valeigen$vectors[ ,1:(P-r1)]%*%diag(1/sqrt(valeigen$values[1:(P-r1)]))
      valeigen <- eigen(t(C)%*%SK0.b1%*%solve(S00.b1)%*%S0K.b1%*%C)
      lambda.res <- valeigen$values
      diff <- t(lambda.res-last)%*%(lambda.res-last)
      last <- lambda.res
      beta2 <- C%*%valeigen$vectors[,1:r2]
      S00.b2 <- S00 - S0K%*%beta2%*%solve(t(beta2)%*%SKK%*%beta2)%*%t(beta2)%*%SK0
      S0K.b2 <- S0K - S0K%*%beta2%*%solve(t(beta2)%*%SKK%*%beta2)%*%t(beta2)%*%SKK
      SK0.b2 <- SK0 - SKK%*%beta2%*%solve(t(beta2)%*%SKK%*%beta2)%*%t(beta2)%*%SK0
      SKK.b2 <- SKK - SKK%*%beta2%*%solve(t(beta2)%*%SKK%*%beta2)%*%t(beta2)%*%SKK
      valeigen <- eigen(t(H)%*%SKK.b2%*%H)
      C <- valeigen$vectors[ ,1:s]%*%diag(1/sqrt(valeigen$values[1:s]))
      valeigen <- eigen(t(C)%*%t(H)%*%SK0.b2%*%solve(S00.b2)%*%S0K.b2%*%H%*%C)
      beta1 <- H%*%valeigen$vectors[,1:r1]
      i <- i + 1
      if(i>max.iter){
        warning("\nNo convergence, used last iterations values.\n")
        break
      }
    }
    Vorg <- cbind(beta1, beta2)
    V <- Vorg
    idx <- ncol(V)
    V <- sapply(1:idx, function(x) V[, x]/V[1, x])
    W <- S0K %*% V %*% solve(t(V) %*% SKK %*% V)
    PI <- W %*% t(V)
    DELTA <- S00 - S0K %*% V %*% solve(t(V) %*% SKK %*% V) %*% t(V) %*% SK0
    GAMMA <- M01 %*% M11inv - PI %*% MK1 %*% M11inv
    Dtemp <- chol(t(beta1)%*%SKK%*%beta1, pivot = TRUE)
    pivot <- attr(Dtemp, "pivot")
    oo <- order(pivot)
    D <- t(Dtemp[, oo])
    Dinv <- solve(D)
    valeigen <- eigen(Dinv %*% t(beta1) %*% SK0 %*% solve(S00) %*% S0K %*% beta1 %*% t(Dinv))
    rho <- valeigen$values
    teststat <- N*(sum(log(1-rho[1:r1])) + sum(log(1-lambda.res[1:r2])) - sum(log(1-lambda[1:r])))
    df <- (P - s - r2)*r1
    pval <- c(1 - pchisq(teststat, df), df)
    new("cajo.test", Z0 = z@Z0, Z1 = z@Z1, ZK = z@ZK, ecdet = z@ecdet, H = H, A = NULL, B = NULL, type = type, teststat = teststat, pval = pval, lambda = lambda.res, Vorg = Vorg, V = V, W = W, PI = PI, DELTA = DELTA, DELTA.bb = NULL, DELTA.ab = NULL, DELTA.aa.b = NULL, GAMMA = GAMMA, test.name = "Johansen-Procedure")
}

##
## lttest
##
lttest <- function(z, r){
  if(!inherits(z, "ca.jo")){
    stop("\nObject 'x' must be of class 'ca.jo'\n")
  }
  r <- as.integer(r)
  if(r >= z@P || r < 1){
    stop("\nProvide number of cointegration vectors in valid range.\n")
  }
  idx <- r + 1
  df <- length(idx:z@P)
  N <- nrow(z@Z0)
  test1 <- ca.jo(z@x, ecdet = "const", K=z@lag, season=z@season, dumvar=z@dumvar)
  lambda1 <- test1@lambda
  test2 <- ca.jo(z@x, ecdet = "none", K=z@lag, season=z@season, dumvar=z@dumvar)
  lambda2 <- test2@lambda
  teststat <- -N*sum(log((1-lambda1[idx:z@P])/(1-lambda2[idx:z@P])))
  pval <- 1 - pchisq(teststat, df)
  lttest <- as.matrix(t(c(teststat, pval)))
  colnames(lttest) <- c("test statistic", "p-value")
  rownames(lttest) <- "LR test"
  cat("LR-test for no linear trend\n")
  cat("\n")
  cat(paste("H0: H*2(r<=", r, ")\n", sep=""))
  cat(paste("H1: H2(r<=", r, ")\n", sep=""))
  cat("\n")
  cat("Test statistic is distributed as chi-square\n")
  cat(paste("with", df, "degress of freedom\n", sep=" "))
  print(round(lttest, 2))
}

##
## Setting methods for classes
##
show.urca <- function(object){
  title <- paste("#", object@test.name, "Unit Root / Cointegration Test #", sep=" ")
  row <- paste(rep("#", nchar(title)), collapse="")
  cat("\n")
  cat(row, "\n")
  cat(title, "\n")
  cat(row, "\n")
  cat("\n")
  cat("The value of the test statistic is:", round(object@teststat, 4), "\n")
  cat('\n')
}

setMethod("show", "ur.kpss", show.urca)
setMethod("show", "ca.jo", show.urca)
setMethod("show", "cajo.test", show.urca)
setMethod("show", "ca.po", show.urca)
setMethod("show", "ur.pp", show.urca)
setMethod("show", "ur.df", show.urca)
setMethod("show", "ur.sp", show.urca)
setMethod("show", "ur.za", show.urca)
setMethod("show", "ur.ers", show.urca)

setMethod("show", "sumurca", function(object){
  if(object@classname=="ur.za"){
    title <- paste("#", object@test.name, "Unit Root Test #", sep=" ")
    row <- paste(rep("#", nchar(title)), collapse="")
    cat("\n")
    cat(row, "\n")
    cat(title, "\n")
    cat(row, "\n")
    cat("\n")
    print(summary(object@testreg))
    cat('\n')
    cat('Teststatistic:', round(object@teststat, 4), '\n')
    cat('Critical values: 0.01=', object@cval[1], '0.05=', object@cval[2], '0.1=', object@cval[3], '\n')
    cat('\n')
    cat('Potential break point at position:', object@bpoint, '\n')
    cat('\n')
    invisible(object)
  }else if(object@classname=="ur.sp"){
    title <- paste("#", object@test.name, "Unit Root Test #", sep=" ")
    row <- paste(rep("#", nchar(title)), collapse="")
    cat("\n")
    cat(row, "\n")
    cat(title, "\n")
    cat(row, "\n")
    cat("\n")
    print(object@testreg)
    cat('\n')
    cat('Value of test-statistic is:', round(object@teststat, 4), '\n')
    cat('Critical value for a significance level of', object@signif, '\n')
    cat('is:', object@cval, '\n')
    cat('\n')
    invisible(object)
  }else if(object@classname=="ur.pp"){
    title <- paste("#", object@test.name, "Unit Root Test #", sep=" ")
    row <- paste(rep("#", nchar(title)), collapse="")
    cat("\n")
    cat(row, "\n")
    cat(title, "\n")
    cat(row, "\n")
    cat("\n")
    cat('Test regression', object@model, '\n')
    cat('\n')
    print(object@testreg)
    cat('\n')
    cat('Value of test-statistic, type:', object@type,' is:', round(object@teststat, 4), '\n')
    cat('\n')
    print(object@auxstat)
    cat('\n')
    if(identical(object@type, "Z-tau")){
      cat('Critical values for Z statistics: \n')
      print(object@cval)
      cat('\n')
    }
    invisible(object)
  }else if(object@classname=="ur.df"){
    title <- paste("#", object@test.name, "Unit Root Test #", sep=" ")
    row <- paste(rep("#", nchar(title)), collapse="")
    cat("\n")
    cat(row, "\n")
    cat(title, "\n")
    cat(row, "\n")
    cat("\n")
    cat('Test regression', object@model, '\n')
    cat('\n')
    print(object@testreg)
    cat('\n')
    cat('Value of test-statistic is:', round(object@teststat, 4), '\n')
    cat('\n')
    cat('Critical values for test statistics: \n')
    print(object@cval)
    cat('\n')
    invisible(object)
  }else if(object@classname=="ca.po"){
    title <- paste("#", object@test.name, "Unit Root Test #", sep=" ")
    row <- paste(rep("#", nchar(title)), collapse="")
    cat("\n")
    cat(row, "\n")
    cat(title, "\n")
    cat(row, "\n")
    cat("\n")
    cat("Test of type", object@type, "\ndetrending of series", object@model, "\n")
    cat("\n")
    print(object@testreg)
    cat('\n')
    cat('Value of test-statistic is:', round(object@teststat, 4), '\n')
    cat('\n')
    cat('Critical values of', object@type, "are:\n")
    print(object@cval)
    cat('\n')
    invisible(object)
    }else if(object@classname=="ur.kpss"){
      title <- paste("#", object@test.name, "Unit Root Test #", sep=" ")
      row <- paste(rep("#", nchar(title)), collapse="")
      cat("\n")
      cat(row, "\n")
      cat(title, "\n")
      cat(row, "\n")
      cat("\n")
      cat('Test is of type:', object@type, 'with', object@lag, 'lags. \n')
      cat('\n')
      cat('Value of test-statistic is:', round(object@teststat, 4), '\n')
      cat('\n')
      cat('Critical value for a significance level of: \n')
      print(object@cval)
      cat('\n')
      invisible(object)
    }else if(object@classname=="cajo.test"){
      title <- paste("#", object@test.name, "#", sep=" ")
      row <- paste(rep("#", nchar(title)), collapse="")
      cat("\n")
      cat(row, "\n")
      cat(title, "\n")
      cat(row, "\n")
      cat("\n")
      cat(object@type, "\n")
      cat("\n")
      cat("The VECM has been estimated subject to: \n")
      cat("beta=H*phi and/or alpha=A*psi\n")
      if(!is.null(object@H)){
        cat("\n")
        print(object@H)
        cat("\n")
      }
      if(!is.null(object@A)){
        cat("\n")
        print(object@A)
        cat("\n")
      }
      cat("Eigenvalues of restricted VAR (lambda):\n")
      print(round(object@lambda, 4))
      cat('\n')
      cat("The value of the likelihood ratio test statistic:\n")
      cat(round(object@teststat, 2), "distributed as chi square with", object@pval[2], "df.\n")
      cat("The p-value of the test statistic is:", round(object@pval[1], 2), "\n")
      cat("\n")
      cat("Eigenvectors, normalised to first column\n")
      cat("of the restricted VAR:\n")
      cat("\n")
      print(round(object@V, 4))
      cat("\n")
      cat("Weights W of the restricted VAR:\n")
      cat("\n")
      print(round(object@W, 4))
      cat("\n")
      invisible(object)
    }else if(object@classname=="ca.jo"){
      title <- paste("#", object@test.name, "#", sep=" ")
      row <- paste(rep("#", nchar(title)), collapse="")
      cat("\n")
      cat(row, "\n")
      cat(title, "\n")
      cat(row, "\n")
      cat("\n")
      cat("Test type:", object@type, ",", object@model, "\n")
      cat("\n")
      cat("Eigenvalues (lambda):\n")
      print(object@lambda)
      cat('\n')
      if(!(is.null(object@cval))){
        res1 <- as.matrix(round(object@teststat, 2))
        colnames(res1) <- "test"
        result <- cbind(res1, object@cval)
        cat("Values of teststatistic and critical values of test:\n")
        cat("\n")
        print(result)
        cat("\n")
      }else{
        cat("Values of test statistic\n")
        cat("\n")
        result <- as.matrix(object@teststat)
        rownames(result) <- c(paste("r <= ", (object@P-1):1, " |",sep=""), "r = 0  |")
        print(result)
        cat("\n")
        invisible(object)
      }
      cat("Eigenvectors, normalised to first column:\n")
      cat("(These are the cointegration relations)\n")
      cat("\n")
      print(object@V)
      cat("\n")
      cat("Weights W:\n")
      cat("(This is the loading matrix)\n")
      cat("\n")
      print(object@W)
      cat("\n")
      invisible(object)
    }else if(object@classname=="ur.ers"){
      title <- paste("#", object@test.name, "Unit Root Test #", sep=" ")
      row <- paste(rep("#", nchar(title)), collapse="")
      cat("\n")
      cat(row, "\n")
      cat(title, "\n")
      cat(row, "\n")
      cat("\n")
      cat("Test of type", object@type, "\ndetrending of series", object@model, "\n")
      cat("\n")
      if(!is.null(object@testreg)){
        print(object@testreg)
        cat('\n')
      }
      cat('Value of test-statistic is:', round(object@teststat, 4), '\n')
      cat('\n')
      cat('Critical values of', object@type, "are:\n")
      print(object@cval)
      cat('\n')
      invisible(object)
    }
})

##
## Schmidt-Phillips Test
##
ur.sp <- function(y, type=c("tau", "rho"), pol.deg=c(1, 2, 3, 4), signif=c(0.01, 0.05, 0.1)){
  y <- na.omit(as.vector(y))
  type <- match.arg(type)
  signif <- signif[1]
  signif.val <- c(0.01, 0.05, 0.1)
  if(!(signif %in% signif.val)){
    warning("\nPlease, provide as signif one of c(0.01, 0.05, 0.1); signif=0.01 used")
  signif <- 0.01}
  pol.deg <- pol.deg[1]
  if(!(pol.deg %in% c(1:4))){
    warning("\nPlease, provide as polynomial degree one of c(1, 2, 3, 4); po.deg=1 used")
  pol.deg <- 1}
  n <- length(y)
  lag <- trunc(12*(n/100)^0.25)
  idx <- 1:lag
  trend1 <- 1:n
  y.diff <- diff(y)
  if(pol.deg==1){
    yi.hat <- (y[n]-y[1])/(n-1)
    phi.y <- y[1]-yi.hat
    S.hat <- y - phi.y - yi.hat*trend1
    S.hat.l1 <- S.hat[-n]
    test.reg <- summary(lm(y.diff ~ 1 + S.hat.l1))
    sp.data <- data.frame(cbind(y[2:n],  y[1:(n-1)], trend1[2:n]))
    colnames(sp.data) <- c("y", "y.lagged", "trend.exp1")
    corr.reg <- summary(lm(sp.data))
    res <- residuals(corr.reg)
    sig.eps <- (1/n)*sum(res^2)
    coprods <- sapply(idx, function(x) t(res[-c(1:x)])%*%(res[-c((n-x):(n-1))]))
    weights <- (2*(lag-idx)/lag)^2
    sig <- sig.eps + (2/n)*(t(weights)%*%coprods)
    omega2.hat <- sig.eps/sig
  }else if(pol.deg==2){
    trend2 <- trend1^2
    S.hat <- c(0, cumsum(residuals(summary(lm(y.diff ~ trend1[2:n])))))
    test.reg <- summary(lm(y.diff ~ S.hat[-n] + trend1[2:n]))
    sp.data <- data.frame(cbind(y[2:n],  y[1:(n-1)], trend1[2:n], trend2[2:n]))
    colnames(sp.data) <- c("y", "y.lagged", "trend.exp1", "trend.exp2")
    corr.reg <- summary(lm(sp.data))
    res <- residuals(corr.reg)
    sig.eps <- (1/n)*sum(res^2)
    coprods <- sapply(idx, function(x) t(res[-c(1:x)])%*%(res[-c((n-x):(n-1))]))
    weights <- (2*(lag-idx)/lag)^2
    sig <- sig.eps + (2/n)*(t(weights)%*%coprods)
    omega2.hat <- sig.eps/sig
  }else if(pol.deg==3){
    trend2 <- trend1^2
    trend3 <- trend1^3    
    S.hat <- c(0, cumsum(residuals(summary(lm(y.diff ~ trend1[2:n] + trend2[2:n])))))
    test.reg <- summary(lm(y.diff ~ S.hat[-n] + trend1[2:n] + trend2[2:n]))
    sp.data <- data.frame(cbind(y[2:n],  y[1:(n-1)], trend1[2:n], trend2[2:n], trend3[2:n]))
    colnames(sp.data) <- c("y", "y.lagged", "trend.exp1", "trend.exp2", "trend.exp3")
    corr.reg <- summary(lm(sp.data))
    res <- residuals(corr.reg)
    sig.eps <- (1/n)*sum(res^2)
    coprods <- sapply(idx, function(x) t(res[-c(1:x)])%*%(res[-c((n-x):(n-1))]))
    weights <- (2*(lag-idx)/lag)^2
    sig <- sig.eps + (2/n)*(t(weights)%*%coprods)
    omega2.hat <- sig.eps/sig
  }else if(pol.deg==4){
    trend2 <- trend1^2
    trend3 <- trend1^3
    trend4 <- trend1^4
    S.hat <- c(0, cumsum(residuals(summary(lm(y.diff ~ trend1[2:n] + trend2[2:n] + trend3[2:n])))))
    test.reg <- summary(lm(y.diff ~ S.hat[-n] + trend1[2:n] + trend2[2:n] + trend3[2:n]))
    sp.data <- data.frame(cbind(y[2:n],  y[1:(n-1)], trend1[2:n], trend2[2:n], trend3[2:n], trend4[2:n]))
    colnames(sp.data) <- c("y", "y.lagged", "trend.exp1", "trend.exp2", "trend.exp3", "trend.exp4")
    corr.reg <- summary(lm(sp.data))
    res <- residuals(corr.reg)
    sig.eps <- (1/n)*sum(res^2)
    coprods <- sapply(idx, function(x) t(res[-c(1:x)])%*%(res[-c((n-x):(n-1))]))
    weights <- (2*(lag-idx)/lag)^2
    sig <- sig.eps + (2/n)*(t(weights)%*%coprods)
    omega2.hat <- sig.eps/sig
  }
  if(type=="rho"){
    rho <- n*coef(test.reg)[2,1]
    teststat <- rho/omega2.hat
    cval <- .spcv(obs=n, type="rho", pol.deg=pol.deg, signif=signif)
  }else if(type=="tau"){
    tau <- coef(test.reg)[2,3]
    teststat <- tau/sqrt(omega2.hat)
    cval <- .spcv(obs=n, type="tau", pol.deg=pol.deg, signif=signif)
  }
  new("ur.sp", y=y, type=type, polynomial=as.integer(pol.deg), teststat=as.numeric(teststat), cval=cval, signif=signif, res=res, testreg=corr.reg, test.name="Schmidt-Phillips")
}

##
## cajools
##
cajools <- function(z, reg.number=NULL)
{
  if ((!inherits(z, "ca.jo")) && (!inherits(z, "cajo.test"))) {
    stop("\nPlease, provide object of class 'ca.jo' or 'cajo.test' as 'z'.\n")
  }
  P <- z@P
  data.mat <- data.frame(z@Z0, z@Z1, z@ZK)
  text <- colnames(data.mat)[-c(1:P)]
  text1 <- paste(text, "", sep="+", collapse="")
  text2 <- paste("~", substr(text1, 1, nchar(text1)-1))
  if (!is.null(reg.number)) {
    reg.number <- as.integer(reg.number)
    if (reg.number > ncol(z@Z0) || reg.number < 1) {
      stop("\nPlease, provide a valid number of the regression within \n the VECM, numbering from 1st to last row.\n")
    }
    form1 <- formula(paste("z@Z0[, reg.number]", text2, "-1"))
    return(lm(substitute(form1), data=data.mat))
  } else if (is.null(reg.number)) {
    form1 <- formula(paste("z@Z0", text2, "-1"))
    return(lm(substitute(form1), data=data.mat))
  } 
}

##
## Augmented-Dickey-Fuller Test
##
ur.df <- function (y, type = c("none", "drift", "trend"), lags = 1, selectlags = c("Fixed", "AIC", "BIC")) 
{
    selectlags<-match.arg(selectlags)
    type <- match.arg(type)
    if (ncol(as.matrix(y)) > 1) 
        stop("\ny is not a vector or univariate time series.\n")
    if (any(is.na(y))) 
        stop("\nNAs in y.\n")
    y <- as.vector(y)
    lag <- as.integer(lags)
    if (lag < 0) 
        stop("\nLags must be set to an non negative integer value.\n")
    CALL <- match.call()
    DNAME <- deparse(substitute(y))
    x.name <- deparse(substitute(y))
    lags <- lags + 1
    z <- diff(y)
    n <- length(z)
    x <- embed(z, lags)
    z.diff <- x[, 1]
    z.lag.1 <- y[lags:n]
    tt <- lags:n
    if (lags > 1) {
    if(selectlags!="Fixed"){
      critRes<-rep(NA, lags)
      for(i in 2:(lags)){
          z.diff.lag = x[, 2:i]
	  if (type == "none") 
            result <- lm(z.diff ~ z.lag.1 - 1 + z.diff.lag)
	  if (type == "drift") 
            result <- lm(z.diff ~ z.lag.1 + 1 + z.diff.lag)  
	  if (type == "trend") 
            result <- lm(z.diff ~ z.lag.1 + 1 + tt + z.diff.lag)
	  critRes[i]<-AIC(result, k = switch(selectlags, "AIC" = 2, "BIC" = log(length(z.diff))))
	}
	lags<-which.min(critRes)
    }
        z.diff.lag = x[, 2:lags]
        if (type == "none") {
            result <- lm(z.diff ~ z.lag.1 - 1 + z.diff.lag)
            tau <- coef(summary(result))[1, 3]
            teststat <- as.matrix(tau)
            colnames(teststat) <- 'tau1'
          }
        if (type == "drift") {
            result <- lm(z.diff ~ z.lag.1 + 1 + z.diff.lag)
            tau <- coef(summary(result))[2, 3]
            phi1.reg <- lm(z.diff ~ -1 + z.diff.lag)
            phi1 <- anova(phi1.reg, result)$F[2]
            teststat <- as.matrix(t(c(tau, phi1)))
            colnames(teststat) <- c('tau2', 'phi1')
          }
        if (type == "trend") {
            result <- lm(z.diff ~ z.lag.1 + 1 + tt + z.diff.lag)
            tau <- coef(summary(result))[2, 3]
            phi2.reg <- lm(z.diff ~ -1 + z.diff.lag)
            phi3.reg <- lm(z.diff ~ z.diff.lag)
            phi2 <- anova(phi2.reg, result)$F[2]
            phi3 <- anova(phi3.reg, result)$F[2]
            teststat <- as.matrix(t(c(tau, phi2, phi3)))
            colnames(teststat) <- c('tau3', 'phi2', 'phi3')
          }
    }
    else {
        if (type == "none") {
            result <- lm(z.diff ~ z.lag.1 - 1)
            tau <- coef(summary(result))[1, 3]
            teststat <- as.matrix(tau)
            colnames(teststat) <- 'tau1'
        }
        if (type == "drift") {
            result <- lm(z.diff ~ z.lag.1 + 1)
            phi1.reg <- lm(z.diff ~ -1)
            phi1 <- anova(phi1.reg, result)$F[2]
            tau <- coef(summary(result))[2, 3]
            teststat <- as.matrix(t(c(tau, phi1)))
            colnames(teststat) <- c('tau2', 'phi1')
        }
        if (type == "trend") {
            result <- lm(z.diff ~ z.lag.1 + 1 + tt)
            phi2.reg <- lm(z.diff ~ -1)
            phi3.reg <- lm(z.diff ~ 1)
            phi2 <- anova(phi2.reg, result)$F[2]
            phi3 <- anova(phi3.reg, result)$F[2]
            tau <- coef(summary(result))[2, 3]
            teststat <- as.matrix(t(c(tau, phi2, phi3)))
            colnames(teststat) <- c('tau3', 'phi2', 'phi3')
        }
    }
    rownames(teststat) <- 'statistic'
    testreg <- summary(result)
    res <- residuals(testreg)
    if(n < 25)
      rowselec <- 1
    if(25 <= n & n < 50)
      rowselec <- 2
    if(50 <= n & n < 100)
      rowselec <- 3
    if(100 <= n & n < 250)
      rowselec <- 4
    if(250 <= n & n < 500)
      rowselec <- 5
    if(n >= 500)
      rowselec <- 6
    if (type == "none"){ 
        cval.tau1 <- rbind(
                           c(-2.66, -1.95, -1.60),
                           c(-2.62, -1.95, -1.61),
                           c(-2.60, -1.95, -1.61),
                           c(-2.58, -1.95, -1.62),
                           c(-2.58, -1.95, -1.62),
                           c(-2.58, -1.95, -1.62))
        cvals <- t(cval.tau1[rowselec, ])
        testnames <- 'tau1'
      }
    if (type == "drift"){ 
        cval.tau2 <- rbind(
                           c(-3.75, -3.00, -2.63),
                           c(-3.58, -2.93, -2.60),
                           c(-3.51, -2.89, -2.58),
                           c(-3.46, -2.88, -2.57),
                           c(-3.44, -2.87, -2.57),
                           c(-3.43, -2.86, -2.57))
        cval.phi1 <- rbind(
                           c(7.88, 5.18, 4.12),
                           c(7.06, 4.86, 3.94),
                           c(6.70, 4.71, 3.86),
                           c(6.52, 4.63, 3.81),
                           c(6.47, 4.61, 3.79),
                           c(6.43, 4.59, 3.78))
        cvals <- rbind(
                      cval.tau2[rowselec, ],
                      cval.phi1[rowselec, ])
        testnames <- c('tau2', 'phi1')
      }
    if (type == "trend"){ 
        cval.tau3 <- rbind(
                           c(-4.38, -3.60, -3.24),
                           c(-4.15, -3.50, -3.18),
                           c(-4.04, -3.45, -3.15),
                           c(-3.99, -3.43, -3.13),
                           c(-3.98, -3.42, -3.13),
                           c(-3.96, -3.41, -3.12))
        cval.phi2 <- rbind(
                           c(8.21, 5.68, 4.67),
                           c(7.02, 5.13, 4.31),
                           c(6.50, 4.88, 4.16),
                           c(6.22, 4.75, 4.07),
                           c(6.15, 4.71, 4.05),
                           c(6.09, 4.68, 4.03))
        cval.phi3 <- rbind(
                           c(10.61, 7.24, 5.91),
                           c( 9.31, 6.73, 5.61),
                           c( 8.73, 6.49, 5.47),
                           c( 8.43, 6.49, 5.47),
                           c( 8.34, 6.30, 5.36),
                           c( 8.27, 6.25, 5.34))  
        cvals <- rbind(
                      cval.tau3[rowselec, ],
                      cval.phi2[rowselec, ],
                      cval.phi3[rowselec, ])

        testnames <- c('tau3', 'phi2', 'phi3')
      }
    colnames(cvals) <- c("1pct", "5pct", "10pct")
    rownames(cvals) <- testnames
   
    new("ur.df", y = y, model = type, cval=cvals, lags=lag, teststat = teststat, testreg=testreg, res=res, test.name="Augmented Dickey-Fuller Test")
}

setMethod("plot", signature(x="ur.ers", y="missing"), function(x){
  if(is.null(x@testreg)){
    stop("No plot method for P-test available")}
  oldpar <- par(no.readonly=TRUE)
  on.exit(par(oldpar))
  par(mfrow=c(1,1))
  layout(matrix(c(1, 2, 3, 1, 2, 4), 3 , 2))
  suppressWarnings(plot.ts(diff(x@yd)[-c(1:x@lag)], main="Diagram of fit for test regression", sub=paste("detrending ", x@model, " and ", x@lag, " lagged differences used in test regression",  sep=""), ylab="Actual and fitted values", xlab=""))
  lines(diff(x@yd)[-c(1:x@lag)] - resid(x@testreg), col="seagreen")
  plot.ts(resid(x@testreg), main="Residuals", ylab="", xlab="")
  abline(h=0, col="red")
  acf(resid(x@testreg), main="Autocorrelations of Residuals")
  pacf(resid(x@testreg), main="Partial Autocorrelations of Residuals")
})

setMethod("plot", signature(x="ca.jo", y="missing"), function(x){
  oldpar <- par(no.readonly=TRUE)
  on.exit(par(oldpar))
  par(mfrow=c(2,1))
  if(x@P==nrow(x@V)){
    ci <- x@x%*%x@V
  }else{
    ci <- x@x%*%x@V[-(x@P+1),]
  }
  for( i in 1:x@P){
    plot.ts(x@x[,i], main=paste("Time series plot of y", i, sep=""), ylab="")
    plot.ts(ci[,i], main=paste("Cointegration relation of ", i, ". variable", sep=""), ylab="")
    if(interactive()){
      cat("\nType <Return> to continue: ")
      readline()
    }
  }
})

setMethod("plot", signature(x="ur.kpss", y="missing"), function(x){
  oldpar <- par(no.readonly=TRUE)
  on.exit(par(oldpar))
  par(mfrow=c(1,1))
  layout(matrix(c(1, 2, 1, 3), 2 , 2))
  plot.ts(x@res, main=paste("Residuals from test regression of type:", x@type, " with", x@lag, "lags", sep=" "), ylab="residuals", xlab="")
  abline(h=0, col="red")
  acf(x@res, main="Autocorrelations of Residuals")
  pacf(x@res, main="Partial Autocorrelations of Residuals")
})

setMethod("plot", signature(x="ca.po", y="missing"), function(x){
  oldpar <- par(no.readonly=TRUE)
  on.exit(par(oldpar))
  par(mfrow=c(1,1))
  if(x@type=="Pu"){
    layout(matrix(c(1, 2, 1, 3), 2 , 2))
    suppressWarnings(plot.ts(x@res[,1], main="Residuals of CI-regression for y1", sub=paste("detrending:", x@model, sep=" "), ylab="", xlab=""))
    abline(h=0, col="red")
    acf(x@res[,1], main="Autocorrelations of Residuals")
    pacf(x@res[,1], main="Partial Autocorrelations of Residuals")
  }else if(x@type=="Pz"){
    m <- ncol(x@z)
    for( i in 1:m){
      layout(matrix(c(1, 2, 1, 3), 2 , 2))
      suppressWarnings(plot.ts(x@res[,i], main=paste("Residuals of CI-regression with y", i, " as lhs", sep=""), sub=paste("detrending:", x@model, sep=" "), ylab="", xlab=""))
      abline(h=0, col="red")
      acf(x@res[,i], main="Autocorrelations of Residuals")
      pacf(x@res[,i], main="Partial Autocorrelations of Residuals")
      if(interactive()){
        cat("\nType <Return> to continue: ")
        readline()
      }
    }
  }     
})

setMethod("plot", signature(x="ur.pp", y="missing"), function(x){
  oldpar <- par(no.readonly=TRUE)
  on.exit(par(oldpar))
  par(mfrow=c(1,1))
  layout(matrix(c(1, 2, 3, 1, 2, 4), 3 , 2))
  plot.ts(x@y[-1], main=paste("Diagram of fit for model", x@model, sep=" "), ylab="Actual and fitted values", xlab="")
  lines(x@y - x@res, col="seagreen")
  plot.ts(x@res, main="Residuals", ylab="", xlab="")
  abline(h=0, col="red")
  acf(x@res, main="Autocorrelations of Residuals")
  pacf(x@res, main="Partial Autocorrelations of Residuals")
})

setMethod("plot", signature(x="ur.df", y="missing"), function(x){
  oldpar <- par(no.readonly=TRUE)
  on.exit(par(oldpar))
  par(mfrow=c(1,1))
  layout(matrix(c(1, 2, 1, 3), 2 , 2))
  plot.ts(x@res, main="Residuals", ylab="", xlab="")
  abline(h=0, col="red")
  acf(x@res, main="Autocorrelations of Residuals")
  pacf(x@res, main="Partial Autocorrelations of Residuals")
})

setMethod("plot", signature(x="ur.sp", y="missing"), function(x){
  oldpar <- par(no.readonly=TRUE)
  on.exit(par(oldpar))
  par(mfrow=c(1,1))
  layout(matrix(c(1, 2, 3, 1, 2, 4), 3 , 2))
  plot.ts(x@y[-1], main=paste("Diagram of fit for model with polynomial degree of ", x@polynomial, sep="") , ylab="Actual and fitted values", xlab="")
  lines(x@y[-1] - x@res, col="seagreen")
  plot.ts(x@res, main="Residuals", ylab="", xlab="")
  abline(h=0, col="red")
  acf(x@res, main="Autocorrelations of Residuals")
  pacf(x@res, main="Partial Autocorrelations of Residuals")
})

setMethod("plot", signature(x="ur.za", y="missing"), function(x){
  oldpar <- par(no.readonly=TRUE)
  on.exit(par(oldpar))
  par(mfrow=c(1,1))
  yvals <- sort(c(x@cval, x@tstats))
  n <- length(x@y)
  xvals <- pretty(1:n)
  plot.ts(x@tstats, main="Zivot and Andrews Unit Root Test", ylab="t-statistics for lagged endogenous variable", ylim=c(min(yvals), max(yvals)))
  abline(h=x@cval, col=c("red", "blue", "seagreen"))
  if(x@teststat < x@cval[3]){
    abline(v=x@bpoint, col="red", lty=2)}
  mtext(paste("Model type:", x@model, sep=" "), side=1, line=4)
  legend(x=n, y=max(yvals), c("1% c.v.", "2.5% c.v.", "5% c.v."), col=c("red", "blue", "seagreen"), xjust=1, yjust=1, lty=1, horiz=TRUE, cex=0.66, bty="n")
})
  

##
## bh5lrtest
##
bh5lrtest <- function (z, H, r) 
{
    if (!inherits(z, "ca.jo")) {
        stop("\nPlease, provide object of class 'ca.jo' as 'z'.\n")
    }
    if (r >= z@P || r < 1) {
        stop("\nCount of cointegrating relationships is out of allowable range.\n")
    }
    if (z@ecdet == "none") {
        P <- z@P
    } else {
        P <- z@P + 1
    }
    r <- as.integer(r)
    H <- as.matrix(H)
    if (!(nrow(H) == P)) {
        stop("\nRow number of 'H' is unequal to VAR order.\n")
    }
    if(ncol(H) > r - 1){
      stop("\nToo many columns in H for provided r.\n")
    }
    r1 <- ncol(H)
    r2 <- r - r1
    type <- "Estimation and testing under partly known beta"
    N <- nrow(z@Z0)
    M00 <- crossprod(z@Z0)/N
    M11 <- crossprod(z@Z1)/N
    MKK <- crossprod(z@ZK)/N
    M01 <- crossprod(z@Z0, z@Z1)/N
    M0K <- crossprod(z@Z0, z@ZK)/N
    MK0 <- crossprod(z@ZK, z@Z0)/N
    M10 <- crossprod(z@Z1, z@Z0)/N
    M1K <- crossprod(z@Z1, z@ZK)/N
    MK1 <- crossprod(z@ZK, z@Z1)/N
    M11inv <- solve(M11)
    S00 <- M00 - M01 %*% M11inv %*% M10
    S0K <- M0K - M01 %*% M11inv %*% M1K
    SK0 <- MK0 - MK1 %*% M11inv %*% M10
    SKK <- MKK - MK1 %*% M11inv %*% M1K
    S00.h <- S00 - S0K%*%H%*%solve(t(H)%*%SKK%*%H)%*%t(H)%*%SK0
    S0K.h <- S0K - S0K%*%H%*%solve(t(H)%*%SKK%*%H)%*%t(H)%*%SKK
    SK0.h <- SK0 - SKK%*%H%*%solve(t(H)%*%SKK%*%H)%*%t(H)%*%SK0
    SKK.h <- SKK - SKK%*%H%*%solve(t(H)%*%SKK%*%H)%*%t(H)%*%SKK
    valeigen <- eigen(SKK.h)
    C <- valeigen$vectors[ ,1:(P-r1)]%*%diag(1/sqrt(valeigen$values[1:(P-r1)]))
    valeigen <- eigen(t(C)%*%SK0.h%*%solve(S00.h)%*%S0K.h%*%C)
    PSI <- C%*%valeigen$vectors[,1:r2]
    Dtemp <- chol(t(H)%*%SKK%*%H, pivot = TRUE)
    pivot <- attr(Dtemp, "pivot")
    oo <- order(pivot)
    D <- t(Dtemp[, oo])
    Dinv <- solve(D)
    rho <- eigen(Dinv %*% t(H) %*% SK0 %*% solve(S00) %*% S0K %*% H %*% t(Dinv))
    Vorg <- cbind(H, PSI)
    idx <- ncol(PSI)
    PSI <- sapply(1:idx, function(x) PSI[, x]/PSI[1, x])
    V <- cbind(H, PSI)
    W <- S0K %*% V %*% solve(t(V) %*% SKK %*% V)
    PI <- W %*% t(V)
    DELTA <- S00 - S0K %*% V %*% solve(t(V) %*% SKK %*% V) %*% t(V) %*% SK0
    GAMMA <- M01 %*% M11inv - PI %*% MK1 %*% M11inv
    lambda.res <- valeigen$values
    lambda <- z@lambda
    teststat <- N *(sum(log(1 - rho$values[1:r1])) + sum(log(1-lambda.res[1:r2])) - sum(log(1 - lambda[1:r])))
    df <- (P - r)*r1
    pval <- c(1 - pchisq(teststat, df), df)
    new("cajo.test", Z0 = z@Z0, Z1 = z@Z1, ZK = z@ZK, ecdet = z@ecdet, H = H, A = NULL, B = NULL, type = type, teststat = teststat, pval = pval, lambda = lambda.res, Vorg = Vorg, V = V, W = W, PI = PI, DELTA = DELTA, DELTA.bb = NULL, DELTA.ab = NULL, DELTA.aa.b = NULL, GAMMA = GAMMA, test.name = "Johansen-Procedure")
}

##
## Phillips-Perron Test
##
ur.pp <- function(x, type=c("Z-alpha", "Z-tau"), model=c("constant", "trend"), lags=c("short", "long"), use.lag=NULL){
  x <- na.omit(as.vector(x))
  n <- length(x)
  y <- x[-1]
  y.l1 <- x[-n]
  n <- n-1
  lags <- match.arg(lags)
  model <- match.arg(model)
  type <- match.arg(type)
  if(!(is.null(use.lag))){
    lmax <- as.integer(use.lag)
    if(lmax < 0){
      warning("\nuse.lag has to be positive and integer; lags='short' used.")
      lmax <- trunc(4*(n/100)^0.25)}
  }else if(lags == "short"){
    lmax <- trunc(4*(n/100)^0.25)
  }else if(lags == "long"){
    lmax <- trunc(12*(n/100)^0.25)}
  if(model=="trend"){
    cval <- as.matrix(t(c(-3.9638-8.353/n-47.44/(n^2), -3.4126-4.039/n-17.83/(n^2), -3.1279-2.418/n-7.58/(n^2))))
    colnames(cval) <- c("1pct", "5pct", "10pct")
    rownames(cval) <- "critical values"
    model <- "with intercept and trend"
    trend <- (1:n) - n/2
    test.reg <- summary(lm(y ~ y.l1 + trend))
    res <- residuals(test.reg)
    my.tstat <- coef(test.reg)[1, 3]
    beta.tstat <- coef(test.reg)[3, 3]
    res <- residuals(test.reg)
    s <- 1/n*(sum(res^2))
    myybar <- (1/n^2)*sum((y-mean(y))^2)
    myy <- (1/n^2)*sum(y^2)
    mty <- (n^(-5/2))*(t(1:n)%*%y)
    my <- (n^(-3/2))*sum(y)
    idx <- 1:lmax
    coprods <- sapply(idx, function(l) t(res[-c(1:l)])%*%(res[-c((n-l+1):n)]))
    weights <- 1 - idx/(lmax+1)
    sig <- s + (2/n)*(t(weights)%*%coprods)
    lambda <- 0.5*(sig-s)
    lambda.prime <- lambda/sig
    M <- (1-n^(-2))*myy - 12*mty^2 + 12*(1 + 1/n)*mty*my - (4 + 6/n + 2/n^2)*my^2
    my.stat <- sqrt(s/sig)*my.tstat - lambda.prime*sqrt(sig)*my/(sqrt(M)*sqrt((M+my^2)))
    beta.stat <- sqrt(s/sig)*beta.tstat - lambda.prime*sqrt(sig)*(0.5*my - mty)/(sqrt(M/12)*sqrt(myybar))
    aux.stat <- as.matrix(c(round(my.stat, 4), round(beta.stat, 4)))
    rownames(aux.stat) <- c("Z-tau-mu", "Z-tau-beta")
    colnames(aux.stat) <- "aux. Z statistics"
    if(type=="Z-tau"){
      tstat <- (coef(test.reg)[2, 1] - 1)/coef(test.reg)[2, 2]
      teststat <- sqrt(s/sig)*tstat-lambda.prime*sqrt(sig)/sqrt(M)
    }else if(type=="Z-alpha"){
      alpha <- coef(test.reg)[2, 1]
      teststat <- n*(alpha-1)-lambda/M
      cval <- as.matrix(t(c(NA, NA, NA)))
    }
  }else if(model=="constant"){
    cval <- as.matrix(t(c(-3.4335-5.999/n-29.25/(n^2), -2.8621-2.738/n-8.36/(n^2), -2.5671-1.438/n-4.48/(n^2))))
    colnames(cval) <- c("1pct", "5pct", "10pct")
    rownames(cval) <- "critical values"
    model <- "with intercept"
    test.reg <- summary(lm(y ~ y.l1))
    my.tstat <- coef(test.reg)[1, 3]
    res <- residuals(test.reg)
    s <- 1/n*(sum(res^2))
    myybar <- (1/n^2)*sum((y-mean(y))^2)
    myy <- (1/n^2)*sum(y^2)
    my <- (n^(-3/2))*sum(y)
    idx <- 1:lmax
    coprods <- sapply(idx, function(l) t(res[-c(1:l)])%*%(res[-c((n-l+1):n)]))
    weights <- 1 - idx/(lmax+1)
    sig <- s + (2/n)*(t(weights)%*%coprods)
    lambda <- 0.5*(sig-s)
    lambda.prime <- lambda/sig
    my.stat <- sqrt(s/sig)*my.tstat + lambda.prime*sqrt(sig)*my/(sqrt(myy)*sqrt(myybar))
    aux.stat <- as.matrix(round(my.stat, 4))
    rownames(aux.stat) <- "Z-tau-mu"
    colnames(aux.stat) <- "aux. Z statistics"
    if(type=="Z-tau"){
      tstat <- (coef(test.reg)[2, 1] - 1)/coef(test.reg)[2, 2]
      teststat <- sqrt(s/sig)*tstat-lambda.prime*sqrt(sig)/sqrt(myybar)
    }else if(type=="Z-alpha"){
      alpha <- coef(test.reg)[2, 1]
      teststat <- n*(alpha-1)-lambda/myybar
      cval <- as.matrix(t(c(NA, NA, NA)))
    }
  }
  new("ur.pp", y=y, type=type, model=model, lag=as.integer(lmax), cval=cval, teststat=as.numeric(teststat), testreg=test.reg, auxstat=aux.stat, res=res, test.name="Phillips-Perron")
}

cajorls <- function(z, r = 1, reg.number = NULL){
    if ((!inherits(z, "ca.jo")) && (!inherits(z, "cajo.test"))) {
        stop("\nPlease, provide object of class 'ca.jo' or 'cajo.test' as 'z'.\n")
    }
    P <- ncol(z@Z0)
    r <- as.integer(r)
    if((r < 1) || (r > P - 1)){
      stop(paste("Please, provide a cointegration rank 'r' in the interval of 1 to", P - 1, ", in accordance with 'z@V'.\n", sep = " "))
    }
    beta <- matrix(z@V[, 1:r], ncol = r)
    C1 <- diag(r)
    C2 <- matrix(0, nrow = nrow(beta) - r, ncol = r)
    C <- rbind(C1, C2)
    betanorm <- beta %*% solve(t(C) %*% beta)
    ECT <- z@ZK %*% betanorm
    colnames(ECT) <- paste("ect", 1:r, sep = "")
    colnames(betanorm) <- colnames(ECT)
    rownames(betanorm) <- colnames(z@ZK)
    data.mat <- data.frame(z@Z0, ECT, z@Z1)
    text <- colnames(data.mat)[-c(1:P)]
    text1 <- paste(text, "", sep = "+", collapse = "")
    text2 <- paste("~", substr(text1, 1, nchar(text1) - 1))
    if (!is.null(reg.number)) {
        reg.number <- as.integer(reg.number)
        if (reg.number > ncol(z@Z0) || reg.number < 1) {
            stop("\nPlease, provide a valid number of the regression within \n the VECM, numbering from 1 to ", P, ".\n")
        }
        form1 <- formula(paste("z@Z0[, reg.number]", text2, "-1"))
        rlm <- lm(substitute(form1), data = data.mat)
    }
    else if (is.null(reg.number)) {
        form1 <- formula(paste("z@Z0", text2, "-1"))
        rlm <- lm(substitute(form1), data = data.mat)
    }
    return(list(rlm = rlm, beta = betanorm))
}

##
## ablrtest
##
ablrtest <- function(z, H, A, r){
    if(!inherits(z , "ca.jo")){
      stop("\nPlease, provide object of class 'ca.jo' as 'z'.\n")
    }
    r <- as.integer(r)
    A <- as.matrix(A)
    H <- as.matrix(H)
    if(!(nrow(A)==z@P)){
      stop("\nRow number of 'A' is unequal to VAR order.\n")
    }
    if(r >= z@P || r<1){
      stop("\nCount of cointegrating relationships is out of allowable range.\n")
    }
    if(z@ecdet == "none"){
      P <- z@P
    }else{
      P <- z@P + 1
    }
    if(!(nrow(H)==P)){
      stop("\nRow number of 'H' is unequal to VAR order.\n")
    }
    type <- "Estimation and testing under linear restrictions on alpha and beta"
    N <- nrow(z@Z0)
    B <- qr.Q(qr(A), complete=TRUE)[,-c(1:ncol(A))]
    M00 <- crossprod(z@Z0)/N
    M11 <- crossprod(z@Z1)/N
    MKK <- crossprod(z@ZK)/N
    M01 <- crossprod(z@Z0, z@Z1)/N
    M0K <- crossprod(z@Z0, z@ZK)/N
    MK0 <- crossprod(z@ZK, z@Z0)/N
    M10 <- crossprod(z@Z1, z@Z0)/N
    M1K <- crossprod(z@Z1, z@ZK)/N
    MK1 <- crossprod(z@ZK, z@Z1)/N
    M11inv <- solve(M11)
    S00 <- M00 - M01%*%M11inv%*%M10
    S0K <- M0K - M01%*%M11inv%*%M1K
    SK0 <- MK0 - MK1%*%M11inv%*%M10
    SKK <- MKK - MK1%*%M11inv%*%M1K
    Sab <- t(A)%*%S00%*%B
    Skb <- t(S0K)%*%B
    Sbb <- t(B)%*%S00%*%B
    Sbbinv <- solve(Sbb)
    RA <- z@R0%*%A - z@R0%*%B%*%Sbbinv%*%t(Sab)
    RK <- z@RK - z@R0%*%B%*%Sbbinv%*%t(Skb)
    Saa.b <- crossprod(RA, RA)/N
    Sak.b <- crossprod(RA, RK)/N
    Ska.b <- crossprod(RK, RA)/N
    Skk.b <- crossprod(RK, RK)/N
    Ctemp <- chol(t(H)%*%Skk.b%*%H, pivot=TRUE)
    pivot <- attr(Ctemp, "pivot")
    oo <- order(pivot)
    C <- t(Ctemp[,oo])
    Cinv <- solve(C)
    Saa.binv <- solve(Saa.b)
    valeigen <- eigen(Cinv%*%t(H)%*%Ska.b%*%Saa.binv%*%Sak.b%*%H%*%t(Cinv))
    lambda.res <- valeigen$values
    e <- valeigen$vector
    V <- H%*%t(Cinv)%*%e
    Vorg <- V
    idx <- 1:r
    V <- sapply(idx, function(x) V[ , x] / V[1,x])
    PHI <- solve(t(A)%*%A)%*%Sak.b%*%Vorg
    ALPHA <- as.matrix(A%*%PHI)
    ALPHAorg <- ALPHA
    ALPHA <- sapply(idx, function(x) ALPHA[ , x] * Vorg[1,x])
    PI <- ALPHA %*% t(V)
    GAMMA <- M01%*%M11inv - PI%*%MK1%*%M11inv
    DELTA.bb <- Sbb
    DELTA.ab <- Sab - t(A)%*%ALPHA%*%t(V)%*%Skb
    DELTA.aa.b <- Saa.b - t(A)%*%ALPHA%*%t(ALPHA)%*%A
    lambda <- z@lambda
    teststat <- N*sum(log((1-lambda.res[1:r])/(1-lambda[1:r])))
    df <- r*(z@P - ncol(A)) + r*(z@P - ncol(H))
    pval <- c(1-pchisq(teststat, df), df)
    new("cajo.test", Z0=z@Z0, Z1=z@Z1, ZK=z@ZK, ecdet=z@ecdet, H=H, A=A, B=B, type=type, teststat=teststat, pval=pval, lambda=lambda.res, Vorg=Vorg, V=V, W=ALPHA, PI=PI, DELTA=NULL, DELTA.bb=DELTA.bb, DELTA.ab=DELTA.ab, DELTA.aa.b=DELTA.aa.b, GAMMA=GAMMA, test.name="Johansen-Procedure")
}

##
## Zivot-Andrews Test
##
ur.za <- function(y, model=c("intercept", "trend", "both"), lag=NULL){
  y <- na.omit(as.vector(y))
  n <- length(y)
  model <- match.arg(model)
  if(is.null(lag)) lag <- 0
  lag <- as.integer(lag)
  if(length(lag) > 1 || lag < 0){
    warning("\nPlease, specify maximal number of lags for differenced series as positive integer; lag=1 is now used.")
    lag <- 1}
  datmat <- matrix(NA, n, lag + 3)
  if(n < ncol(datmat) + 2){
    stop("\nInsufficient number of obeservations.")}
  idx <- 1:(n-1)
  trend <- seq(1, n)
  datmat[,1] <- y
  datmat[,2] <- c(NA, y)[1:n]
  datmat[,3] <- trend
  datmat <- as.data.frame(datmat)
  colnames(datmat)[1:3] <- c("y", "y.l1", "trend")
  if(lag > 0){
    for(i in 1:lag){
      datmat[ , i + 3] <- c(rep(NA, i + 1), diff(y))[1:n]
    }
  colnames(datmat) <- c("y", "y.l1", "trend", paste("y.dl", 1:lag, sep=""))
  }
  if(model=="intercept"){
    roll <- function(z){
      du <- c(rep(0, z), rep(1, (n-z)))
      rollmat <- cbind(datmat, du)
      roll.reg <- coef(summary(lm(rollmat)))
      (roll.reg[2,1]-1.0)/roll.reg[2,2]
    }
    roll.stat <- sapply(idx, roll)
    cval <- c(-5.34, -4.8, -4.58)
    bpoint <- which.min(roll.stat)
    du <- c(rep(0, bpoint), rep(1, (n-bpoint)))
    testmat <- cbind(datmat, du)
    test.reg <- lm(testmat) 
  }else if(model=="trend"){
    roll <- function(z){
      dt <- c(rep(0, z), 1:(n-z))
      rollmat <- cbind(datmat, dt)
      roll.reg <- coef(summary(lm(rollmat)))
      (roll.reg[2,1]-1.0)/roll.reg[2,2]
    }
    roll.stat <- sapply(idx, roll)
    cval <- c(-4.93, -4.42, -4.11)
    bpoint <- which.min(roll.stat)
    dt <- c(rep(0, bpoint), 1:(n-bpoint))
    testmat <- cbind(datmat, dt)
    test.reg <- lm(testmat) 
  }else if(model=="both"){
    test.reg <- lm(datmat)
    roll <- function(z){
      du <- c(rep(0, z), rep(1, (n-z)))
      dt <- c(rep(0, z), 1:(n-z))
      rollmat <- cbind(datmat, du, dt)
      roll.reg <- coef(summary(lm(rollmat)))
      (roll.reg[2,1]-1.0)/roll.reg[2,2]
    }
    roll.stat <- sapply(idx, roll)
    cval <- c(-5.57, -5.08, -4.82)
    bpoint <- which.min(roll.stat)
    du <- c(rep(0, bpoint), rep(1, (n-bpoint)))
    dt <- c(rep(0, bpoint), 1:(n-bpoint))
    testmat <- cbind(datmat, du, dt)
    test.reg <- lm(testmat) 
  }
  teststat <- roll.stat[bpoint]
  new("ur.za", y=y, model=model, lag=lag, teststat=teststat, cval=cval, bpoint=bpoint, tstats=roll.stat, res=test.reg$residuals, testreg=test.reg, test.name="Zivot-Andrews")
}

##
## alphaols
##
alphaols <- function(z, reg.number = NULL) 
{
  if (!inherits(z, "ca.jo")) {
    stop("\nPlease, provide object of class 'ca.jo' as 'z'.\n")
  }
  RKV <- z@RK%*%z@V
  colnames(RKV) <- paste("V", colnames(z@RK), sep=".")
  P <- z@P
  data.mat <- data.frame(z@R0, RKV)
  text <- colnames(data.mat)[-c(1:P)]
  text1 <- paste(text, "", sep = "+", collapse = "")
  text2 <- paste("~", substr(text1, 1, nchar(text1) - 1))
  if (!is.null(reg.number)) {
    reg.number <- as.integer(reg.number)
    if (reg.number > ncol(z@R0) || reg.number < 1) {
      stop("\nPlease, provide a valid number of the regression within \n the VECM, numbering from 1st to last row.\n")
    }
    form1 <- formula(paste("z@R0[, reg.number]", text2, "-1"))
    return(lm(substitute(form1), data = data.mat))
  }
  else if (is.null(reg.number)) {
    form1 <- formula(paste("z@R0", text2, "-1"))
    return(lm(substitute(form1), data = data.mat))
  }
}

##
## KPSS-Test
##
ur.kpss <- function(y, type=c("mu", "tau"), lags=c("short", "long", "nil"), use.lag=NULL){
  y <- na.omit(as.vector(y))
  n <- length(y)
  type <- match.arg(type)
  lags <- match.arg(lags)
  if(!(is.null(use.lag))){
    lmax <- as.integer(use.lag)
    if(lmax < 0){
      warning("\nuse.lag has to be positive and integer; lags='short' used.")
    lmax <- trunc(4*(n/100)^0.25)}
  }else if(lags == "short"){
    lmax <- trunc(4*(n/100)^0.25)
  }else if(lags == "long"){
    lmax <- trunc(12*(n/100)^0.25)
  }else if(lags == "nil"){
    lmax <- 0
  }
  if(type=="mu"){
    cval <- as.matrix(t(c(0.347, 0.463, 0.574, 0.739)))
    colnames(cval) <- c("10pct", "5pct", "2.5pct", "1pct")
    rownames(cval) <- "critical values"
    res <- y - mean(y)
  }else if(type=="tau"){
    cval <- as.matrix(t(c(0.119, 0.146, 0.176, 0.216)))
    colnames(cval) <- c("10pct", "5pct", "2.5pct", "1pct")
    rownames(cval) <- "critical values"
    trend <- 1:n
    res <- residuals(lm(y ~ trend))
  }
  S <- cumsum(res)
  nominator <- sum(S^2)/n^2
  s2 <- sum(res^2)/n
  if(lmax == 0){
    denominator <- s2
  }else{
    index <- 1:lmax
    x.cov <- sapply(index, function(x) t(res[-c(1:x)])%*%res[-c((n-x+1):n)])
    bartlett <- 1-index/(lmax+1)
    denominator <- s2 + 2/n*t(bartlett)%*%x.cov
  }
  teststat <- nominator/denominator
  new("ur.kpss", y=y, type=type, lag=as.integer(lmax), teststat=as.numeric(teststat), cval=cval, res=res , test.name="KPSS") 
}

setMethod("summary", "ur.ers", function(object){
  return(new("sumurca", classname="ur.ers", test.name=object@test.name, testreg=object@testreg, teststat=object@teststat, cval=object@cval, bpoint=NULL, signif=NULL, model=object@model, type=object@type, auxstat=NULL, lag=NULL, H=NULL, A=NULL, lambda=NULL, pval=NULL, V=NULL, W=NULL, P=NULL))  
})

setMethod("summary", "ca.jo", function(object){
  return(new("sumurca", classname="ca.jo", test.name=object@test.name, testreg=NULL, teststat=object@teststat, cval=object@cval, bpoint=NULL, signif=NULL, model=object@model, type=object@type, auxstat=NULL, lag=NULL, H=NULL, A=NULL, lambda=object@lambda, pval=NULL, V=object@V, W=object@W, P=object@P))
})

setMethod("summary", "cajo.test", function(object){
  return(new("sumurca", classname="cajo.test", test.name=object@test.name, testreg=NULL, teststat=object@teststat, cval=NULL, bpoint=NULL, signif=NULL, model=NULL, type=object@type, auxstat=NULL, lag=NULL, H=object@H, A=object@A, lambda=object@lambda, pval=object@pval, V=object@V, W=object@W, P=NULL))
})

setMethod("summary", "ur.kpss", function(object){
  return(new("sumurca", classname="ur.kpss", test.name=object@test.name, testreg=NULL, teststat=object@teststat, cval=object@cval, bpoint=NULL, signif=NULL, model=NULL, type=object@type, auxstat=NULL, lag=object@lag, H=NULL, A=NULL, lambda=NULL, pval=NULL, V=NULL, W=NULL, P=NULL))
})

setMethod("summary", "ca.po", function(object){
  return(new("sumurca", classname="ca.po", test.name=object@test.name, testreg=object@testreg, teststat=object@teststat, cval=object@cval, bpoint=NULL, signif=NULL, model=object@model, type=object@type, auxstat=NULL, lag=NULL, H=NULL, A=NULL, lambda=NULL, pval=NULL, V=NULL, W=NULL, P=NULL))
})

setMethod("summary", "ur.pp", function(object){
  return(new("sumurca", classname="ur.pp", test.name=object@test.name, testreg=object@testreg, teststat=object@teststat, cval=object@cval, bpoint=NULL, signif=NULL, model=object@model, type=object@type, auxstat=object@auxstat, lag=NULL, H=NULL, A=NULL, lambda=NULL, pval=NULL, V=NULL, W=NULL, P=NULL))
})

setMethod("summary", "ur.df", function(object){
  return(new("sumurca", classname="ur.df", test.name=object@test.name, testreg=object@testreg, teststat=object@teststat, cval=object@cval, bpoint=NULL, signif=NULL, model=object@model, type=NULL, auxstat=NULL, lag=NULL, H=NULL, A=NULL, lambda=NULL, pval=NULL, V=NULL, W=NULL, P=NULL))
})

setMethod("summary", "ur.sp", function(object){
  return(new("sumurca", classname="ur.sp", test.name=object@test.name, testreg=object@testreg, teststat=object@teststat, cval=object@cval, bpoint=NULL, signif=object@signif, model=NULL, type=NULL, auxstat=NULL, lag=NULL, H=NULL, A=NULL, lambda=NULL, pval=NULL, V=NULL, W=NULL, P=NULL))
})

setMethod("summary", "ur.za", function(object){
  return(new("sumurca", classname="ur.za", test.name=object@test.name, testreg=object@testreg, teststat=object@teststat, cval=object@cval, bpoint=object@bpoint, signif=NULL, model=NULL, type=NULL, auxstat=NULL, lag=NULL, H=NULL, A=NULL, lambda=NULL, pval=NULL, V=NULL, W=NULL, P=NULL))
})

##
## blrtest
##
blrtest <- function(z, H, r){
  if(!inherits(z, "ca.jo")){
    stop("\nPlease, provide object of class 'ca.jo' as 'z'.\n")
  }
  if(r >= z@P || r < 1){
    stop("\nCount of cointegrating relationships is out of allowable range.\n")
  }
  if(z@ecdet == "none"){
    P <- z@P
  }else{
    P <- z@P + 1
  }
  r <- as.integer(r)
  H <- as.matrix(H)
  if(!(nrow(H)==P)){
    stop("\nRow number of 'H' is unequal to VAR order.\n")
  }
  type <- "Estimation and testing under linear restrictions on beta"
  N <- nrow(z@Z0)
  M00 <- crossprod(z@Z0)/N
  M11 <- crossprod(z@Z1)/N
  MKK <- crossprod(z@ZK)/N
  M01 <- crossprod(z@Z0, z@Z1)/N
  M0K <- crossprod(z@Z0, z@ZK)/N
  MK0 <- crossprod(z@ZK, z@Z0)/N
  M10 <- crossprod(z@Z1, z@Z0)/N
  M1K <- crossprod(z@Z1, z@ZK)/N
  MK1 <- crossprod(z@ZK, z@Z1)/N
  M11inv <- solve(M11)
  S00 <- M00 - M01%*%M11inv%*%M10
  S0K <- M0K - M01%*%M11inv%*%M1K
  SK0 <- MK0 - MK1%*%M11inv%*%M10
  SKK <- MKK - MK1%*%M11inv%*%M1K
  Ctemp <- chol(t(H)%*%SKK%*%H, pivot=TRUE)
  pivot <- attr(Ctemp, "pivot")
  oo <- order(pivot)
  C <- t(Ctemp[,oo])
  Cinv <- solve(C)
  S00inv <- solve(S00)
  valeigen <- eigen(Cinv%*%t(H)%*%SK0%*%S00inv%*%S0K%*%H%*%t(Cinv))
  e <- valeigen$vector
  V <- H%*%t(Cinv)%*%e
  Vorg <- V
  idx <- ncol(V)
  V <- sapply(1:idx, function(x) V[,x]/V[1,x])
  W <- S0K%*%V%*%solve(t(V)%*%SKK%*%V)
  PI <- W %*% t(V)
  DELTA <- S00 - S0K%*%V%*%solve(t(V)%*%SKK%*%V)%*%t(V)%*%SK0
  GAMMA <- M01%*%M11inv - PI%*%MK1%*%M11inv
  lambda.res <- valeigen$values
  lambda <- z@lambda
  teststat <- N*sum(log((1-lambda.res[1:r])/(1-lambda[1:r])))
  df <- r*(P - ncol(H))
  pval <- c(1-pchisq(teststat, df), df)
  new("cajo.test", Z0=z@Z0, Z1=z@Z1, ZK=z@ZK, ecdet=z@ecdet, H=H, A=NULL, B=NULL, type=type, teststat=teststat, pval=pval, lambda=lambda.res, Vorg=Vorg, V=V, W=W, PI=PI, DELTA=DELTA, DELTA.bb=NULL, DELTA.ab=NULL, DELTA.aa.b=NULL, GAMMA=GAMMA, test.name="Johansen-Procedure")
}

##
## plotres
##
plotres <- function (x){
  if (!inherits(x, "ca.jo"))
    stop("\nObject is not of class 'ca.jo' \n")
  resids <- x@Z0 - x@Z1%*%t(x@GAMMA) - x@ZK%*%t(x@PI)
  oldpar <- par(no.readonly = TRUE)
  on.exit(par(oldpar))
  par(mfrow = c(1, 1))
  for (i in 1:x@P) {
    layout(matrix(c(1, 2, 1, 3), 2, 2))
    plot.ts(resids[, i], main = paste("Residuals of ", i, ". VAR regression", sep = ""), ylab = "", xlab = "")
    abline(h = 0, col = "red")
    acf(x@R0[, i], main = "Autocorrelations of Residuals")
    pacf(x@R0[, i], main = "Partial Autocorrelations of Residuals")
    if (interactive()){
      cat("\nType <Return> to continue: ")
      readline()
    }
  }
}

##
## Elliott, Rothenberg and Stock-Test
##
ur.ers <- function(y, type=c("DF-GLS", "P-test"), model=c("constant", "trend"), lag.max=4){
  type <- match.arg(type)
  model <- match.arg(model)
  lag.max <- as.integer(lag.max)
  if(lag.max < 0){
    warning("\nlag.max bust be greater or equal to one and integer; setting lag.max=4")
  lag.max <- 4}
  lag.max <- lag.max+1
  idx <- 2:lag.max
  y <- na.omit(as.vector(y))
  nobs <- length(y)
  if(nobs < 50){
    rowsel <- 1
  }else if(nobs < 100){
    rowsel <- 2
  }else if(nobs <= 200){
    rowsel <- 3
  }else if(nobs > 200){
    rowsel <- 4}
  if(model=="constant"){
    ahat <- 1 - 7.0/nobs
    ya <- c(y[1], y[2:nobs]-ahat*y[1:(nobs-1)])
    za1 <- c(1, rep(1-ahat, nobs-1))
    yd.reg <- summary(lm(ya ~ -1 + za1))
    yd <- y - coef(yd.reg)[1]
  }else if(model=="trend"){
    ahat <- 1 - 13.5/nobs
    ya <- c(y[1], y[2:nobs]-ahat*y[1:(nobs-1)])
    za1 <- c(1, rep(1-ahat, nobs-1))
    trd <- 1:nobs
    za2 <- c(1, trd[2:nobs]-ahat*trd[1:(nobs-1)])
    yd.reg <- summary(lm(ya ~ -1 + za1 + za2))
    yd <- y - coef(yd.reg)[1] - coef(yd.reg)[2]*trd 
  }
  what <- function(x, z=y){
    z.l <-  z[1:(nobs-1)]
    z.diff <- diff(z)
    z.dlags <- embed(diff(z), x)[, -1]
    data.what <- data.frame(cbind(z.diff[-(1:(x-1))], z.l[-(1:(x-1))], z.dlags))
    bic <- BIC(lm(data.what))
    return(bic)
  }
  if(type=="P-test"){
    cvals.ptest <- array(c(1.87, 1.95, 1.91, 1.99, 2.97, 3.11, 3.17, 3.26, 3.91, 4.17, 4.33, 4.48, 4.22, 4.26, 4.05, 3.96, 5.72, 5.64, 5.66, 5.62, 6.77, 6.79, 6.86, 6.89), c(4, 3, 2))
    res <- residuals(yd.reg)
    if(model=="constant"){
      null.res <- c(0, diff(y))
      cvals <- as.matrix(t(cvals.ptest[rowsel, , 1]))
      model <- "with intercept"
    }else if(model=="trend"){
      null.res <- c(0, diff(y))
      null.res <- null.res - mean(null.res)
      cvals <- as.matrix(t(cvals.ptest[rowsel, , 2]))
      model <- "with intercept and trend"
    }
  sig.null <- sum(null.res^2)
  sig.res <- sum(res^2)
  if(lag.max > 1){
    bic <- sapply(idx, what, z=y)
    BIC.opt <- which.min(bic)+1
    y.l <-  y[1:(nobs-1)]
    y.diff <- diff(y)
    y.dlags <- embed(diff(y), BIC.opt)[, -1]
    data.what <- data.frame(cbind(y.diff[-(1:(BIC.opt-1))], y.l[-(1:(BIC.opt-1))], y.dlags))
    what.reg <- summary(lm(data.what))
    npar <- nrow(what.reg$coef)
    sumlc <- sum(what.reg$coef[3:npar,1])
    lag.max <- BIC.opt-1
  }else if(lag.max <= 1){
    y.diff <- diff(y)
    y.l <- y[1:(nobs-1)]
    what.reg <- summary(lm(y.diff ~ y.l))
    sumlc <- 0
    lag.max <- lag.max-1
  }
  what.sq <- what.reg$sigma^2/(1-sumlc)^2
  teststat <- (sig.res - ahat*sig.null)/what.sq
  test.reg <- NULL
  }else if(type=="DF-GLS"){
    if(model=="constant"){
      cvals <- as.matrix(t(c(-2.5658-1.960/nobs-10.04/(nobs**2),-1.9393-0.398/nobs,-1.6156-0.181/nobs)))
      model <- "with intercept"
    }else if(model=="trend"){
      cvals.dfgls.tau <- matrix(-1*c(3.77, 3.58, 3.46, 3.48, 3.19, 3.03, 2.93, 2.89, 2.89, 2.74, 2.64, 2.57), nrow=4, ncol=3)
      cvals <- as.matrix(t(cvals.dfgls.tau[rowsel,]))
      model <- "with intercept and trend"
    }
    yd.l <-  yd[1:(nobs-1)]
    yd.diff <- diff(yd)
    if(lag.max > 1){
      yd.dlags <- embed(diff(yd), lag.max)[, -1]
      data.dfgls <- data.frame(cbind(yd.diff[-(1:(lag.max-1))], yd.l[-(1:(lag.max-1))], yd.dlags))
      colnames(data.dfgls) <- c("yd.diff", "yd.lag", paste("yd.diff.lag", 1:(lag.max-1), sep=""))
      dfgls.form <- formula(paste("yd.diff ~ -1 + ", paste(colnames(data.dfgls)[-1], collapse=" + ")))
    }else if(lag.max <=1){
      data.dfgls <- data.frame(cbind(yd.diff, yd.l))
      colnames(data.dfgls) <- c("yd.diff", "yd.lag")
      dfgls.form <- formula("yd.diff ~ -1 + yd.lag")
    }
    dfgls.reg <- summary(lm(dfgls.form, data=data.dfgls))
    teststat <- coef(dfgls.reg)[1,3]
    test.reg <- dfgls.reg
    lag.max <- lag.max-1
  }
  colnames(cvals) <- c("1pct", "5pct", "10pct")
  rownames(cvals) <- c("critical values")
  new("ur.ers", y=y, yd=yd, type=type, model=model, lag=as.integer(lag.max), cval=round(cvals, 2), teststat=teststat, testreg=test.reg, test.name="Elliot, Rothenberg and Stock")
}

##
## Setting classes for unit root tests
##

setClass("urca", representation(test.name="character"))

setClass("ur.ers", representation(y="vector",
                                  yd="vector",
                                  type="character",
                                  model="character",
                                  lag="integer",
                                  cval="matrix",
                                  teststat="numeric",
                                  testreg="ANY"),
         contains="urca")

setClass("ca.jo", representation(x="ANY",
                                 Z0="matrix",
                                 Z1="matrix",
                                 ZK="matrix",
                                 type="character",
                                 model="character",
                                 ecdet="character",
                                 lag="integer",
                                 P="integer",
                                 season="ANY",
                                 dumvar="ANY",
                                 cval="ANY",
                                 teststat="ANY",
                                 lambda="vector",
                                 Vorg="matrix",
                                 V="matrix",
                                 W="matrix",
                                 PI="matrix",
                                 DELTA="matrix",
                                 GAMMA="matrix",
                                 R0="matrix",
                                 RK="matrix",
                                 bp="ANY",
                                 spec="character",
                                 call="call"),
         contains="urca")

setClass("cajo.test", representation(Z0="matrix",
                                     Z1="matrix",
                                     ZK="matrix",
                                     H="ANY",
                                     A="ANY",
                                     B="ANY",
                                     type="character",
                                     ecdet="character",
                                     teststat="numeric",
                                     pval="vector",
                                     lambda="vector",
                                     Vorg="matrix",
                                     V="matrix",
                                     W="matrix",
                                     PI="matrix",
                                     DELTA="ANY",
                                     DELTA.bb="ANY",
                                     DELTA.ab="ANY",
                                     DELTA.aa.b="ANY",
                                     GAMMA="matrix"),
         contains="urca")

setClass("ur.kpss", representation(y="vector",
                                   type="character",
                                   lag="integer",
                                   cval="matrix",
                                   teststat="numeric",
                                   res="vector"),
         contains="urca")

setClass("ca.po", representation(z="ANY",
                                 type="character",
                                 model="character",
                                 lag="integer",
                                 cval="matrix",
                                 res="matrix",
                                 teststat="numeric",
                                 testreg="ANY"),
         contains="urca")

setClass("ur.pp", representation(y="vector",
                                 type="character",
                                 model="character",
                                 lag="integer",
                                 cval="matrix",
                                 teststat="numeric",
                                 testreg="ANY",
                                 auxstat="matrix",
                                 res="vector"),
         contains="urca")

setClass("ur.df", representation(y="vector",
                                 model="character",
                                 lags="integer",
                                 cval="matrix",
                                 res="vector",
                                 teststat="matrix",
                                 testreg="ANY"),
         contains="urca")

setClass("ur.sp", representation(y="vector",
                                 type="character",
                                 polynomial="integer",
                                 signif="numeric",
                                 teststat="numeric",
                                 cval="numeric",
                                 res="vector",
                                 testreg="ANY"),
         contains="urca")

setClass("ur.za", representation(y="vector",
                                 model="character",
                                 lag="integer",
                                 teststat="numeric",
                                 cval="vector",
                                 bpoint="integer",
                                 tstats="vector",
                                 res="vector",
                                 testreg="ANY"),
         contains="urca")

setClassUnion("otherornull", c("ANY", "character", "integer", "matrix", "vector", "NULL")) 

setClass("sumurca", representation(classname="character",
                                   test.name="character",
                                   testreg="ANY",
                                   teststat="otherornull",
                                   cval="otherornull",
                                   bpoint="otherornull",
                                   signif="otherornull",
                                   model="otherornull",
                                   type="otherornull",
                                   auxstat="otherornull",
                                   lag="otherornull",
                                   H="otherornull",
                                   A="otherornull",
                                   lambda="otherornull",
                                   pval="otherornull",
                                   V="otherornull",
                                   W="otherornull",
                                   P="otherornull")
         )

\name{Raotbl2} 
\alias{Raotbl2}
\docType{data}
\encoding{latin1}
\title{Data set used by Dickey, Jansen and Thornton (1994)}
\description{
  This data set contains the time series used by David A. Dickey, Dennis
  W. Jansen and Daniel L. Thornton in their article: ``A Primer on
  Cointegrating with an Application to Money and Income''. 
}
\usage{data(Raotbl2)}
\format{
  A data frame with quarterly oberservations (ts objects) starting in
  1953:1 until 1988:4 for the following 4 variables (all transformed to
  natural logarithms.
  
    \tabular{rl}{
    \code{m1p} \tab Real money balances M1. \cr
    \code{m2p} \tab Real money balances M2. \cr
    \code{mbp} \tab Real adjusted monetary base. \cr
    \code{nm1m2p} \tab Real non-M1 component of M2.
    }
}
\source{
  Dickey, David A., Dennis W. Jansen and Daniel L. Thornton (1994), A
  Primer on Cointegration with an Application to Money and Income, in:
  Cointegration for the Applied Economist, ed. B. Bhaskara Rao, chapter
  2, Data Appendix, Table D.2.
}
\author{Bernhard Pfaff}
\keyword{datasets}
\concept{data set Rao Dickey Jansen Thornton}

\name{ur.sp-class}
\docType{class}
\alias{ur.sp-class}
\encoding{latin1}
\title{Representation of class ur.sp}
\description{
  This class contains the relevant information by applying the Schmidt
  and Phillips unit root test to a time series.
}
\section{Slots}{
  \describe{
    \item{\code{y}:}{Object of class \code{"vector"}: The time series to
      be tested.}
    \item{\code{type}:}{Object of class \code{"character"}: Test type,
      \code{"rho"} or \code{"tau"} test statistic.}
    \item{\code{polynomial}:}{Object of class \code{"integer"}:
      Deterministic trend specification}
    \item{\code{signif}:}{Object of class \code{"numeric"}: Critical values.}
    \item{\code{teststat}:}{Object of class \code{"numeric"}: Value of
      the test statistic.}
    \item{\code{cval}:}{Object of class \code{"numeric"}: The critical
      values, depending on \code{"signif"}, \code{"polynomial"} and the
      sample size.}
    \item{\code{res}:}{Object of class \code{"vector"}: The residuals of
      the test regression.}
    \item{\code{testreg}:}{Object of class \code{"ANY"}: The summary
      output of the test regression.}
    \item{\code{test.name}:}{Object of class \code{"character"}: The
      name of the test, \emph{i.e.} `"Schmidt and Phillips'.}
  }
}
\section{Extends}{
Class \code{urca}, directly.
}
\section{Methods}{
  Type \code{showMethods(classes="ur.sp")} at the R prompt for a
  complete list of methods which are available for this class.
  
  Useful methods include
  \describe{
    \item{\code{show}:}{test statistic.}
    \item{\code{summary}:}{like show, but critical value and summary of
      test regression added.}
    \item{\code{plot}:}{Diagram of fit plot, residual plot and their
      acfs' and pacfs'.}
  }
}
\references{
  Schmidt, P. and Phillips, P.C.B. (1992), LM Test for a Unit Root in
  the Presence of Deterministic Trends, \emph{Oxford Bulletin of Economics and
  Statistics}, \bold{54(3)}, 257--287.

  Download possible at: \url{https://cowles.yale.edu/}, see rubric
  'Discussion Papers (CFDPs)'.
}
\seealso{
  \code{\link{ur.sp}} and \code{\link{urca-class}}. 
}
\author{Bernhard Pfaff}
\keyword{classes}

\name{blrtest}
\alias{blrtest}
\encoding{latin1}
\title{Likelihood ratio test for restrictions on beta}
\description{
  This function estimates a restricted VAR, where the restrictions are
  base upon \eqn{\bold{\beta}}, \emph{i.e.} the cointegration vectors. The test
  statistic is distributed as \eqn{\chi^2} with \eqn{r(p-s)} degrees of
  freedom, with \eqn{s} equal to the columns of the restricting matrix
  \eqn{\bold{H}}.
}
\usage{
blrtest(z, H, r)
}
\arguments{
  \item{z}{An object of class \code{ca.jo}.}
  \item{H}{The \eqn{(p \times s)} matrix containing the restrictions on
    \eqn{\bold{\beta}}.}
  \item{r}{The count of cointegrating relationships; \cr
    inferred from \code{summary(ca.jo-object)}.}
}
\details{
  Please note, that in the case of nested hypothesis, the reported
  p-value should be adjusted to \eqn{r(s1-s2)} (see Johansen, S. and
  K. Juselius (1990)).
}
\value{
  An object of class \code{cajo.test}.
}
\references{

  Johansen, S. (1988), Statistical Analysis of Cointegration Vectors,
  \emph{Journal of Economic Dynamics and Control}, \bold{12}, 231--254.

  Johansen, S. and Juselius, K. (1990), Maximum Likelihood Estimation and
  Inference on Cointegration -- with Applications to the Demand for
  Money, \emph{Oxford Bulletin of Economics and Statistics}, \bold{52,
    2}, 169--210.

  Johansen, S. (1991), Estimation and Hypothesis Testing of
  Cointegration Vectors in Gaussian Vector Autoregressive Models,
  \emph{Econometrica}, \bold{Vol. 59, No. 6}, 1551--1580.

}
\seealso{
  \code{\link{ca.jo}}, \code{\link{alrtest}}, \code{\link{ablrtest}},
  \code{\link{bh5lrtest}}, \code{\link{bh6lrtest}}, \code{\link{cajo.test-class}},
  \code{\link{ca.jo-class}} and \code{\link{urca-class}}. 
}
\examples{
data(denmark)
sjd <- denmark[, c("LRM", "LRY", "IBO", "IDE")]
sjd.vecm <- ca.jo(sjd, ecdet="const", type="eigen", K=2, spec="longrun",
season=4)
HD0 <- matrix(c(-1, 1, 0, 0, 0, 0, 0, 1, 0, 0, 0, 0, 0, 1, 0, 0, 0, 0, 0, 1), c(5,4))
summary(blrtest(sjd.vecm, H=HD0, r=1))
}
\author{Bernhard Pfaff}
\keyword{regression}

\name{alrtest}
\alias{alrtest}
\encoding{latin1}
\title{Likelihood ratio test for restrictions on alpha}
\description{
  This function estimates a restricted VAR, where the restrictions are
  base upon \eqn{\bold{\alpha}}, \emph{i.e.} the loading vectors. The test
  statistic is distributed as \eqn{\chi^2} with \eqn{r(p-m)} degrees of
  freedom, with \eqn{m} equal to the columns of the restricting matrix
  \eqn{\bold{A}}.
}
\usage{
alrtest(z, A, r)
}
\arguments{
  \item{z}{An object of class \code{ca.jo}.}
  \item{A}{The \eqn{(p \times m)} matrix containing the restrictions on
    \eqn{\bold{\alpha}}.}
  \item{r}{The count of cointegration relationships; \cr
    inferred from \code{summary(ca.jo-object)}.}
}
\details{
  The orthogonal matrix to \eqn{\bold{A}} can be accessed as
  \code{object@B}. The restricted \eqn{\bold{\alpha}} matrix is
  normalised with respect to the first variable.
}
\value{
  An object of class \code{cajo.test}.
}
\references{

  Johansen, S. and Juselius, K. (1990), Maximum Likelihood Estimation and
  Inference on Cointegration -- with Applications to the Demand for
  Money, \emph{Oxford Bulletin of Economics and Statistics}, \bold{52,
    2}, 169--210.

  Johansen, S. (1991), Estimation and Hypothesis Testing of
  Cointegration Vectors in Gaussian Vector Autoregressive Models,
  \emph{Econometrica}, \bold{Vol. 59, No. 6}, 1551--1580.
}   
\seealso{
  \code{\link{ca.jo}}, \code{\link{blrtest}}, \code{\link{ablrtest}},
  \code{\link{cajo.test-class}}, \code{\link{ca.jo-class}} and
  \code{\link{urca-class}}. 
}
\examples{
data(denmark)
sjd <- denmark[, c("LRM", "LRY", "IBO", "IDE")]
sjd.vecm <- ca.jo(sjd, ecdet = "const", type="eigen", K=2, spec="longrun",
season=4)
DA <- matrix(c(1,0,0,0), c(4,1))
summary(alrtest(sjd.vecm, A=DA, r=1))
}
\author{Bernhard Pfaff}
\keyword{regression}

\name{ur.sp}
\alias{ur.sp}
\encoding{latin1}
\title{Schmidt and Phillips Unit Root Test}
\description{
  Performs the Schmidt and Phillips unit root test, where under the Null
  and Alternative Hypothesis the coefficients of the deterministic
  variables are included.
}
\usage{
ur.sp(y, type = c("tau", "rho"), pol.deg = c(1, 2, 3, 4),
      signif = c(0.01, 0.05, 0.1))
}
\arguments{
  \item{y}{Vector to be tested for a unit root.}
  \item{type}{Test type, either \code{tau} or \code{rho} test.}
  \item{pol.deg}{Degree of polynomial in the test regression.}
  \item{signif}{Significance level for the critical value of the test statistic.}
}
\details{
  Under the Null and the Alternative hypothesis the coefficients of the
  deterministic part of the test regression are included. Two test types
  are available: the \code{rho}-test and the \code{tau}-test.
  Both test are extracted from the LM principle.
}
\value{
  An object of class \code{"ur.sp"}.
}
\references{
  Schmidt, P. and Phillips, P.C.B. (1992), LM Test for a Unit Root in
  the Presence of Deterministic Trends, \emph{Oxford Bulletin of Economics and
  Statistics}, \bold{54(3)}, 257--287.

  Download possible at: \url{https://cowles.yale.edu/}, see rubric
  'Discussion Papers (CFDPs)'.
}
\seealso{\code{\link{ur.sp-class}}}
\examples{
data(nporg)
gnp <- na.omit(nporg[, "gnp.r"])
sp.gnp <- ur.sp(gnp, type="tau", pol.deg=1, signif=0.01)
summary(sp.gnp)
}
\author{Bernhard Pfaff}
\keyword{regression}

\name{npext}
\docType{data}
\alias{npext}
\encoding{latin1}
\title{Nelson and Plosser extended data set}
\description{
  This data set contains the fourteen U.S. economic time series used by
  Schotman and Dijk. All series are transformed by taking logarithms
  except for the bond yield. The sample period ends in 1988.
}
\usage{data(npext)}
\format{
  A data frame containing fourteen series.
		
  \tabular{rl}{
    \code{year} \tab Time index from 1860 until 1988. \cr 
    \code{realgnp} \tab Real GNP, [Billions of 1958 Dollars], \cr
    \tab [1909 -- 1988] \cr
    \code{nomgnp} \tab Nominal GNP, \cr
    \tab [Millions of Current Dollars], [1909 -- 1988] \cr
    \code{gnpperca} \tab Real Per Capita GNP, \cr
    \tab [1958 Dollars], [1909 -- 1988] \cr
    \code{indprod} \tab Industrial Production Index, \cr
    \tab [1967 = 100], [1860 -- 1988] \cr
    \code{employmt} \tab Total Employment, \cr
    \tab [Thousands], [1890 -- 1988] \cr
    \code{unemploy} \tab Total Unemployment Rate, \cr
    \tab [Percent], [1890 -- 1988] \cr
    \code{gnpdefl} \tab GNP Deflator, \cr
    \tab [1958 = 100], [1889 -- 1988] \cr
    \code{cpi} \tab Consumer Price Index, \cr
    \tab [1967 = 100], [1860 -- 1988] \cr
    \code{wages} \tab Nominal Wages \cr
    \tab (Average annual earnings per full-time employee in manufacturing), \cr
    \tab [current Dollars], [1900 -- 1988] \cr
    \code{realwag} \tab Real Wages, \cr
    \tab [Nominal wages/CPI], [1900 -- 1988] \cr
    \code{M} \tab Money Stock (M2), \cr
    \tab [Billions of Dollars, annual averages], [1889 -- 1988] \cr
    \code{velocity} \tab Velocity of Money, \cr
    \tab [1869 -- 1988] \cr
    \code{interest} \tab Bond Yield (Basic Yields of 30-year
       corporate bonds), \cr
    \tab [Percent per annum], [1900 -- 1988] \cr
    \code{sp500} \tab Stock Prices, \cr
    \tab [Index; 1941 -- 43 = 100], [1871 -- 1988] \cr
    }
}
\source{
  Schotman, P.C. and van Dijk, H.K. (1991), On Bayesian Routes to Unit
  Roots, \emph{Journal of Applied Econometrics}, \bold{6}, 387--401.

  Koop, G. and Steel, M.F.J. (1994), A Decision-Theoretic Analysis of
  the Unit-Root Hypothesis using Mixtures of Elliptical Models,
  \emph{Journal of Business and Economic Statistics}, \bold{12}, 95--107. 
}
\references{\url{https://www.amstat.org}}
\author{Bernhard Pfaff}
\keyword{datasets}
\concept{data set Nelson Plosser macroeconomic variables}

\name{ablrtest}
\alias{ablrtest}
\encoding{latin1}
\title{Likelihood ratio test for restrictions on alpha and beta}
\description{
  This function estimates a restricted VAR, where the restrictions are
  based upon \eqn{\bold{\alpha}}, \emph{i.e.} the loading vectors and
  \eqn{\bold{\beta}}, \emph{i.e} the matrix of cointegration vectors. The test
  statistic is distributed as \eqn{\chi^2} with \eqn{(p-m)r + (p-s)r} degrees of
  freedom, with \eqn{m} equal to the columns of the restricting matrix
  \eqn{\bold{A}}, \eqn{s} equal to the columns of the restricting matrix
  \eqn{\bold{H}} and \eqn{p} the order of the VAR.
}
\usage{
ablrtest(z, H, A, r)
}
\arguments{
  \item{z}{An object of class \code{ca.jo}.}
  \item{H}{The \eqn{(p \times s)} matrix containing the restrictions on
    \eqn{\bold{\beta}}.}
  \item{A}{The \eqn{(p \times m)} matrix containing the restrictions on
    \eqn{\bold{\alpha}}.}
  \item{r}{The count of cointegrating relationships; \cr
    inferred from \code{summary(ca.jo-object)}.}
}
\details{
  The restricted \eqn{\bold{\alpha}} matrix, as well as \eqn{\bold{\beta}} is
  normalised with respect to the first variable.
}
\value{
  An object of class \code{cajo.test}.
}
\references{

  Johansen, S. and Juselius, K. (1990), Maximum Likelihood Estimation and
  Inference on Cointegration -- with Applications to the Demand for
  Money, \emph{Oxford Bulletin of Economics and Statistics}, \bold{52,
    2}, 169--210.

  Johansen, S. (1991), Estimation and Hypothesis Testing of
  Cointegration Vectors in Gaussian Vector Autoregressive Models,
  \emph{Econometrica}, \bold{Vol. 59, No. 6}, 1551--1580.

}
\seealso{
  \code{\link{ca.jo}}, \code{\link{alrtest}}, \code{\link{blrtest}},
  \code{\link{cajo.test-class}}, \code{\link{ca.jo-class}} and
  \code{\link{urca-class}}. 
}
\examples{
data(denmark)
sjd <- denmark[, c("LRM", "LRY", "IBO", "IDE")]
sjd.vecm <- ca.jo(sjd, ecdet = "const", type="eigen", K=2, spec="longrun",
season=4)
HD1 <- matrix(c(1, -1, 0, 0, 0, 0, 0, 1, -1, 0, 0, 0, 0, 0, 1), c(5,3))
DA <- matrix(c(1,0,0,0, 0, 1, 0, 0, 0, 0, 0, 1), c(4,3))
summary(ablrtest(sjd.vecm, H=HD1, A=DA, r=1))
}
\author{Bernhard Pfaff}
\keyword{regression}

\name{sumurca-class}
\docType{class}
\alias{sumurca-class}
\encoding{latin1}
\title{Representation of class sumurca}
\description{
  A class for objects returned by applying method summary() to objects
  from classes: \code{ur.ers}, \code{ca.jo}, \code{cajo.test},
  \code{ur.kpss}, \code{ca.po}, \code{ur.pp}, \code{ur.df}, \code{ur.sp}
  or \code{ur.za}.
 
}
\section{Slots}{
  \describe{
    \item{\code{classname}:}{The class name of the original object to
      which method summary is applied.}
    \item{\code{test.name}:}{The name of the test, \emph{i.e.}
      `Johansen-Procedure'.}
    \item{\code{testreg}:}{The test regression where applicable,
      otherwise set to \code{NULL}.}
    \item{\code{teststat}:}{The test statististic where applicable,
      otherwise set to \code{NULL}.}
    \item{\code{cval}:}{The critical values of the test where applicable,
      otherwise set to \code{NULL}.}
    \item{\code{bpoint}:}{Potential break point where applicable,
      otherwise set to \code{NULL}.}
    \item{\code{signif}:}{Significance level of the test where
      applicable, otherwise set to \code{NULL}.}
    \item{\code{model}:}{Description of the underlying model where applicable,
      otherwise set to \code{NULL}.}
    \item{\code{type}:}{The test type where applicable,
      otherwise set to \code{NULL}.}
    \item{\code{auxstat}:}{The result of an auxiliary regression where
      applicable, otherwise set to \code{NULL}.}
    \item{\code{lag}:}{The number of lags included where applicable,
      otherwise set to \code{NULL}.}
    \item{\code{H}:}{The matrix containing the restrictions placed upon
      \eqn{\bold{\beta}} where applicable, otherwise set to \code{NULL}.}
    \item{\code{A}:}{The matrix containing the restrictions placed upon
      \eqn{\bold{\alpha}} where applicable, otherwise set to \code{NULL}.}
    \item{\code{lambda}:}{The eigenvalues where applicable,
      otherwise set to \code{NULL}.}
    \item{\code{pval}:}{The p-value and the degrees of freedom where
      applicable, otherwise set to \code{NULL}.}
    \item{\code{V}:}{The matrix of eigenvectors, normalised with respect
      to the first variable where applicable,
      otherwise set to \code{NULL}.}
    \item{\code{W}:}{The matrix of loading weights where applicable,
      otherwise set to \code{NULL}.}
    \item{\code{P}:}{The count of variables where applicable,
      otherwise set to \code{NULL}.}
  }
}
\section{Methods}{
  For this class a \code{print} method is available, that nicely prints the
  summary results of objects belonging to either one of the following
  classes: \code{ur.ers}, \code{ca.jo}, \code{cajo.test}, 
  \code{ur.kpss}, \code{ca.po}, \code{ur.pp}, \code{ur.df}, \code{ur.sp} or
  \code{ur.za}.
}
\seealso{
  \code{\link{summary}}, \code{\link{ur.ers-class}},
  \code{\link{ur.kpss-class}}, \code{\link{ca.jo-class}},
  \code{\link{cajo.test-class}}, \code{\link{ca.po-class}},
  \code{\link{ur.pp-class}}, \code{\link{ur.df-class}},
  \code{\link{ur.sp-class}} and \code{\link{ur.za-class}}. 
}
\author{Bernhard Pfaff}
\keyword{classes}

\name{Raotbl3} 
\alias{Raotbl3}
\encoding{latin1}
\docType{data}
\title{Data set used by Holden and Perman (1994)}
\description{
  This data set contains the time series used by Darryl Holden and Roger
  Perman in their article: ``Unit Roots and Cointegration for the Economist". 
}
\usage{data(Raotbl3)}
\format{
  A data frame with quarterly data (ts objects) from the United Kingdom
  starting in 1966:4 until 1991:2 for the following 6 variables (all
  transformed to natural logarithms).
  
    \tabular{rl}{
    \code{lc} \tab Real consumption expenditure. \cr
    \code{li} \tab Real income. \cr
    \code{lw} \tab Real wealth. \cr
    \code{dd682} \tab Dummy variable for 68:2. \cr
    \code{dd792} \tab Dummy variable for 79:2. \cr
    \code{dd883} \tab Dummy variable for 88:3. 
  }
  
  More details about the data are provided in the data appendix of Rao,
  ``Cointegration for the Applied Economist" (see source below).
}
\source{
  Holden, Darryl and Roger Perman (1994), Unit Roots and Cointegration
  for the Economist, in:  Cointegration for the Applied Economist,
  ed. B. Bhaskara Rao, chapter 3, Data Appendix, Table D.3.
}
\author{Bernhard Pfaff}
\keyword{datasets}
\concept{data set Rao Holden Perman}
  

\name{alphaols}
\alias{alphaols}
\encoding{latin1}
\title{OLS regression of VECM weighting matrix}
\description{
  This functions estimates the \eqn{\bold{\alpha}} matrix of a VECM.
  The following OLS regression of the R-form of the VECM is hereby
  utilised:
  \deqn{\bold{R}_{0t} = \bold{\alpha}\bold{\beta}\prime \bold{R}_{kt} +
    \bold{\varepsilon}_t \qquad t=1, \dots, T}
}
\usage{
alphaols(z, reg.number = NULL)
}
\arguments{
  \item{z}{An object of class \code{ca.jo}.}
  \item{reg.number}{The number of the equation in the R-form that should
    be estimated or if set to \code{NULL} (the default), all equations
    within the R-form are estimated.}
}
\details{
  The cointegrating relations, \emph{i.e.} \eqn{\bold{R}_{kt}\prime
    \bold{\beta}} are calculated by using \code{z@RK} and \code{z@V}.
}
\value{
  Returns an object of class \code{lm}.
}
\references{

  Johansen, S. (1988), Statistical Analysis of Cointegration Vectors,
  \emph{Journal of Economic Dynamics and Control}, \bold{12}, 231--254.

  Johansen, S. and Juselius, K. (1990), Maximum Likelihood Estimation and
  Inference on Cointegration -- with Applications to the Demand for
  Money, \emph{Oxford Bulletin of Economics and Statistics}, \bold{52,
    2}, 169--210.

  Johansen, S. (1991), Estimation and Hypothesis Testing of
  Cointegration Vectors in Gaussian Vector Autoregressive Models,
  \emph{Econometrica}, \bold{Vol. 59, No. 6}, 1551--1580.

}
\seealso{
  \code{\link{ca.jo}}, \code{\link[stats]{lm}}, \code{\link{ca.jo-class}}
  and \code{\link{urca-class}}. 
}
\examples{
data(denmark)
sjd <- denmark[, c("LRM", "LRY", "IBO", "IDE")]
sjd.vecm1 <- ca.jo(sjd, ecdet = "const", type="eigen", K=2, spec="longrun",
season=4)
summary(alphaols(sjd.vecm1))
summary(alphaols(sjd.vecm1, reg.number=1))
}
\author{Bernhard Pfaff}
\keyword{regression}
\concept{VECM OLS Loading Johansen Juselius Cointegration Co-integration}

\name{UKpppuip}
\alias{UKpppuip}
\docType{data}
\encoding{latin1}
\title{Data set for the United Kingdom: ppp and uip}
\description{
  This data set contains the series used by in Johansen and Juselius
  (1992), Testing structural hypothesis in a multivariate cointegration
  analysis of the PPP and UIP for UK, Journal of Econometrics, 53, 211-244.
}
\usage{data(UKpppuip)}
\format{
  A data frame of quarterly data ranging from 1971:Q1
  until 1987:Q2. All variables are expressed in logarithms.
 

    \tabular{rl}{
    \code{p1} \tab UK wholesale price index. \cr
    \code{p2} \tab Trade weighted foreign whole sale price index.\cr
    \code{e12} \tab UK effective exchange rate.\cr
    \code{i1} \tab Three-month treasury bill rate in the UK.\cr
    \code{i2} \tab Three-month Eurodollar interest rate.\cr
    \code{dpoil0} \tab World oil price at period \code{t}.\cr
    \code{dpoil1} \tab World oil price at period \code{t-1}.
  }
}
\references{
  Johansen, S. and K. Juselius (1992), Testing structural hypothesis in
  a multivariate cointegration analysis of the PPP and UIP for UK,
  Journal of Econometrics, 53, 211-244.
 
}
\author{Bernhard Pfaff}
\keyword{datasets}
\concept{data set UK PPP and UIP}

\name{Raotbl1} 
\alias{Raotbl1}
\docType{data}
\encoding{latin1}
\title{Data set used by Dickey, Jansen \& Thornton (1994)}
\description{
  This data set contains the time series used by David A. Dickey, Dennis
  W. Jansen and Daniel L. Thornton in their article: ``A Primer on
  Cointegrating with an Application to Money and Income''. 
}
\usage{data(Raotbl1)}
\format{
  A data frame with quarterly oberservations (ts objects) starting in
  1953:1 until 1988:4 for the following 4 variables (all transformed to
  natural logarithms.
  
    \tabular{rl}{
    \code{k} \tab Ratio of currency to total checkable deposits. \cr
    \code{ksa} \tab seasonally adjusted series of \code{k}. \cr
    \code{r3m} \tab Nominal 3 month T-Bill rate. \cr
    \code{r10y} \tab Nominal yield on 10-year Government securities. \cr
    \code{rgnp} \tab Real GNP.
    }
}
\source{
  Dickey, David A., Dennis W. Jansen and Daniel L. Thornton (1994), A
  Primer on Cointegration with an Application to Money and Income, in:
  Cointegration for the Applied Economist, ed. B. Bhaskara Rao, chapter
  2, Data Appendix, Table D.1.
}
\author{Bernhard Pfaff}
\keyword{datasets}
\concept{data set Rao Dickey Jansen Thornton}

\name{ur.df-class}
\docType{class}
\alias{ur.df-class}
\encoding{latin1}
\title{Representation of class ur.df}
\description{
  This class contains the relevant information by applying the augmented
  Dickey-Fuller unit root test to a time series.
}
\section{Slots}{
  \describe{
    \item{\code{y}:}{Object of class \code{"vector"}: The time series to
      be tested.}
    \item{\code{model}:}{Object of class \code{"character"}: The type of
      the deterministic part, either \code{"none"}, \code{"drift"} or
      \code{"trend"}. The latter includes a constant term, too.}
    \item{\code{lags}:}{Object of class \code{"integer"}: Number of lags
      for error correction.}
    \item{\code{cval}:}{Object of class \code{"matrix"}: Critical values
      at the 1\%, 5\% and 10\% level of significance.}
    \item{\code{teststat}:}{Object of class \code{"matrix"}: Value of
      the test statistic.}
    \item{\code{testreg}:}{Object of class \code{"ANY"}: The summary
      output of the test regression.}
    \item{\code{res}:}{Object of class \code{"vector"}: The residuals of
      the test regression.}
    \item{\code{test.name}:}{Object of class \code{"character"}: The
      name of the test, \emph{i.e} `Augmented-Dickey-Fuller Test'.}
  }
}
\section{Extends}{
Class \code{urca}, directly.
}
\section{Methods}{
  Type \code{showMethods(classes="ur.df")} at the R prompt for a
  complete list of methods which are available for this class.
  
  Useful methods include
  \describe{
    \item{\code{show}:}{test statistic.}
    \item{\code{summary}:}{like show, but critical value and summary of
      test regression added.}
    \item{\code{plot}:}{Residual plot, acfs' and pacfs'.}
  }
}
\references{
  Dickey, D. A. and Fuller, W. A. (1979), Distributions of the
  Estimators For Autoregressive Time Series with a Unit Root,
  \emph{Journal of the American Statistical Association},  \bold{75}, 427--431.
  
  Dickey, D. A. and Fuller, W. A. (1981), Likelihood Ratio Statistics
  for Autoregressive Time Series with a Unit Root, \emph{Econometrica},
  \bold{49}, 1057--1072.

  Hamilton (1994), \emph{Time Series Analysis}, Princeton University Press.
}
\seealso{
  \code{\link{ur.df}} and \code{\link{urca-class}}
}
\author{Bernhard Pfaff}
\keyword{classes}

\name{ur.ers-class}
\docType{class}
\alias{ur.ers-class}
\title{Representation of class ur.ers}
\description{
  This class contains the relevant information by applying the Elliott,
  Rothenberg and Stock unit root test.
}
\section{Slots}{
  \describe{
    \item{\code{y}:}{Object of class \code{"vector"}: The time series to
      be tested.}
    \item{\code{yd}:}{Object of class \code{"vector"}: The detrended
      time series.}
    \item{\code{type}:}{Object of class \code{"character"}: Test type,
      either \code{"DF-GLS"} (default), or \code{"P-test"}.}
    \item{\code{model}:}{Object of class \code{"character"}: The
      deterministic model used for detrending, either intercept only, or
      intercept with linear trend.}
    \item{\code{lag}:}{Object of class \code{"integer"}: The number of
      lags used in the test/auxiliary regression.}
    \item{\code{cval}:}{Object of class \code{"matrix"}: The critical
      values of the test at the 1\%, 5\% and 10\% level of significance.}
    \item{\code{teststat}:}{Object of class \code{"numeric"}: The value
      of the test statistic.}
    \item{\code{testreg}:}{Object of class \code{"ANY"}: The test
      regression, only set for \code{"DF-GLS"}.}
    \item{\code{test.name}:}{Object of class \code{"character"}: The
      name of the test, \emph{i.e.} `Elliott, Rothenberg and Stock'.}
  }
}
\section{Extends}{
Class \code{urca}, directly.
}
\section{Methods}{
  Type \code{showMethods(classes="ur.ers")} at the R prompt for a
  complete list of methods which are available for this class.
  
  Useful methods include
  \describe{
    \item{\code{show}:}{test statistic.}
    \item{\code{summary}:}{like show, but test type, test regression (\code{type="DF-GLS"}) and critical values added.}
    \item{\code{plot}:}{Diagram of fit, residual plot and their acfs'
      and pacfs' for \code{type="DF-GLS"}.}
  }

}
\references{
  Elliott, G., Rothenberg, T.J. and Stock, J.H. (1996), Efficient Tests
  for an Autoregressive Unit Root, \emph{Econometrica},
  \bold{Vol. 64, No. 4}, 813--836.

  MacKinnon, J.G. (1991), Critical Values for Cointegration Tests,
  \emph{Long-Run Economic Relationships}, eds. R.F. Engle and
  C.W.J. Granger, London, Oxford, 267--276.

}
\seealso{
  \code{\link{ur.ers}} and \code{\link{urca-class}}.
}
\author{Bernhard Pfaff}
\keyword{classes}

\name{show-methods}
\docType{methods}
\alias{show}
\alias{show-methods}
\alias{show,ur.ers-method}
\alias{show,ur.kpss-method}
\alias{show,ca.jo-method}
\alias{show,ca.po-method}
\alias{show,cajo.test-method}
\alias{show,ur.pp-method}
\alias{show,ur.df-method}
\alias{show,ur.sp-method}
\alias{show,ur.za-method}
\alias{show,sumurca-method}
\encoding{latin1}
\title{Methods for Function show in Package `urca'}
\description{
  Displays the outcome of the unit root/cointegration tests.
}
\section{Methods}{\describe{

\item{object = "ca.jo"}{Displays the test statistic of the Johansen procedure.}
\item{object = "cajo.test"}{Displays the test statistic of a restricted
  VAR with respect to \eqn{\bold{\alpha}} and/or \eqn{\bold{\beta}}.}
\item{object = "ca.po"}{Displays the test statistic of the Phillips and
  Ouliaris cointegration test.}
\item{object = "ur.df"}{Displays the test statistic of the Augmented,
  Dickey and Fuller unit root test.}
\item{object = "ur.ers"}{Displays the test statistic of the Elliott,
  Rothenberg and Stock unit root test.}
\item{object = "ur.kpss"}{Displays the test statistic of the Kwiatkowski
  \emph{et al.} unit root test.}
\item{object = "ur.pp"}{Displays the test statistic of the Phillips and
  Perron unit root test.}
\item{object = "ur.df"}{Displays the test statistic of the augmented
  Dickey-Fuller unit root test.}
\item{object = "ur.sp"}{Displays the test statistic of the Schmidt and
  Phillips unit root test.}
\item{object = "ur.za"}{Displays the test statistic of the Zivot and
  Andrews unit root test.}
\item{object = "sumurca"}{Displays the summary output.}

}}
\seealso{
  \code{\link{ca.jo-class}}, \code{\link{cajo.test-class}},
  \code{\link{ca.po-class}}, \code{\link{ur.ers-class}},
  \code{\link{ur.kpss-class}}, \code{\link{ur.pp-class}},
  \code{\link{ur.sp-class}}, \code{\link{ur.df-class}} and \code{\link{ur.za-class}}.
}
\examples{
data(nporg)
gnp <- na.omit(nporg[, "gnp.r"])
gnp.l <- log(gnp)
#
ers.gnp <- ur.ers(gnp, type="DF-GLS", model="trend", lag.max=4)
show(ers.gnp)
#
kpss.gnp <- ur.kpss(gnp.l, type="tau", lags="short")
show(kpss.gnp)
#
pp.gnp <- ur.pp(gnp, type="Z-tau", model="trend", lags="short")
show(pp.gnp)
#
df.gnp <- ur.df(gnp, type="trend", lags=4)
show(df.gnp)
#
sp.gnp <- ur.sp(gnp, type="tau", pol.deg=1, signif=0.01)
show(sp.gnp)
#
za.gnp <- ur.za(gnp, model="both", lag=2)
show(za.gnp)
#
data(denmark)
sjd <- denmark[, c("LRM", "LRY", "IBO", "IDE")]
sjd.vecm <- ca.jo(sjd, ecdet = "const", type="eigen", K=2, season=4)
show(sjd.vecm)
#
HD0 <- matrix(c(-1, 1, 0, 0, 0, 0, 0, 1, 0, 0, 0, 0, 0, 1, 0, 0, 0, 0, 0, 1), c(5,4))
show(blrtest(sjd.vecm, H=HD0, r=1))
}
\author{Bernhard Pfaff}
\keyword{methods}

\name{UKconinc}
\alias{UKconinc}
\docType{data}
\encoding{latin1}
\title{Data set for the United Kingdom}
\description{
  This data set contains the series used by Hylleberg, S., R. F. Engle,
  C. W. J. Granger and B. S. Yoo (1990), Seasonal Integration and
  Cointegration, Journal of econometrics, 44, 215--238.
}
\usage{data(UKconinc)}
\format{
  A data frame of quarterly data ranging from 1955:Q1
  until 1984:Q4. The data is expressed in natural logarithms.
 

    \tabular{rl}{
    \code{consl} \tab The log of total real consumption in the U.K. \cr
    \code{incl} \tab The log of real disposable income in the U.K.
    }
}
\source{
  Journal of Applied Econometrics Data Archive
  \url{http://qed.econ.queensu.ca/jae/}
}
\references{
Hylleberg, S., R. F. Engle, C. W. J. Granger and B. S. Yoo (1990),
Seasonal Integration and Cointegration, \emph{Journal of econometrics},
\bold{44}, 215--238.}
 
\author{Bernhard Pfaff}
\keyword{datasets}
\concept{data set UK consumption income HEGY}

\name{cajools}
\alias{cajools}
\encoding{latin1}
\title{OLS regression of VECM}
\description{
  This function returns the OLS regressions of an unrestricted VECM,
  \emph{i.e.} it returns an object of class \code{lm}. The user can provide a
  certain number of which equation in the VECM should be estimated and
  reported, or if \code{"reg.number=NULL"} each equation in the VECM
  will be estimated and its results are reported.
}
\usage{
cajools(z, reg.number = NULL)
}
\arguments{
  \item{z}{An object of class \code{ca.jo} or \code{cajo.test}.}
  \item{reg.number}{The number of the equation in the VECM that should
    be estimated or if set to \code{NULL} (the default), all equations
    within the VECM are estimated.}
}
\value{
  Returns an object of class \code{lm}.
}
\references{

  Johansen, S. (1988), Statistical Analysis of Cointegration Vectors,
  \emph{Journal of Economic Dynamics and Control}, \bold{12}, 231--254.

  Johansen, S. and Juselius, K. (1990), Maximum Likelihood Estimation and
  Inference on Cointegration -- with Applications to the Demand for
  Money, \emph{Oxford Bulletin of Economics and Statistics}, \bold{52,
    2}, 169--210.

  Johansen, S. (1991), Estimation and Hypothesis Testing of
  Cointegration Vectors in Gaussian Vector Autoregressive Models,
  \emph{Econometrica}, \bold{Vol. 59, No. 6}, 1551--1580.

}
\seealso{
  \code{\link{ca.jo}}, \code{\link{cajorls}}, \code{\link[stats]{lm}},
  \code{\link{ca.jo-class}} and \code{\link{urca-class}}. 
}
\examples{
data(denmark)
sjd <- denmark[, c("LRM", "LRY", "IBO", "IDE")]
sjd.vecm1 <- ca.jo(sjd, ecdet = "const", type="eigen", K=2, spec="longrun",
season=4)
sjd.vecm2 <- ca.jo(sjd, ecdet = "const", type="eigen", K=2, spec="transitory",
season=4)
sjd.vecm.ols1 <- cajools(sjd.vecm1)
sjd.vecm.ols2 <- cajools(sjd.vecm2)
summary(sjd.vecm.ols1)
summary(sjd.vecm.ols2)
}
\author{Bernhard Pfaff}
\keyword{regression}
\concept{VECM OLS Johansen Juselius Cointegration Co-integration}

\name{ur.kpss-class}
\docType{class}
\alias{ur.kpss-class}
\encoding{latin1}
\title{Representation of class ur.kpss}
\description{
  This class contains the relevant information by applying the
  Kwiatkowski, Phillips, Schmidt and Shin unit root test to a time series.
}
\section{Slots}{
  \describe{
    \item{\code{y}:}{Object of class \code{"vector"}: The time series to
      be tested.}
    \item{\code{type}:}{Object of class \code{"character"}: Test type,
      \code{"mu"} or \code{"tau"} depending on the deterministic part.}
    \item{\code{lag}:}{Object of class \code{"integer"}: Number of lags
      for error term correction.}
    \item{\code{cval}:}{Object of class \code{"matrix"}: Critical value
      of test.}
    \item{\code{teststat}:}{Object of class \code{"numeric"}: Value of
      test statistic.}
    \item{\code{res}:}{Object of class \code{"vector"}: Residuals of
      test regression.}
    \item{\code{test.name}:}{Object of class \code{"character"}: The
      name of the test, \emph{i.e.} `KPSS'.}
  }
}
\section{Extends}{
Class \code{urca}, directly.
}
\section{Methods}{
  Type \code{showMethods(classes="ur.kpss")} at the R prompt for a
  complete list of methods which are available for this class.
  
  Useful methods include
  \describe{
    \item{\code{show}:}{test statistic.}
    \item{\code{summary}:}{like show, but critical values, lags and test
      type added.}
    \item{\code{plot}:}{Residual plot and their acfs' and pacfs'.}
  }

}
\references{
  Kwiatkowski, D., Phillips, P.C.B., Schmidt, P. and Shin, Y., (1992),
  Testing the Null Hypothesis of Stationarity Against the Alternative of
  a Unit Root: How Sure Are We That Economic Time Series Have a Unit
  Root?, \emph{Journal of Econometrics}, \bold{54}, 159--178.

  Download possible at: \url{https://cowles.yale.edu/}, see rubric
  'Discussion Papers (CFDPs)'.
}
\seealso{
  \code{\link{ur.kpss}} and \code{\link{urca-class}}.
}
\author{Bernhard Pfaff}
\keyword{classes}

\name{summary-methods}
\docType{methods}
\alias{summary}
\alias{summary-methods}
\alias{summary,ur.ers-method}
\alias{summary,ur.kpss-method}
\alias{summary,ca.jo-method}
\alias{summary,cajo.test-method}
\alias{summary,ca.po-method}
\alias{summary,ur.pp-method}
\alias{summary,ur.df-method}
\alias{summary,ur.sp-method}
\alias{summary,ur.za-method}
\encoding{latin1}
\title{Methods for Function summary in Package `urca'}
\description{
  Summarises the outcome of unit root/cointegration tests by creating a new object of class \code{sumurca}.
}
\section{Methods}{\describe{
    \item{object = "ur.df"}{The test type, its statistic, the test
      regression and the critical values for the Augmented Dickey and
      Fuller test are returned.}  
    \item{object = "ur.ers"}{The test type, its statistic and the
      critical values for the Elliott, Rothenberg and Stock test are
      returned. In case of test \code{"DF-GLS"} the summary output
      of the test regression is provided, too.}
    \item{object = "ur.kpss"}{The test statistic, the critical value as
      well as the test type and the number of lags used for error
      correction for the Kwiatkowski \emph{et al.} unit root test is
      returned.}
    \item{object = "ca.jo"}{The \code{"trace"} or \code{"eigen"} statistic,
      the critical values as well as the eigenvalues, eigenvectors and
      the loading matrix of the Johansen procedure are reported.}
    \item{object = "cajo.test"}{The test statistic of a restricted VAR
      with respect to \eqn{\bold{\alpha}} and/or \eqn{\bold{\beta}} with
      p-value and degrees of freedom is reported. Furthermore, the
      restriction matrix(ces), the eigenvalues and eigenvectors as well
      as the loading matrix are returned.}  
    \item{object = "ca.po"}{The \code{"Pz"} or \code{"Pu"} statistic,
      the critical values as well as the summary output of the test
      regression for the Phillips and Ouliaris cointegration test.}
    \item{object = "ur.pp"}{The Z statistic, the critical values as
      well as the summary output of the test regression for the Phillips
      and Perron test, as well as the test statistics for the
      coefficients of the deterministic part is returned.}
    \item{object = "ur.df"}{The relevant tau statistic, the critical
      values as well as the summary output of the test regression for
      the augmented Dickey-Fuller test is returned.}
    \item{object = "ur.sp"}{The test statistic, the critical value as
      well as the summary output of the test regression for the Schmidt
      and Phillips test is returned.}
    \item{object = "ur.za"}{The test statistic, the critical values as
      well as the summary output of the test regression for the Zivot and
      Andrews test is returned.}
}}
\seealso{
  \code{\link{ur.ers-class}}, \code{\link{ur.kpss-class}},
  \code{\link{ca.jo-class}}, \code{\link{cajo.test-class}},
  \code{\link{ca.po-class}}, \code{\link{ur.pp-class}},
  \code{\link{ur.df-class}}, \code{\link{ur.sp-class}},
  \code{\link{ur.za-class}} and \code{\link{sumurca-class}}.}
\examples{
data(nporg)
gnp <- na.omit(nporg[, "gnp.r"])
gnp.l <- log(gnp)
#
ers.gnp <- ur.ers(gnp, type="DF-GLS", model="trend", lag.max=4)
summary(ers.gnp)
#
kpss.gnp <- ur.kpss(gnp.l, type="tau", lags="short")
summary(kpss.gnp)
#
pp.gnp <- ur.pp(gnp, type="Z-tau", model="trend", lags="short")
summary(pp.gnp)
#
df.gnp <- ur.df(gnp, type="trend", lags=4)
summary(df.gnp)
#
sp.gnp <- ur.sp(gnp, type="tau", pol.deg=1, signif=0.01)
summary(sp.gnp)
#
za.gnp <- ur.za(gnp, model="both", lag=2)
summary(za.gnp)
#
data(finland)
sjf <- finland
sjf.vecm <- ca.jo(sjf, ecdet="none", type="eigen", K=2, season=4)
summary(sjf.vecm)
#
HF0 <- matrix(c(-1, 1, 0, 0, 0, 0, 1, 0, 0, 0, 0, 1), c(4,3))
summary(blrtest(sjf.vecm, H=HF0, r=3))
}
\author{Bernhard Pfaff}
\keyword{methods}

\name{Raotbl7} 
\alias{Raotbl7}
\docType{data}
\encoding{latin1}
\title{Data set used by Glenn Otto (1994)}
\description{
  This data set contains Canadian quarterly data used by Glenn Otto in his
  article: ``Diagnostic Testing: An Application to the Demand for M1". 
}
\usage{data(Raotbl7)}
\format{
  A data frame with quarterly data from 1956:1 until 1988:4. 
  
    \tabular{rl}{
    \code{m1} \tab Money stock M1. \cr
    \code{p} \tab Implicit price deflator for GDP, 1981=100. \cr
    \code{gdp} \tab GDP at constant 1981 prices. \cr
    \code{r} \tab 90-day prime corporate rate. 
  }
}
\source{
  Glenn Otto (1994), Diagnostic Testing: An Application to the Demand
  for M1, in:  Cointegration for the Applied Economist, 
  ed. B. Bhaskara Rao, chapter 6, Data Appendix, Table D.6.
}
\author{Bernhard Pfaff}
\keyword{datasets}
\concept{data set Rao Otto}

\name{ur.kpss}
\alias{ur.kpss}
\encoding{latin1}
\title{Kwiatkowski et al. Unit Root Test}
\description{
  Performs the KPSS unit root test, where the Null hypothesis is
  stationarity. The test types specify as deterministic component either
  a constant \code{"mu"} or a constant with linear trend \code{"tau"}.
}
\usage{
ur.kpss(y, type = c("mu", "tau"), lags = c("short", "long", "nil"),
        use.lag = NULL)
}
\arguments{
  \item{y}{Vector to be tested for a unit root.}
  \item{type}{Type of deterministic part.}
  \item{lags}{Maximum number of lags used for error term correction.}
  \item{use.lag}{User specified number of lags.}
}
\details{
  \code{lags="short"} sets the number of lags to
  \eqn{\sqrt[4]{4 \times (n/100)}}, whereas
  \code{lags="long"} sets the number of lags to
  \eqn{\sqrt[4]{12 \times (n/100)}}. If \code{lags="nil"} is choosen,
  then no error correction is made. Furthermore, one can specify a
  different number of maximum lags by setting \code{use.lag}
  accordingly.
}
\value{
  An object of class \code{ur.kpss}.
}
\references{
  Kwiatkowski, D., Phillips, P.C.B., Schmidt, P. and Shin, Y., (1992),
  Testing the Null Hypothesis of Stationarity Against the Alternative of
  a Unit Root: How Sure Are We That Economic Time Series Have a Unit
  Root?, \emph{Journal of Econometrics}, \bold{54}, 159--178.

  Download possible at: \url{https://cowles.yale.edu/}, see rubric
  'Discussion Papers (CFDPs)'.
}
\seealso{\code{\link{ur.kpss-class}}}
\examples{
data(nporg)
gnp <- na.omit(nporg[, "gnp.r"])
gnp.l <- log(gnp)
kpss.gnp <- ur.kpss(gnp.l, type="tau", lags="short")
summary(kpss.gnp)
}
\author{Bernhard Pfaff}
\keyword{regression}

\name{plotres}
\alias{plotres}
\encoding{latin1}
\title{Graphical inspection of VECM residuals}
\description{
  The function \code{plotres} should be used for graphical inspection
  of the VAR residuals, \emph{i.e.} the estimated specification as
  elaborated in the `Details' section of \code{ca.jo}. It displays the
  residuals for each equation within a VAR and their acf's and pacf's.
}
\usage{
plotres(x)
}
\arguments{
  \item{x}{Object of class `ca.jo'.}
}
\references{
  Johansen, S. and Juselius, K. (1990), Maximum Likelihood Estimation and
  Inference on Cointegration -- with Applications to the Demand for
  Money, \emph{Oxford Bulletin of Economics and Statistics}, \bold{52,
    2}, 169--210.
  
}
\seealso{
  \code{\link{ca.jo}} and \code{\link{ca.jo-class}}.
}
\examples{
data(denmark)
sjd <- denmark[, c("LRM", "LRY", "IBO", "IDE")]
sjd.vecm <- ca.jo(sjd, ecdet="const", type="eigen", K=2, spec="longrun",
season=4)
plotres(sjd.vecm)
}
\author{Bernhard Pfaff}
\keyword{regression}
\concept{VECM Residuals Johansen Juselius}

\name{ur.df}
\alias{ur.df}
\encoding{latin1}
\title{Augmented-Dickey-Fuller Unit Root Test}
\description{
  Performs the augmented Dickey-Fuller unit root test. 
}
\usage{
ur.df(y, type = c("none", "drift", "trend"), lags = 1,
      selectlags = c("Fixed", "AIC", "BIC")) 
}
\arguments{
  \item{y}{Vector to be tested for a unit root.}
  \item{type}{Test type, either \code{"none"}, \code{"drift"}  or
    \code{"trend"}.}
  \item{lags}{Number of lags for endogenous variable to be included.}
  \item{selectlags}{Lag selection can be achieved according to the
    Akaike \code{"AIC"} or the Bayes \code{"BIC"} information
    criteria. The maximum number of lags considered is set by
    \code{lags}. The default is to use a \code{"fixed"} lag length set
    by \code{lags}.} 
}
\details{
  The function \code{ur.df()} computes the augmented Dickey-Fuller
  test. If type is set to \code{"none"} neither an intercept nor a trend
  is included in the test regression. If it is set to \code{"drift"} an
  intercept is added and if it is set to \code{"trend"} both an intercept
    and a trend is added. The critical values are taken from Hamilton
    (1994) and Dickey and Fuller(1981).
  }
\value{
  An object of class \code{ur.df}.
}
\references{
  Dickey, D. A. and Fuller, W. A. (1979), Distributions of the
  Estimators For Autoregressive Time Series with a Unit Root,
  \emph{Journal of the American Statistical Association},  \bold{75},
  427--431.

  Dickey, D. A. and Fuller, W. A. (1981), Likelihood Ratio Statistics
  for Autoregressive Time Series with a Unit Root, \emph{Econometrica},
  \bold{49}, 1057--1072.
  
  Hamilton (1994), \emph{Time Series Analysis}, Princeton University Press.
}
\seealso{\code{\link{ur.df-class}}.}
\examples{
data(Raotbl3)
attach(Raotbl3)
lc.df <- ur.df(y=lc, lags=3, type='trend')
summary(lc.df)
}
\author{Bernhard Pfaff}
\keyword{regression}

\name{cajolst}
\alias{cajolst}
\encoding{latin1}
\title{Testing Cointegrating Rank with Level Shift at Unknown time}
\description{
  The function \code{cajolst} implements the procedure by Luetkepohl
  \emph{et al.} to test for the cointegration rank of a VAR process with
  a level shift at an unknown time.
}
\usage{
cajolst(x, trend = TRUE, K = 2, season = NULL)
}
\arguments{
  \item{x}{Data matrix to be investigated for cointegration.}
  \item{trend}{A linear trend is included in the auxiliary regressions
    for data adjustment (default is \code{TRUE}).}
  \item{K}{The lag order of the series (levels) in the VAR, must be at
    least equal to \eqn{K = 2}.}
  \item{season}{If seasonal dummies should be included, the data
    frequency must be set accordingly, \emph{i.e} \sQuote{4} for quarterly data.}
}
\details{
  Note, that the slot \code{"x"} of the returned object contains the
  adjusted data series, that is, a matrix adjusted for the temptative
  break point, and if applicable, a linear trend and/or seasonal
  effects. The VECM is then estimated and tested for cointegration rank
  subject to the adjusted matrix. The break point is contained in the
  slot \code{"bp"}. Please note, that the \emph{transitory} VECM
  specification is estimated and that only the trace test is
  available. The critical values are taken from Trenkler, Carsten (2003).
}
\value{
  Returns an object of class \code{ca.jo}.
}
\references{
  
  L\"utkepohl, H., Saikkonen, P. and Trenkler, C. (2004), Testing for the
  Cointegrating Rank of a VAR Process with Level Shift at Unknown Time,
  \emph{Econometrica}, \bold{Vol. 72, No. 2}, 647--662.

  Trenkler, Carsten (2003), A new set of critical values for systems
  cointegration tests with a prior adjustment for deterministic terms,
  \emph{Economics Bulletin}, \bold{Vol. 3, No. 11}, 1--9. 
  
}
\seealso{
  
  \code{\link{plotres}}, \code{\link{alrtest}}, \code{\link{ablrtest}},
  \code{\link{blrtest}}, \code{\link{ca.jo}}, \code{\link{cajools}},
  \code{\link{lttest}}, \code{\link{ca.jo-class}} and \code{\link{urca-class}}. 
}
\examples{
data(denmark)
sjd <- denmark[, c("LRM", "LRY", "IBO", "IDE")]
sjd.lst <- cajolst(sjd, trend=TRUE, K=2, season=4)
summary(sjd.lst)
}
\author{Bernhard Pfaff}
\keyword{regression}

\name{ur.pp-class}
\docType{class}
\alias{ur.pp-class}
\encoding{latin1}
\title{Representation of class ur.pp}
\description{
  This class contains the relevant information by applying the Phillips
  and Perron unit root test to a time series.
}
\section{Slots}{
  \describe{
    \item{\code{y}:}{Object of class \code{"vector"}: The time series to
      be tested.}
    \item{\code{type}:}{Object of class \code{"character"}: Test type of
      Z statistic, either \code{"Z-alpha"} or \code{"Z-tau"}.}
    \item{\code{model}:}{Object of class \code{"character"}: The type of
      the deterministic part, either \code{"constant"} or
      \code{"trend"}. The latter includes a constant term, too.}
    \item{\code{lag}:}{Object of class \code{"integer"}: Number of lags
      for error correction.}
    \item{\code{cval}:}{Object of class \code{"matrix"}: Critical values
      at the 1\%, 5\% and 10\% level of significance.}
    \item{\code{teststat}:}{Object of class \code{"numeric"}: Value of
      the test statistic.}
    \item{\code{testreg}:}{Object of class \code{"ANY"}: The summary
      output of the test regression.}
    \item{\code{auxstat}:}{Object of class \code{"matrix"}: Test
      statistic(s) of the deterministic part.}
    \item{\code{res}:}{Object of class \code{"vector"}: The residuals of
      the test regression.}
    \item{\code{test.name}:}{Object of class \code{"character"}: The
      name of the test, \emph{i.e} `Phillips-Perron'.}
  }
}
\section{Extends}{
Class \code{urca}, directly.
}
\section{Methods}{
  Type \code{showMethods(classes="ur.pp")} at the R prompt for a
  complete list of methods which are available for this class.
  
  Useful methods include
  \describe{
    \item{\code{show}:}{test statistic.}
    \item{\code{summary}:}{like show, but critical value and summary of
      test regression added.}
    \item{\code{plot}:}{Diagram of fit plot, residual plot and their
      acfs' and pacfs'.}
  }
}
\references{
  Phillips, P.C.B. and Perron, P. (1988), Testing for a unit root in
  time series regression, \emph{Biometrika},  \bold{75(2)}, 335--346.

  MacKinnon, J.G. (1991), Critical Values for Cointegration Tests,
  \emph{Long-Run Economic Relationships}, eds. R.F. Engle and
  C.W.J. Granger, London, Oxford, 267--276.
  
  Download possible at: \url{https://cowles.yale.edu/}, see rubric
  'Discussion Papers (CFDPs)'.
}
\seealso{
  \code{\link{ur.pp}} and \code{\link{urca-class}}
}
\author{Bernhard Pfaff}
\keyword{classes}

\name{ur.za}
\alias{ur.za}
\encoding{latin1}
\title{Zivot and Andrews Unit Root Test}
\description{
  Performs the Zivot and Andrews unit root test, which allows a
  break at an unknown point in either the intercept, the linear
  trend or in both.
}
\usage{
ur.za(y, model = c("intercept", "trend", "both"), lag=NULL)
}
\arguments{
  \item{y}{Vector to be tested for a unit root.}
  \item{model}{Specification if the potential break occured in either the intercept, the linear trend or in both.}
  \item{lag}{The highest number of lagged endogenous differenced variables to be included in the test regression}
}
\details{
  This test is based upon the recursive estimation of a test
  regression. The test statistic is defined as the minimum t-statistic
  of the coeffcient of the lagged endogenous variable. 
}
\value{
  An object of class \code{ur.za}.
}
\references{
  Zivot, E. and Andrews, Donald W.K. (1992), Further Evidence on the
  Great Crash, the Oil-Price Shock, and the Unit-Root Hypothesis,
  \emph{Journal of Business and Economic Statistics}, \bold{10(3)},
  251--270.

  Download possible at: \url{https://cowles.yale.edu/}, see rubric
  'Discussion Papers (CFDPs)'.
}
\seealso{\code{\link{ur.za-class}}}
\examples{
data(nporg)
gnp <- na.omit(nporg[, "gnp.r"])
za.gnp <- ur.za(gnp, model="both", lag=2)
summary(za.gnp)
}
\author{Bernhard Pfaff}
\keyword{regression}

\name{finland}
\alias{finland}
\docType{data}
\encoding{latin1}
\title{Data set for Finland, Johansen and Juseliues (1990)}
\description{
  This data set contains the series used by S. Johansen and K. Juselius
  for estimating a money demand function of Finland.
}
\usage{data(finland)}
\format{
  A data frame with 106 observations on the following 4 variables,
  ranging from 1958:Q2 until 1984:Q3.

    \tabular{rl}{
    \code{lrm1} \tab Logarithm of real money, M1. \cr
    \code{lny} \tab Logarithm of real income. \cr
    \code{lnmr} \tab Marginal rate of interest. \cr
    \code{difp} \tab Inflation rate. \cr
    }
}
\source{
  Johansen, S. and Juselius, K. (1990), Maximum Likelihood Estimation and
  Inference on Cointegration -- with Applications to the Demand for
  Money, \emph{Oxford Bulletin of Economics and Statistics}, \bold{52,
    2}, 169--210.
  
}
\references{
  \url{http://web.math.ku.dk/~sjo/}
}
\author{Bernhard Pfaff}
\keyword{datasets}

\name{ecb}
\alias{ecb}
\docType{data}
\encoding{latin1}
\title{Macroeconomic data of the Euro Zone}
\description{
  This data set contains some macroeconomic figures of the Euro Zone in
  order to estimate an exemplary money demand function.
}
\usage{data(ecb)}
\format{
  A data frame containing five series.
  
  \tabular{rl}{
    \code{period} \tab Time index from Q31997 until Q42003.\cr  
    \code{gdp.defl} \tab Gross Domestic Product Deflator,\cr 
    \tab [Index 2000=100, seasonally adjusted] \cr
    \code{gdp.nom} \tab Nominal Gross Domestic Product, \cr 
    \tab [Current prices, EUR billions, seasonally adjusted] \cr 
    \code{m3} \tab Monetary Aggregate M3, \cr
    \tab [outstanding amount at end of quarter, EUR billions, seasonally
    adjusted] \cr 
    \code{rl} \tab Benchmark Government Bond yield with a maturity of 10
    years, \cr
    \tab [percentages per annum, average of last quarter's month].
    }
}
\source{
  European Central Bank, Monthly Bulletins, Frankfurt am Main, Germany.
}
\references{
 \url{https://www.ecb.europa.eu}
}
\author{Bernhard Pfaff}
\keyword{datasets}
\concept{data set ECB money demand macroeconomic data Eurozone}

\name{lttest}
\alias{lttest}
\encoding{latin1}
\title{Likelihood ratio test for no linear trend in VAR}
\description{
  Conducts a likelihood ratio test for no inclusion of a linear trend in a
  VAR. That is, the Null hypothesis is for not including a linear trend
  and is assigned as 'H2*(r)'. The test statistic is distributed as
  \eqn{\chi^2} square with \eqn{(p-r)} degrees of freedom.
}
\usage{
lttest(z, r)
}
\arguments{
  \item{z}{An object of class `ca.jo'.}
  \item{r}{The count of cointegrating relationships.}
}
\details{
  The count of cointegrating relations should be given as integer and
  should be in the interval \eqn{1 \leq r < P}.
}
\value{
  \item{lttest}{Matrix containing the value of the test statistic and its
    p-value.}
}
\references{

  Johansen, S. and Juselius, K. (1990), Maximum Likelihood Estimation and
  Inference on Cointegration -- with Applications to the Demand for
  Money, \emph{Oxford Bulletin of Economics and Statistics}, \bold{52,
    2}, 169--210.

  Johansen, S. (1991), Estimation and Hypothesis Testing of
  Cointegration Vectors in Gaussian Vector Autoregressive Models,
  \emph{Econometrica}, \bold{Vol. 59, No. 6}, 1551--1580.

}
\seealso{
  \code{\link{ca.jo}} and \code{\link{ca.jo-class}}.
}
\examples{
data(denmark)
sjd <- as.matrix(denmark[, c("LRM", "LRY", "IBO", "IDE")])
sjd.vecm <- ca.jo(sjd, ecdet = "const", type="eigen", K=2, spec="longrun",
season=4)
lttest(sjd.vecm, r=1)
#
data(finland)
sjf <- as.matrix(finland)
sjf.vecm <- ca.jo(sjf, ecdet = "none", type="eigen", K=2,
spec="longrun", season=4)
lttest(sjf.vecm, r=3)
}
\author{Bernhard Pfaff}
\keyword{regression}
\concept{VECM Test Linear Trend Johansen Juselius}

\name{ca.po-class}
\docType{class}
\alias{ca.po-class}
\encoding{latin1}
\title{Representation of class ca.po}
\description{
  This class contains the relevant information by applying the Phillips
  and Ouliaris cointegration test to a data matrix.
}
\section{Slots}{
  \describe{
    \item{\code{z}:}{Object of class \code{"ANY"}: A data matrix, or an
      object that can be coerced to it.}
    \item{\code{type}:}{Object of class \code{"character"}: The type of
      the test, either the \code{"Pu"}-test or the normalisation
	invariant \code{"Pz"}-test.}
    \item{\code{model}:}{Object of class \code{"character"}: Determines
      how the series should be detrended.}
    \item{\code{lag}:}{Object of class \code{"integer"}: The lags used
      for variance/covariance correction.}
    \item{\code{cval}:}{Object of class \code{"matrix"}: The critical
      values of the test at the 1\%, 5\% and 10\% level of significance.}
    \item{\code{res}:}{Object of class \code{"matrix"}: The residuals of
      the the cointegration regression(s).}
    \item{\code{teststat}:}{Object of class \code{"numeric"}: The value
      of the test statistic.}
    \item{\code{testreg}:}{Object of class \code{"ANY"}: The summary
      output of the cointegration regression(s).}
    \item{\code{test.name}:}{Object of class \code{"character"}: The
      name of the test, \emph{i.e.} `Phillips and Ouliaris'.}
  }
}
\section{Extends}{
Class \code{urca}, directly.
}
\section{Methods}{
  Type \code{showMethods(classes="ca.po")} at the R prompt for a
  complete list of methods which are available for this class.
  
  Useful methods include
  \describe{
    \item{\code{show}:}{test statistic.}
    \item{\code{summary}:}{like show, but critical value and summary of
      test regression(s) added.}
    \item{\code{plot}:}{Residual plot(s) and their acfs' and pacfs'.}
  }
}
\references{
  Phillips, P.C.B. and Ouliaris, S. (1990), Asymptotic Properties of
  Residual Based Tests for Cointegration, \emph{Econometrica},
  \bold{Vol. 58, No. 1}, 165--193.
}
  
\seealso{
  \code{\link{ca.po}} and \code{\link{urca-class}}. 
}
\author{Bernhard Pfaff}
\keyword{classes}

\name{UKconsumption}
\alias{UKconsumption}
\docType{data}
\encoding{latin1}
\title{Data set for the United Kingdom}
\description{
  This data set contains the series used by in Charemza, W. (1997), New
  Directions in Econometric Practice, 2nd edition, Edward Elgar,
  Cheltenha, Uk. for analysing private in the United Kingdom.
}
\usage{data(UKconsumption)}
\format{
  A data frame of quarterly \code{ts} objects ranging from 1957:Q1
  until 1975:Q4.
 

    \tabular{rl}{
    \code{cons} \tab Consumers` non-durable expenditure in the U.K. in
    1970 prices. \cr
    \code{inc} \tab Personal disposable income in the U.K. in 1970
    prices. \cr
    \code{price} \tab Consumers` expenditure deflator index, 1970=100.
    }
}
\source{
  Pokorny, M. (1987), An Introduction to Econometrics, page 408, Basil
  Blackwell Ltd. Original data source: Economic Trends, Annual
  Supplements, 1976 and 1981, HMSO.
}
\references{
  Charemza, W. (1997), New Directions in Econometrics Practice, 2nd
  edition, Edward Elgar, Cheltenham, U.K.
}
\author{Bernhard Pfaff}
\keyword{datasets}
\concept{data set UK consumption Charemza}

\name{ur.ers}
\alias{ur.ers}
\encoding{latin1}
\title{Elliott, Rothenberg and Stock Unit Root Test}
\description{
  Performs the Elliott, Rothenberg and Stock unit root test. 
}
\usage{
ur.ers(y, type = c("DF-GLS", "P-test"), model = c("constant", "trend"),
       lag.max = 4)
}
\arguments{
  \item{y}{Vector to be tested for a unit root.}
  \item{type}{Test type, either \code{"DF-GLS"} (default), or \code{"P-test"}.}
  \item{model}{The deterministic model used for detrending.}
  \item{lag.max}{The maximum numbers of lags used for testing of a
    decent lag truncation for the \code{"P-test"} (\code{BIC} used),
    or the maximum number of lagged differences to be included in the test
    regression for \code{"DF-GLS"}.}
}
\details{
  To improve the power of the unit root test, Elliot, Rothenberg and Stock
  proposed a local to unity detrending of the time series. ERS developed
  a feasible point optimal test, \code{"P-test"}, which takes serial
  correlation of the error term into account. The second test type is
  the \code{"DF-GLS"} test, which is an ADF-type test applied to the
  detrended data without intercept. Critical values for this test are
  taken from MacKinnon in case of \code{model="constant"} and else from
  Table 1 of Elliot, Rothenberg and Stock.
}
\value{
  An object of class \code{ur.ers}.
}

\references{
  Elliott, G., Rothenberg, T.J. and Stock, J.H. (1996), Efficient Tests
  for an Autoregressive Unit Root, \emph{Econometrica},
  \bold{Vol. 64, No. 4}, 813--836.

  MacKinnon, J.G. (1991), Critical Values for Cointegration Tests,
  \emph{Long-Run Economic Relationships}, eds. R.F. Engle and
  C.W.J. Granger, London, Oxford, 267--276.

}

\seealso{\code{\link{ur.ers-class}}}
\examples{
data(nporg)
gnp <- na.omit(nporg[, "gnp.r"])
ers.gnp <- ur.ers(gnp, type="DF-GLS", model="const", lag.max=4)
summary(ers.gnp)
}
\author{Bernhard Pfaff}
\keyword{regression}

\name{Raotbl5} 
\alias{Raotbl5}
\encoding{latin1}
\docType{data}
\title{Data set used by Pierre Perron (1994)}
\description{
  This data set contains the time series used by Pierre Perron in his
  article: ''Trend, Unit Root and Structural Change in Macroeconomic
  Time Series". 
}
\usage{data(Raotbl5)}
\format{
  A data frame on real aggregate output for various countries; annual
  data starting in 1870 until 1986.
  
    \tabular{rl}{
    \code{ita} \tab Italy. \cr
    \code{nor} \tab Norway. \cr
    \code{swe} \tab Sweden. \cr
    \code{ukg} \tab United Kingdom. \cr
    \code{usa} \tab United States of America.
  }

  For further details about the data see \emph{Notes} in the data
  appendix `Table D.5' of Rao, ``Cointegration for the Applied
  Economist".

}
\source{
  Pierre Perron (1994), Trend, Unit Root and Structural Change in
  Macroeconomic Time Series, in:  Cointegration for the Applied Economist,
  ed. B. Bhaskara Rao, chapter 4, Data Appendix, Table D.5.
}
\author{Bernhard Pfaff}
\keyword{datasets}
\concept{data set Rao Perron}

\name{plot-methods}
\docType{methods}
\alias{plot}
\alias{plot-methods}
\alias{plot,ur.ers,missing-method}
\alias{plot,ur.kpss,missing-method}
\alias{plot,ca.jo,missing-method}
\alias{plot,ca.po,missing-method}
\alias{plot,ur.pp,missing-method}
\alias{plot,ur.sp,missing-method}
\alias{plot,ur.za,missing-method}
\alias{plot,ur.df,missing-method}
\encoding{latin1}
\title{Methods for Function plot in Package urca}
\description{
  Plot methods for objects belonging to classes set in package
  \code{urca}. Depending on the unit root/cointegration test a
  suitable graphical presentation is selected.
  }
\section{Methods}{\describe{

\item{x = "ur.ers", y = "missing"}{Diagram of fit of the Elliott,
  Rothenberg and Stock unit root test of type \code{"DF-GLS"} with
  residual plot and their acfs' and pacfs'.}
\item{x = "ur.kpss", y = "missing"}{Residual plot and their acfs' and
  pacfs' of the KPSS test.}
\item{x = "ca.jo", y = "missing"}{Time series plots and associated
  cointegration relations for the Johansen procedure.}
\item{x = "ca.po", y = "missing"}{Residual plot and their acfs' and
  pacfs' of the cointegration regression(s) for the Phillips and Ouliaris
  test.}
\item{x = "ur.pp", y = "missing"}{Diagram of fit of the Phillips and
  Perron unit root test, residual plot and their acfs' and pacfs'.}
\item{x = "ur.sp", y = "missing"}{Diagram of fit of the Schmidt and
  Phillips unit root test, residual plot and their acfs' and pacfs'.}
\item{x = "ur.za", y = "missing"}{Plot of recursive t-statistics
  as outcome of Zivot and Andrews unit root test.}
}}
\seealso{
  \code{\link{ur.ers-class}}, \code{\link{ur.kpss-class}},
  \code{\link{ca.jo-class}}, \code{\link{ca.po-class}},
  \code{\link{ur.pp-class}}, \code{\link{ur.sp-class}} and
  \code{\link{ur.za-class}}.}
\examples{
data(nporg)
gnp <- na.omit(nporg[, "gnp.r"])
gnp.l <- log(gnp)
#
ers.gnp <- ur.ers(gnp, type="DF-GLS", model="trend", lag.max=4)
plot(ers.gnp)
#
kpss.gnp <- ur.kpss(gnp.l, type="tau", lags="short")
plot(kpss.gnp)
#
pp.gnp <- ur.pp(gnp, type="Z-tau", model="trend", lags="short")
plot(pp.gnp)
#
sp.gnp <- ur.sp(gnp, type="tau", pol.deg=1, signif=0.01)
plot(sp.gnp)
#
za.gnp <- ur.za(gnp, model="both", lag=2)
plot(za.gnp)
#
data(denmark)
sjd <- denmark[, c("LRM", "LRY", "IBO", "IDE")]
sjd.vecm <- ca.jo(sjd, ecdet="const", type="eigen", K=2, season=4)
plot(sjd.vecm)
}
\author{Bernhard Pfaff}
\keyword{methods}

\name{ca.jo}
\alias{ca.jo}
\encoding{latin1}
\title{Johansen Procedure for VAR}
\description{
  Conducts the Johansen procedure on a given data set. The
  \code{"trace"} or \code{"eigen"} statistics are reported and the
  matrix of eigenvectors as well as the loading matrix.
}
\usage{
ca.jo(x, type = c("eigen", "trace"), ecdet = c("none", "const", "trend"), K = 2,
spec=c("longrun", "transitory"), season = NULL, dumvar = NULL)
}
\arguments{
  \item{x}{Data matrix to be investigated for cointegration.}
  \item{type}{The test to be conducted, either \sQuote{\code{eigen}} or
    \sQuote{\code{trace}}.}
  \item{ecdet}{Character, \sQuote{\code{none}} for no intercept in cointegration,
    \sQuote{\code{const}} for constant term in cointegration and
    \sQuote{\code{trend}} for trend variable in cointegration.}
  \item{K}{The lag order of the series (levels) in the VAR.}
  \item{spec}{Determines the specification of the VECM, see details below.}
  \item{season}{If seasonal dummies should be included, the data
    frequency must be set accordingly, \emph{i.e} \sQuote{4} for quarterly
    data.}
  \item{dumvar}{If dummy variables should be included, a matrix with
    row dimension equal to \code{x} can be provided.} 
}

\details{

  Given a general VAR of the form:

  \deqn{\bold{X}_t = \bold{\Pi}_1 \bold{X}_{t-1} + \dots + \bold{\Pi}_k
    \bold{X}_{t-k} + \bold{\mu} + \bold{\Phi D}_t + \bold{\varepsilon}_t
    , \quad (t = 1, \dots, T),}

  the following two specifications of a VECM exist:

  \deqn{\Delta \bold{X}_t = \bold{\Gamma}_1 \Delta \bold{X}_{t-1} +
    \dots + \bold{\Gamma}_{k-1} \Delta \bold{X}_{t-k+1} + \bold{\Pi
    X}_{t-k} + \bold{\mu} + \bold{\Phi D}_t + \bold{\varepsilon}_t}

  where

  \deqn{\bold{\Gamma}_i = - (\bold{I} - \bold{\Pi}_1 - \dots -
    \bold{\Pi}_i), \quad (i = 1, \dots , k-1),}

  and

  \deqn{\bold{\Pi} = -(\bold{I} - \bold{\Pi}_1 - \dots - \bold{\Pi}_k)}

  The \eqn{\bold{\Gamma}_i} matrices contain the cumulative long-run
  impacts, hence if \code{spec="longrun"} is choosen, the above VECM is
  estimated.

  The other VECM specification is of the form:

  \deqn{\Delta \bold{X}_t = \bold{\Gamma}_1 \Delta \bold{X}_{t-1} +
    \dots + \bold{\Gamma}_{k-1} \Delta \bold{X}_{t-k+1} + \bold{\Pi
    X}_{t-1} + \bold{\mu} + \bold{\Phi D}_t + \bold{\varepsilon}_t}

  where

  \deqn{\bold{\Gamma}_i = - (\bold{\Pi}_{i+1} + \dots + \bold{\Pi}_k),
    \quad(i = 1, \dots , k-1),}

  and

  \deqn{\bold{\Pi} = -(\bold{I} - \bold{\Pi}_1 - \dots - \bold{\Pi}_k).}

  The \eqn{\bold{\Pi}} matrix is the same as in the first specification.
  However, the \eqn{\bold{\Gamma}_i} matrices now differ, in the sense
  that they measure transitory effects, hence by setting
  \code{spec="transitory"} the second VECM form is estimated. Please note
  that inferences drawn on \eqn{\bold{\Pi}} will be the same, regardless
  which specification is choosen and that the explanatory power is the
  same, too. 

  If \code{"season"} is not NULL, centered seasonal dummy variables are
  included. 

  If \code{"dumvar"} is not NULL, a matrix of dummy variables is included
  in the VECM. Please note, that the number of rows of the matrix
  containing the dummy variables must be equal to the row number of
  \code{x}.

  Critical values are only reported for systems with less than
  11 variables and are taken from Osterwald-Lenum.
  
}

\value{
  An object of class \code{ca.jo}.
}

\references{

  Johansen, S. (1988), Statistical Analysis of Cointegration Vectors,
  \emph{Journal of Economic Dynamics and Control}, \bold{12}, 231--254.

  Johansen, S. and Juselius, K. (1990), Maximum Likelihood Estimation and
  Inference on Cointegration -- with Applications to the Demand for
  Money, \emph{Oxford Bulletin of Economics and Statistics}, \bold{52,
    2}, 169--210.

  Johansen, S. (1991), Estimation and Hypothesis Testing of
  Cointegration Vectors in Gaussian Vector Autoregressive Models,
  \emph{Econometrica}, \bold{Vol. 59, No. 6}, 1551--1580.

  Osterwald-Lenum, M. (1992), A Note with Quantiles of the Asymptotic
  Distribution of the Maximum Likelihood Cointegration Rank Test
  Statistics, \emph{Oxford Bulletin of Economics and Statistics},
  \bold{55, 3}, 461--472.

}

\seealso{
  
  \code{\link{plotres}}, \code{\link{alrtest}}, \code{\link{ablrtest}},
  \code{\link{blrtest}}, \code{\link{cajolst}}, \code{\link{cajools}},
  \code{\link{lttest}}, \code{\link{ca.jo-class}} and \code{\link{urca-class}}. 
}
\examples{
data(denmark)
sjd <- denmark[, c("LRM", "LRY", "IBO", "IDE")]
sjd.vecm <- ca.jo(sjd, ecdet = "const", type="eigen", K=2, spec="longrun",
season=4)
summary(sjd.vecm)
#
data(finland)
sjf <- finland
sjf.vecm <- ca.jo(sjf, ecdet = "none", type="eigen", K=2,
spec="longrun", season=4)
summary(sjf.vecm)
}
\author{Bernhard Pfaff}
\keyword{regression}

\name{Raotbl6} 
\alias{Raotbl6}
\encoding{latin1}
\docType{data}
\title{Data set used by Yash P. Mehra (1994)}
\description{
  This data set contains quarterly data for the U.S.A. in Yash
  P. Mehra's article: ``Wage Growth and the Inflation Process: An
  Empirical Approach" for his wage-price equations. 
}
\usage{data(Raotbl6)}
\format{
  A data frame with quarterly data from 1959:1 until 1989:3. 
  
    \tabular{rl}{
    \code{rgnp} \tab Real GNP. \cr
    \code{pgnp} \tab Potential real GNP. \cr
    \code{ulc} \tab Unit labor cost. \cr
    \code{gdfco} \tab Fixed weight deflator for personal consumption
    expenditure excluding food and energy.\cr
    \code{gdf} \tab Fixed weight GNP deflator. \cr
    \code{gdfim} \tab Fixed weight import deflator. \cr
    \code{gdfcf} \tab Fixed weight deflator for food in personal consumption
    expenditure.  \cr
    \code{gdfce} \tab Fixed weight deflator for energy in personal
    consumption expenditure. 
  }

  Further details about the data can be found in the data appendix of
  Rao, ``Cointegration for the Applied Economist".
  
}
\source{
  Yash P. Mehra (1994), Wage Growth and the Inflation Process: An
  Empirical Approach, in:  Cointegration for the Applied Economist,
  ed. B. Bhaskara Rao, chapter 5, Data Appendix, Table D.6.
}
\author{Bernhard Pfaff}
\keyword{datasets}
\concept{data set Rao Mehra}

\name{nporg}
\docType{data}
\alias{nporg}
\encoding{latin1}
\title{Nelson and Plosser original data set}
\description{
  This data set contains the fourteen U.S. economic time series used by
  Nelson and Plosser in their seminal paper.
}
\usage{data(nporg)}
\format{
  A data frame containing fourteen series.

  \tabular{rl}{
    \code{year} \tab Time index from 1860 until 1970. \cr 
    \code{gnp.r} \tab Real GNP, \cr
    \tab [Billions of 1958 Dollars], [1909 -- 1970] \cr
    \code{gnp.n} \tab Nominal GNP, \cr
    \tab [Millions of Current Dollars], [1909 -- 1970] \cr
    \code{gnp.pc} \tab Real Per Capita GNP, \cr
    \tab [1958 Dollars], [1909 -- 1970] \cr
    \code{ip} \tab Industrial Production Index, \cr
    \tab [1967 = 100], [1860 -- 1970] \cr
    \code{emp} \tab Total Employment, \cr
    \tab [Thousands], [1890 -- 1970] \cr
    \code{ur} \tab Total Unemployment Rate, \cr
    \tab [Percent], [1890 -- 1970] \cr
    \code{gnp.p} \tab GNP Deflator, \cr
    \tab [1958 = 100], [1889 -- 1970] \cr
    \code{cpi} \tab Consumer Price Index, \cr
    \tab [1967 = 100], [1860 -- 1970] \cr
    \code{wg.n} \tab Nominal Wages \cr
    \tab (Average annual earnings per full-time employee in manufacturing), \cr
    \tab [current Dollars], [1900 -- 1970] \cr
    \code{wg.r} \tab Real Wages, \cr
    \tab [Nominal wages/CPI], [1900 -- 1970] \cr
    \code{M} \tab Money Stock (M2), \cr
    \tab [Billions of Dollars, annual averages], [1889 -- 1970] \cr
    \code{vel} \tab Velocity of Money, \cr
    \tab [1869 -- 1970] \cr
    \code{bnd} \tab Bond Yield (Basic Yields of 30-year
        corporate bonds), \cr
    \tab [Percent per annum], [1900 -- 1970] \cr
    \code{sp} \tab Stock Prices, \cr
    \tab [Index; 1941 -- 43 = 100], [1871 -- 1970] \cr
    }
}
\source{
  Nelson, C.R. and Plosser, C.I. (1982), Trends and Random Walks in
  Macroeconomic Time Series, \emph{Journal of Monetary Economics},
  \bold{10}, 139--162.
}
\references{
  \url{http://korora.econ.yale.edu/phillips/index.htm}
}
\author{Bernhard Pfaff}
\keyword{datasets}
\concept{data set Nelson Plosser macroeconomic variables}

\name{ca.po}
\alias{ca.po}
\encoding{latin1}
\title{Phillips and Ouliaris Cointegration Test}
\description{
  Performs the Phillips and Ouliaris \code{"Pu"} and \code{"Pz"}
  cointegration test.
}
\usage{
ca.po(z, demean = c("none", "constant", "trend"),
      lag = c("short", "long"), type = c("Pu", "Pz"), tol = NULL)
}
\arguments{
  \item{z}{Data matrix to be investigated for cointegration.}
  \item{demean}{The method for detrending the series, either
    \code{"none"}, \code{"constant"} or \code{"trend"}.}
  \item{lag}{Either a short or long lag number used for
    variance/covariance correction.}
  \item{type}{The test type, either \code{"Pu"} or \code{"Pz"}.}
  \item{tol}{Numeric, this argument is passed to \code{solve()} in
    \code{ca.po()}.}
}
\details{
  The test \code{"Pz"}, compared to the test \code{"Pu"}, has the
  advantage that it is invariant to the normalization of the
  cointegration vector, \emph{i.e.} it does not matter which variable
  is on the left hand side of the equation. In case convergence
  problems are encountered by matrix inversion, one can pass a higher
  tolerance level \emph{via} \code{"tol=..."} to the \code{solve()}-function.
}
\value{
  An object of class \code{ca.po}.
}
\references{
  Phillips, P.C.B. and Ouliaris, S. (1990), Asymptotic Properties of
  Residual Based Tests for Cointegration, \emph{Econometrica},
  \bold{Vol. 58, No. 1}, 165--193.
}
  
\seealso{\code{\link{ca.po-class}}}
\examples{
data(ecb)
m3.real <- ecb[,"m3"]/ecb[,"gdp.defl"]
gdp.real <- ecb[,"gdp.nom"]/ecb[,"gdp.defl"]
rl <- ecb[,"rl"]
ecb.data <- cbind(m3.real, gdp.real, rl)
m3d.po <- ca.po(ecb.data, type="Pz")
summary(m3d.po)
}
\author{Bernhard Pfaff}
\keyword{regression}

\name{bh6lrtest}
\alias{bh6lrtest}
\encoding{latin1}
\title{Likelihood ratio test for restrictions under partly known beta in
a subspace}
\description{
  This function estimates a restricted VAR, where some restrictions are
  placed on \eqn{r1} cointegrating relations which are chosen in the
  space of the matrix H. The test statistic is distributed as
  \eqn{\chi^2} with \eqn{(p-s-r2)r1} degrees of freedom, with \eqn{s}
  equal to the number of columns of \eqn{\bold{H}}, \eqn{r1} the number
  of cointegrating relations in the first partition and \eqn{r2} the
  number of cointegrating relations in the second partition which will
  be estimated without any restrictions.
}
\usage{
bh6lrtest(z, H, r, r1, conv.val = 0.0001, max.iter = 50)
}
\arguments{
  \item{z}{An object of class \code{ca.jo}.}
  \item{H}{The \eqn{(p \times s)} matrix containing the known
    cointegration relations.}
  \item{r}{The count of cointegrating relationships; \cr
    inferred from \code{summary(ca.jo-object)}.\cr}
  \item{r1}{The count of cointegrating relationships in the first
    partition of the cointegration space; \cr}
  \item{conv.val}{The convergence value of the algorithm. (see details); \cr}
  \item{max.iter}{The maximal number of iterations.} 
}

\details{
  Please note, that the following ordering of the dimensions should be
  obeyed: \eqn{r1 \leq s \leq p - r2}. A two-step algorithm is used to
  determine the eigen values of the restricted model. Convergence is
  achieved if the quadratic norm of the eigen values is smaller than
  \code{conv.val}.
}
\value{
  An object of class \code{cajo.test}.
}
\references{

Johansen, S. (1995), \emph{Likelihood-Based Inference in Cointegrated Vector
Autoregressive Models}, Oxford University Press, Oxford.

Johansen, S. and Juselius, K. (1992), Testing structural hypotheses in a
multivariate cointegration analysis of the PPP and the UIP for UK,
\emph{Journal of Econometrics}, \bold{53}, 211--244.

}
\seealso{
  \code{\link{ca.jo}}, \code{\link{alrtest}}, \code{\link{ablrtest}},
  \code{\link{blrtest}}, \code{\link{bh5lrtest}}, \code{\link{cajo.test-class}},
  \code{\link{ca.jo-class}} and \code{\link{urca-class}}. 
}
\examples{
data(UKpppuip)
attach(UKpppuip)
dat1 <- cbind(p1, p2, e12, i1, i2)
dat2 <- cbind(doilp0, doilp1)
H1 <- ca.jo(dat1, type='trace', K=2, season=4, dumvar=dat2)
H6 <- matrix(c(1,0,0,0,0, 0,1,0,0,0, 0,0,1,0,0), c(5,3))
bh6lrtest(z=H1, H=H6, r=2, r1=1, conv.val=0.0001, max.iter=50)
}
\author{Bernhard Pfaff}
\keyword{regression}

\name{bh5lrtest}
\alias{bh5lrtest}
\encoding{latin1}
\title{Likelihood ratio test for restrictions under partly known beta}
\description{
  This function estimates a restricted VAR, where some of the
  cointegration vectors are known. The known cointegration relationships
  have to be provided in an \eqn{p x r1} matrix \eqn{\bold{H}}. The test
  statistic is distributed as \eqn{\chi^2} with \eqn{(p-r)r1} degrees of
  freedom, with \eqn{r} equal to total number of cointegration relations.
}
\usage{
bh5lrtest(z, H, r)
}
\arguments{
  \item{z}{An object of class \code{ca.jo}.}
  \item{H}{The \eqn{(p \times r1)} matrix containing the known
    cointegration relations.}
  \item{r}{The count of cointegrating relationships; \cr
    inferred from \code{summary(ca.jo-object)}.}
}
\details{
  Please note, that the number of columns of \eqn{\bold{H}} must be
  smaller than the count of cointegration relations \eqn{r}.
}
\value{
  An object of class \code{cajo.test}.
}
\references{

Johansen, S. (1995), \emph{Likelihood-Based Inference in Cointegrated Vector
Autoregressive Models}, Oxford University Press, Oxford.

Johansen, S. and Juselius, K. (1992), Testing structural hypotheses in a
multivariate cointegration analysis of the PPP and the UIP for UK,
\emph{Journal of Econometrics}, \bold{53}, 211--244.

}
\seealso{
  \code{\link{ca.jo}}, \code{\link{alrtest}}, \code{\link{ablrtest}},
  \code{\link{blrtest}}, \code{\link{bh6lrtest}}, \code{\link{cajo.test-class}},
  \code{\link{ca.jo-class}} and \code{\link{urca-class}}. 
}
\examples{
data(UKpppuip)
attach(UKpppuip)
dat1 <- cbind(p1, p2, e12, i1, i2)
dat2 <- cbind(doilp0, doilp1)
H1 <- ca.jo(dat1, type='trace', K=2, season=4, dumvar=dat2)
H51 <- c(1, -1, -1, 0, 0)
H52 <- c(0, 0, 0, 1, -1)
summary(bh5lrtest(H1, H=H51, r=2))
summary(bh5lrtest(H1, H=H52, r=2))
}
\author{Bernhard Pfaff}
\keyword{regression}

\name{show.urca}
\alias{show.urca}
\encoding{latin1}
\title{Function to show objects of classes for unit root tests}
\description{
  The function \code{show.urca} is called within the defined methods
  for classes \code{ca.jo}, \code{cajo.test}, \code{ca.po},
  \code{ur.df}, \code{ur.ers}, \code{ur.kpss}, \code{ur.po}, \code{ur.pp},
  \code{ur.df}, \code{ur.sp} and \code{ur.za}.
 
}
\usage{
show.urca(object)
}
\arguments{
  \item{object}{Object of class contained in \code{urca}.}
}
\details{
  This function is called by method \code{show}.
}
\value{
  The Name and test statistic of a unit root/cointegration test.
}
\author{Bernhard Pfaff}
\keyword{methods}

\name{cajorls}
\alias{cajorls}
\encoding{latin1}
\title{OLS regression of VECM}
\description{
  This function returns the OLS regressions of a restricted VECM,
  \emph{i.e.} it returns a list object with elements of class `lm'
  containing the restricted VECM and a matrix object with the normalised
  cointegrating relationships. The user can provide a certain number of
  which equation in the VECM should be estimated and reported, or if
  \code{"reg.number = NULL"} each equation in the VECM will be estimated
  and its results are reported. Furthermore, the cointegratioon rank has
  to be supplied too.
}
\usage{
cajorls(z, r = 1, reg.number = NULL)
}
\arguments{
  \item{z}{An object of class \code{ca.jo} or \code{cajo.test}.}
  \item{r}{An integer, signifiying the cointegration rank.}
  \item{reg.number}{The number of the equation in the VECM that should
    be estimated or if set to \code{NULL} (the default), all equations
    within the VECM are estimated.}
}
\details{
 The cointegration space is normalised as \eqn{\bold{\beta}_c =
   \bold{\beta}(S'\bold{\beta})^{-1}}, with \eqn{S' = (I_r, 0)}. 
}
\value{
  Returns a list object with elements of class \code{lm} for the
  restricted VECM and a matrix object with the normalised cointegrating
  vectors. 
}
\references{

  Johansen, S. (1995), \emph{Likelihood-Based Inference in Cointegrated Vector
    Autoregressive Models}, Oxford University Press, Oxford.

  Lütkepohl, H. (2006), \emph{New Introduction to Multiple Time Series
    Analysis}, Springer, New York.

}
\seealso{
  \code{\link{ca.jo}}, \code{\link{cajools}}, \code{\link[stats]{lm}},
  \code{\link{ca.jo-class}} and \code{\link{urca-class}}. 
}
\examples{
data(finland)
sjf <- finland
sjf.vecm <- ca.jo(sjf, ecdet = "none", type = "eigen", K = 2,
spec = "longrun", season = 4)
sjf.vecm.rls <- cajorls(sjf.vecm, r = 2)
summary(sjf.vecm.rls$rlm)
sjf.vecm.rls$beta
}
\author{Bernhard Pfaff}
\keyword{regression}
\concept{VECM OLS Johansen Juselius Cointegration Co-integration}

\name{ur.za-class}
\docType{class}
\alias{ur.za-class}
\encoding{latin1}
\title{Representation of class ur.za}
\description{
  This class contains the relevant information by applying the Zivot and Andrews
  unit root test to a time series.}
\section{Slots}{
  \describe{
    \item{\code{y}:}{Object of class \code{"vector"}: The time series to
      be tested.}
    \item{\code{model}:}{Object of class \code{"character"}: The model
      to be used, \emph{i.e.} intercept, trend or both}
    \item{\code{lag}:}{Object of class \code{"integer"}: The highest
      number of lags to include in the test regression.}
    \item{\code{teststat}:}{Object of class \code{"numeric"}: The t-statistic.}
    \item{\code{cval}:}{Object of class \code{"vector"}: Critical values
      at the 1\%, 5\% and 10\% level of significance.}
    \item{\code{bpoint}:}{Object of class \code{"integer"}: The
      potential break point.}
    \item{\code{tstats}:}{Object of class \code{"vector"} The
      t-statistics of the rolling regression.}
    \item{\code{res}:}{Object of class \code{"vector"} The residuals of
      the test regression.}
    \item{\code{test.name}:}{Object of class \code{"character"} The name
      of the test, \emph{i.e.} `Zivot and Andrews'.}
    \item{\code{testreg}:}{Object of class \code{"ANY"} The summary
      output of the test regression.}
  }
}
\section{Extends}{
Class \code{urca}, directly.
}
\section{Methods}{
  Type \code{showMethods(classes="ur.za")} at the R prompt for a
  complete list of methods which are available for this class.
  
  Useful methods include
  \describe{
    \item{\code{show}:}{test statistic and critical values.}
    \item{\code{summary}:}{like show, but summary of test regression
      added.}
    \item{\code{plot}:}{plot of recursive t-statistics.}
  }
}
\references{
  Zivot, E. and Andrews, Donald W.K. (1992), Further Evidence on the
  Great Crash, the Oil-Price Shock, and the Unit-Root Hypothesis,
  \emph{Journal of Business and Economic Statistics}, \bold{10(3)},
  251--270.

  Download possible at: \url{https://cowles.yale.edu/}, see rubric
  'Discussion Papers (CFDPs)'.
}
\seealso{
  \code{\link{ur.za}} and \code{\link{urca-class}}. 
}
\author{Bernhard Pfaff}
\keyword{classes}

\name{urca-internal}
\alias{.spcv}
\encoding{latin1}
\title{Critical values for Schmidt and Phillips Unit Root Test}
\description{
  This function is an internal function and is called by
  \code{ur.sp}. It computes the critical value of the Schmidt and
  Phillips test, given a level of significance, the polynomial degree of
  the test regression, the test type and the sample size.
}
\usage{
.spcv(obs, type, pol.deg, signif)
}
\arguments{
  \item{obs}{The sample size.}
  \item{type}{The test type.}
  \item{pol.deg}{The polynomial degree.}
  \item{signif}{The significance level.}
}
\value{
  The critical value of the test.
}
\references{
  Schmidt, P. and Phillips, P.C.B. (1992), LM Test for a Unit Root in
  the Presence of Deterministic Trends, \emph{Oxford Bulletin of Economics and
  Statistics}, \bold{54(3)}, 257--287.

  Download possible at: \url{https://cowles.yale.edu/}, see rubric
  'Discussion Papers (CFDPs)'.
}
\seealso{\code{\link{ur.sp}}.}
\author{Bernhard Pfaff}
\keyword{regression}

\name{ur.pp}
\alias{ur.pp}
\encoding{latin1}
\title{Phillips and Perron Unit Root Test}
\description{
  Performs the Phillips and Perron unit root test. Beside the Z
  statistics Z-alpha and Z-tau, the Z statistics for the deterministic
  part of the test regression are computed, too.
}
\usage{
ur.pp(x, type = c("Z-alpha", "Z-tau"), model = c("constant", "trend"),
      lags = c("short", "long"), use.lag = NULL)
}
\arguments{
  \item{x}{Vector to be tested for a unit root.}
  \item{type}{Test type, either \code{"Z-alpha"} or \code{"Z-tau"}.}
  \item{model}{Determines the deterministic part in the test regression.}
  \item{lags}{Lags used for correction of error term.}
  \item{use.lag}{Use of a different lag number, specified by the user.}
}
\details{
  The function \code{ur.pp()} computes the Phillips and Perron test. For
  correction of the error term a Bartlett window is used. 
}
\value{
  An object of class \code{ur.pp}.
}
\references{
  Phillips, P.C.B. and Perron, P. (1988), Testing for a unit root in
  time series regression, \emph{Biometrika},  \bold{75(2)}, 335--346.

  MacKinnon, J.G. (1991), Critical Values for Cointegration Tests,
  \emph{Long-Run Economic Relationships}, eds. R.F. Engle and
  C.W.J. Granger, London, Oxford, 267--276.
  
  Download possible at: \url{https://cowles.yale.edu/}, see rubric
  'Discussion Papers (CFDPs)'.
}
\seealso{\code{\link{ur.pp-class}}.}
\examples{
data(nporg)
gnp <- na.omit(nporg[, "gnp.r"])
pp.gnp <- ur.pp(gnp, type="Z-tau", model="trend", lags="short")
summary(pp.gnp)
}
\author{Bernhard Pfaff}
\keyword{regression}

\name{ca.jo-class}
\docType{class}
\alias{ca.jo-class}
\encoding{latin1}
\title{Representation of class ca.jo}
\description{
  This class contains the relevant information by applying the Johansen
  procedure to a matrix of time series data.
}
\section{Slots}{
  \describe{
    \item{\code{x}:}{Object of class \code{"ANY"}: A data matrix, or an
      object that can be coerced to it.}
    \item{\code{Z0}:}{Object of class \code{"matrix"}: The matrix of the
      differenced series.}
    \item{\code{Z1}:}{Object of class \code{"matrix"}: The regressor
      matrix, except for the lagged variables in levels.}
    \item{\code{ZK}:}{Object of class \code{"matrix"}: The matrix of the
      lagged variables in levels.}
    \item{\code{type}:}{Object of class \code{"character"}: The type of the
      test, either \code{"trace"} or \code{"eigen"}.}
    \item{\code{model}:}{Object of class \code{"character"}: The model
      description in prose, with respect to the inclusion of a linear
      trend.}
    \item{\code{ecdet}:}{Object of class \code{"character"}: Specifies
      the deterministic term to be included in the cointegration
      relation. This can be either "none", "const", or "trend".}
    \item{\code{lag}:}{Object of class \code{"integer"}: The lag order
      for the variables in levels.}
    \item{\code{P}:}{Object of class \code{"integer"}: The count of
      variables.}
    \item{\code{season}:}{Object of class \code{"ANY"}: The frequency of
      the data, if seasonal dummies should be included, otherwise NULL.}
    \item{\code{dumvar}:}{Object of class \code{"ANY"}: A matrix
      containing dummy variables. The row dimension must be equal to
      \code{x}, otherwise NULL.}
    \item{\code{cval}:}{Object of class \code{"ANY"}: The critical
      values of the test at the 1\%, 5\% and 10\% level of significance.}
    \item{\code{teststat}:}{Object of class \code{"ANY"}: The values
      of the test statistics.}
    \item{\code{lambda}:}{Object of class \code{"vector"}: The eigenvalues.}
    \item{\code{Vorg}:}{Object of class \code{"matrix"}: The matrix of
      eigenvectors, such that \eqn{\hat V'S_{kk}\hat V = I}.}
    \item{\code{V}:}{Object of class \code{"matrix"}: The matrix of
      eigenvectors, normalised with respect to the first variable.}
    \item{\code{W}:}{Object of class \code{"matrix"}: The matrix of
      loading weights.}
    \item{\code{PI}:}{Object of class \code{"matrix"}: The coeffcient
      matrix of the lagged variables in levels.}
    \item{\code{DELTA}:}{Object of class \code{"matrix"}: The
      variance/covarinace matrix of \code{V}.}
    \item{\code{GAMMA}:}{Object of class \code{"matrix"}: The
      coeffecient matrix of \code{Z1}.}
    \item{\code{R0}:}{Object of class \code{"matrix"}: The matrix of
      residuals from the regressions in differences.}
    \item{\code{RK}:}{Object of class \code{"matrix"}: The matrix of
      residuals from the regression in lagged levels.}
    \item{\code{bp}:}{Object of class \code{"ANY"}: Potential break
      point, only set if function \code{cajolst} is called, otherwise
      \code{NA}.}
    \item{\code{test.name}:}{Object of class \code{"character"}: The
      name of the test, \emph{i.e.} `Johansen-Procedure'.}
    \item{\code{spec}:}{Object of class \code{"character"}: The
      specification of the VECM.}
    \item{\code{call}:}{Object of class \code{"call"}: The
      call of function \code{ca.jo}.}
  }
}
\section{Extends}{
Class \code{urca}, directly.
}
\section{Methods}{
  Type \code{showMethods(classes="ca.jo")} at the R prompt for a
  complete list of methods which are available for this class.

  Useful methods include
  \describe{
    \item{\code{show}:}{test statistic.}
    \item{\code{summary}:}{like show, but critical values, eigenvectors
      and loading matrix added.}
    \item{\code{plot}:}{The series of the VAR and their potential
      cointegration relations.}
  }
}
\references{

  Johansen, S. (1988), Statistical Analysis of Cointegration Vectors,
  \emph{Journal of Economic Dynamics and Control}, \bold{12}, 231--254.

  Johansen, S. and Juselius, K. (1990), Maximum Likelihood Estimation and
  Inference on Cointegration -- with Applications to the Demand for
  Money, \emph{Oxford Bulletin of Economics and Statistics}, \bold{52,
    2}, 169--210.

  Johansen, S. (1991), Estimation and Hypothesis Testing of
  Cointegration Vectors in Gaussian Vector Autoregressive Models,
  \emph{Econometrica}, \bold{Vol. 59, No. 6}, 1551--1580.

}
\seealso{
  \code{\link{ca.jo}}, \code{\link{plotres}} and \code{\link{urca-class}}.
}
\author{Bernhard Pfaff}
\keyword{classes}

\name{cajo.test-class}
\docType{class}
\alias{cajo.test-class}
\encoding{latin1}
\title{Representation of class cajo.test}
\description{
  This class contains the relevant information by estimating and testing
  a VAR under linear restrictions on \eqn{\bold{\alpha}} and
  \eqn{\bold{\beta}}.
}
\section{Slots}{
  \describe{
    \item{\code{Z0}:}{Object of class \code{"matrix"}: The matrix of the
      differenced series.}
    \item{\code{Z1}:}{Object of class \code{"matrix"}: The regressor
      matrix, except for the lagged variables in levels.}
    \item{\code{ZK}:}{Object of class \code{"matrix"}: The matrix of the
      lagged variables in levels.}
    \item{\code{ecdet}:}{Object of class \code{"character"}: Specifies
      the deterministic term to be included in the cointegration
      relation. This can be either "none", "const", or "trend".}
    \item{\code{H}:}{Object of class \code{"ANY"}: The matrix
      containing the restrictions placed upon \eqn{\bold{\beta}}.}
    \item{\code{A}:}{Object of class \code{"ANY"}: The matrix
      containing the restrictions placed upon \eqn{\bold{\alpha}}.}
    \item{\code{B}:}{Object of class \code{"ANY"}: The matrix
      orthogonal to matrix \eqn{\bold{A}}.}
    \item{\code{type}:}{Object of class \code{"character"}: The test type.}
    \item{\code{teststat}:}{Object of class \code{"numeric"}: The value
      of the test statistic.}
    \item{\code{pval}:}{Object of class \code{"vector"}: The p-value and
      the degrees of freedom.}
    \item{\code{lambda}:}{Object of class \code{"vector"}: The
      eigenvalues of the restricted model.}
    \item{\code{Vorg}:}{Object of class \code{"matrix"}: The matrix of
      eigenvectors, such that \eqn{\hat V_{\dots}'(H'S_{\dots}H)\hat
	V_{\dots} = I}.}
    \item{\code{V}:}{Object of class \code{"matrix"}: The matrix of the
      restricted eigenvectors, normalised with respect to the first variable.}
    \item{\code{W}:}{Object of class \code{"matrix"}: The matrix of the
      corresponding loading weights.}
    \item{\code{PI}:}{Object of class \code{"matrix"}: The coefficient
      matrix of the lagged variables in levels.}
    \item{\code{DELTA}:}{Object of class \code{"ANY"}: The
      variance/covarinace matrix of \eqn{\bold{V}}.}
    \item{\code{DELTA.bb}:}{Object of class \code{"ANY"}: The
      variance/covarinace matrix of the marginal factor
      \eqn{\bold{B}'\bold{R}_{0t}}.}
    \item{\code{DELTA.ab}:}{Object of class \code{"ANY"}: The
      variance/covarinace matrix of the conditional distribution of
      \eqn{\bold{A}'\bold{R}_{0t}} and \eqn{\bold{R}_{kt}}.}
    \item{\code{DELTA.aa.b}:}{Object of class \code{"ANY"}: The
      variance/covarinace matrix of the restricted loading matrix.}
    \item{\code{GAMMA}:}{Object of class \code{"matrix"}: The
      coefficient matrix of \eqn{\bold{Z1}}.}
    \item{\code{test.name}:}{Object of class \code{"character"}: The
      name of the test, \emph{i.e.} `Johansen-Procedure'.}
  }
}
\section{Extends}{
Class \code{urca}, directly.
}
\section{Methods}{
  Type \code{showMethods(classes="cajo.test")} at the R prompt for a
  complete list of methods which are available for this class.

  Useful methods include
  \describe{
    \item{\code{show}:}{test-statistic.}
    \item{\code{summary}:}{like show, but p-value of test statistic,
      restricted eigenvectors, loading matrix and restriction matrices
      \eqn{\bold{H}} and \eqn{\bold{A}}, where applicable, added.}
    }
}
\references{

  Johansen, S. (1988), Statistical Analysis of Cointegration Vectors,
  \emph{Journal of Economic Dynamics and Control}, \bold{12}, 231--254.

  Johansen, S. and Juselius, K. (1990), Maximum Likelihood Estimation and
  Inference on Cointegration -- with Applications to the Demand for
  Money, \emph{Oxford Bulletin of Economics and Statistics}, \bold{52,
    2}, 169--210.

  Johansen, S. (1991), Estimation and Hypothesis Testing of
  Cointegration Vectors in Gaussian Vector Autoregressive Models,
  \emph{Econometrica}, \bold{Vol. 59, No. 6}, 1551--1580.

}
\seealso{
  \code{\link{ablrtest}}, \code{\link{alrtest}}, \code{\link{blrtest}},
  \code{\link{ca.jo}}, \code{\link{ca.jo-class}} and \code{\link{urca-class}}.
}
\author{Bernhard Pfaff}
\keyword{classes}

\name{urca-class}
\docType{class}
\alias{urca-class}

\title{Class `urca'. Parent of classes in package `urca'}
\description{
  This class is the parent class of the specific classes
  designed holding the test specific information of the unit
  root/cointegration tests.
}
\section{Objects from the Class}{
Objects can be created by calls of the form \code{new("urca", ...)},
but most often the slot \code{test.name} is set by calling one of the
unit root/cointegration functions, \emph{e.g} \code{ur.za}.
}
\section{Slots}{
  \describe{
    \item{\code{test.name}:}{Object of class \code{"character"}. The
      name of the unit root/cointegration test.}
  }
}
\section{Methods}{
No methods defined with class `urca'.
}
\seealso{
  \code{\link{ur.ers-class}}, \code{\link{ur.kpss-class}},
  \code{\link{ca.jo-class}}, \code{\link{ca.po-class}},
  \code{\link{ur.pp-class}}, \code{\link{ur.sp-class}}
  and \code{\link{ur.za-class}}.
}
\author{Bernhard Pfaff}
\keyword{classes}

\name{denmark}
\alias{denmark}
\docType{data}
\encoding{latin1}
\title{Data set for Denmark, Johansen and Juselius (1990)}
\description{
  This data set contains the series used by S. Johansen and K. Juselius
  for estimating a money demand function of Denmark.
}
\usage{data(denmark)}
\format{
  A data frame with 55 observations on the following 6 variables.

    \tabular{rl}{
    \code{period} \tab Time index from 1974:Q1 until 1987:Q3. \cr 
    \code{LRM} \tab Logarithm of real money, M2. \cr
    \code{LRY} \tab Logarithm of real income. \cr
    \code{LPY} \tab Logarithm of price deflator. \cr
    \code{IBO} \tab Bond rate. \cr
    \code{IDE} \tab Bank deposit rate. \cr
    }
}
\source{
  Johansen, S. and Juselius, K. (1990), Maximum Likelihood Estimation and
  Inference on Cointegration -- with Applications to the Demand for
  Money, \emph{Oxford Bulletin of Economics and Statistics}, \bold{52,
    2}, 169--210.
  
}
\references{
  \url{http://web.math.ku.dk/~sjo/}
}
\author{Bernhard Pfaff}
\keyword{datasets}
\concept{data set Johansen Juselius Denmark}
